/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import org.jdom.Attribute;
import org.jdom.Element;

/**
 * Klasse zur Repräsentation eines XML-Konfigelements
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 *
 */
public class ElementKonfig implements Comparable<ElementKonfig> {
	/**
	 * ID des Elementes
	 */
	private int id = -1;

	/**
	 * Bezeichnung
	 */
	private String bezeichnung;

	/**
	 * Das Element selbst
	 */
	private Element xmlElement;

	/**
	 * Konstruktor, initialisiert einen {@link ElementKonfig} an Hand des JDOM-
	 * {@link Element}es
	 *
	 * @param element
	 */
	public ElementKonfig(final Element element) {
		initialisiere(element);
	}

	/**
	 * Führt die Initialisierung durch.
	 *
	 * @param initialId
	 *            ID zur Initialisierung
	 * @param initialBezeichnung
	 *            Bezeichnung zur Initialisierung
	 */
	private void initialisiere(final int initialId, final String initialBezeichnung) {
		id = initialId;
		bezeichnung = initialBezeichnung;
		xmlElement = new Element(XmlElementKonfigVerwaltung.XML_ELEMENT_KB);
		final Attribute attId = new Attribute(XmlElementKonfigVerwaltung.XML_ELEMENT_PID, Integer.toString(initialId));
		xmlElement.setAttribute(attId);
		final Attribute attBez = new Attribute(XmlElementKonfigVerwaltung.XML_ELEMENT_PID, initialBezeichnung);
		xmlElement.setAttribute(attBez);
	}

	/**
	 * Initialisiert das Element an Hand eines JDOM-{@link Element}es
	 *
	 * @param element
	 *            JDOM-{@link Element}
	 */
	private void initialisiere(final Element element) {

		xmlElement = element;

		/*
		 * Bezeichnung Holen
		 */
		bezeichnung = xmlElement.getAttributeValue(XmlElementKonfigVerwaltung.XML_ELEMENT_PID);

	}

	/**
	 * Liefert diesen {@link ElementKonfig} als JDOM-{@link Element} zurück.
	 *
	 * @return {@link ElementKonfig} als JDOM-{@link Element}
	 */
	public Element getElement() {
		initialisiere(id, bezeichnung);
		return xmlElement;
	}

	/**
	 * Liefert die ID dieses Elementes
	 *
	 * @return ID
	 */
	public int getId() {
		return id;
	}

	/**
	 * Setzt die ID dieses Elementes
	 *
	 * @param id
	 *            ID
	 */
	public void setId(final int id) {
		this.id = id;
	}

	/**
	 * Liefert die Bezeichnung dieses Elementes.
	 *
	 * @return Bezeichnung
	 */
	public String getBezeichnung() {
		return bezeichnung;
	}

	/**
	 * Setzt die Bezeichnung dieses Elementes.
	 *
	 * @param bezeichnung
	 *            Zu setzende Bezeichnung
	 */
	public void setBezeichnung(final String bezeichnung) {
		this.bezeichnung = bezeichnung;
	}

	@Override
	public String toString() {
		final String re = "Filter " + id + ": " + bezeichnung;
		return re;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result) + id;
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final ElementKonfig other = (ElementKonfig) obj;
		if (id != other.id) {
			return false;
		}
		return true;
	}

	@Override
	public int compareTo(final ElementKonfig o) {
		int re = 0;

		if (this.id < o.id) {
			re = -1;
		} else if (this.id == o.id) {
			re = 0;
		} else if (this.id > o.id) {
			re = 1;
		}

		return re;
	}

}