/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import org.eclipse.jface.action.IAction;

import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

/**
 * Interface, das von Clients implementiert werden muss, damit das KonfigAss
 * Plug-In den registrierten Teilkonfigurationsassistenten das
 * Konfig-Datenmodell übergeben kann. Es werden zusätzlich Methoden
 * bereitgestellt um Informationen an das KonfigAss Plug-In zu übergeben.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, Pittner (Dambach)
 *
 */
public interface IKonfigAssDatenModell {
	/**
	 * Holt die PluginId des Plugins welches der KonfigAss verwendet, wird vom
	 * Contributor geliefert
	 *
	 * @return Id
	 */
	String holePluginId();

	/**
	 * Holt den Namen des Plugins welches der KonfigAss verwendet, wird vom
	 * Contributor geliefert
	 *
	 * @return Name
	 */
	String holeName();

	/**
	 * Holt die Beschreibung des Plugins welches der KonfigAss verwendet, wird vom
	 * Contributor geliefert
	 *
	 * @return Beschreibung
	 */
	String holeBeschreibung();

	/**
	 * Holt die Action des Plugins welches der KonfigAss verwendet, wird vom
	 * Contributor geliefert
	 *
	 * @return Aktion
	 */
	IAction holeAktion();

	/**
	 * Übergibt den Plug-In welche den Erweiterungspunkt verwenden das
	 * Konfig-Datenmodell, wird vom Enabler geliefert
	 *
	 * @param dataModel
	 *            das Konfig-Datenmodell
	 * @param pfad
	 *            der Pfad zu Verwaltungsdatei der Konfiguration aus der das
	 *            Konfig-Datenmodell erzeugt wurde
	 */
	void setzeKonfigDatenModell(ConfigDataModel dataModel, String pfad);

	/**
	 * Informiert die Plug-In welche den Erweiterungspunkt verwenden und bereits ein
	 * Datenmodell erhalten haben darüber, dass eine Konfiguration geladen wird.
	 *
	 * @param pfad
	 *            der Pfad zu Verwaltungsdatei der Konfiguration aus der das
	 *            Konfig-Datenmodell erzeugt wird
	 */
	void ladeKonfigDatenModell(String pfad);
}
