/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

/**
 * Fabrik zur Erstellung der benötigten Bilder, damit diese nicht mehrfach
 * erstellt werden.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, Krause (Dambach)
 *
 */
public final class ImageFactory {
	/**
	 * Singleton-Instanz
	 */
	private static ImageFactory instanz;

	/*
	 * Hashmap, die die Bilder beinhaltet
	 */
	private final Map<String, Image> imageRegistry;

	/**
	 * Liefert die Singletoninstanz.
	 *
	 * @return Instanz dieser Klasse.
	 */
	public static synchronized ImageFactory getInstanz() {
		if (ImageFactory.instanz == null) {
			ImageFactory.instanz = new ImageFactory();
		}
		return ImageFactory.instanz;
	}

	/**
	 * Privater Konstruktor
	 */
	private ImageFactory() {
		imageRegistry = new HashMap<>();
		loadImages();
	}

	/**
	 * Fügt die Bilder der Registry hinzu
	 *
	 */
	private void loadImages() {
		ImageDescriptor id = KonfigurationsAssistent.getImageDescriptor("icons/c1.png");
		final Image imageDefault = id.createImage();
		imageRegistry.put("icons/c1.png", imageDefault);

		id = KonfigurationsAssistent.getImageDescriptor("icons/a1.png");
		imageRegistry.put("icons/a1.png", id.createImage());

		id = KonfigurationsAssistent.getImageDescriptor("icons/c1.png");
		imageRegistry.put("icons/c1.png", id.createImage());

	}

	/**
	 * Liefert das Bild zum gegebenen Bildpfad.
	 *
	 * @param imageFilePath
	 *            Pfad zum Bild
	 * @return das Image oder <code>null</code>
	 */
	public Image getImage(final String imageFilePath) {

		Image image = imageRegistry.get(imageFilePath);

		if (image == null) {
			setImage(imageFilePath);
			image = imageRegistry.get(imageFilePath);
		}

		return image;

	}

	/**
	 * Registrieren eines Bildes im Register, falls vorhanden, ansonsten wird ein
	 * Default-bild eingestellt.
	 *
	 * @param imageFilePath
	 *            Pfad zum Bild
	 */
	private void setImage(final String imageFilePath) {
		final ImageDescriptor id = KonfigurationsAssistent.getImageDescriptor(imageFilePath);
		Image image = null;
		if (id != null) {
			image = id.createImage();
		} else {
			image = imageRegistry.get("icons/a1.png");
		}
		imageRegistry.put(imageFilePath, image);
	}

}
