/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * @author BitCtrl Systems GmbH, Uwe Peuker, Pittner (Dambach)
 */
public class KonfigAssDarstellungAssistenten extends Composite {

	private static final Debug LOGGER = Debug.getLogger();

	private TableViewer tvZufluesse1;

	private Button button2;

	private final String secondId;

	public KonfigAssDarstellungAssistenten(final Composite parent, final int style, final String secondId) {
		super(parent, style);
		this.secondId = secondId;
		initialize();
	}

	/**
	 * Führt die Basis-Initialisierung des Composites durch
	 */
	private void initialize() {
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.heightHint = -1;
		gridData1.widthHint = -1;
		gridData1.horizontalIndent = 0;
		gridData1.verticalAlignment = GridData.FILL;
		setLayout(new GridLayout(1, false));
		this.setSize(new Point(770, 561));
		setLayoutData(gridData1);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.makeColumnsEqualWidth = false;
		gridLayout.numColumns = 1;
		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.heightHint = -1;
		gridData2.widthHint = -1;
		gridData2.horizontalIndent = 0;
		gridData2.verticalAlignment = GridData.FILL;
		final Composite composite = new Composite(this, SWT.NONE);
		composite.setLayoutData(gridData2);
		composite.setLayout(gridLayout);

		erzeugeCompositeOben(composite);
		erzeugeCompositeUnten(composite);

		// Für die F1 Taste
		PlatformUI.getWorkbench().getHelpSystem().setHelp(this, KonfigurationsAssistent.PLUGIN_ID + ".assistenten");
	}

	private void erzeugeTabelleTeilkonfigurationsassistenten(final Composite parent) {
		final FillLayout fillLayout = new FillLayout();
		fillLayout.type = SWT.HORIZONTAL;
		final Composite composite1 = new Composite(parent, SWT.None);
		composite1.setLayout(fillLayout);
		tvZufluesse1 = new TableViewer(composite1,
				SWT.CHECK | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

		final TableViewerColumn tc1 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc1.getColumn().setWidth(60);
		tc1.getColumn().setText("Auswahl");
		final TableViewerColumn tc2 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc2.getColumn().setWidth(90);
		tc2.getColumn().setText("Kurzbezeichnung");
		final TableViewerColumn tc3 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc3.getColumn().setWidth(500);
		tc3.getColumn().setText("Beschreibung");

		tvZufluesse1.setContentProvider(new TabelleTeilkonfigurationsassistentenInhaltsVersorger());
		tvZufluesse1.setLabelProvider(new TabelleTeilkonfigurationsassistentenDarstellungsVersorger());

		tvZufluesse1.getTable().setLinesVisible(true);
		tvZufluesse1.getTable().setHeaderVisible(true);

		tvZufluesse1.addDoubleClickListener(new IDoubleClickListener() {

			@Override
			public void doubleClick(final DoubleClickEvent event) {
				final TableItem[] selection = tvZufluesse1.getTable().getSelection();
				oeffneTkaAusSelektion(selection);
			}
		});

	}

	/**
	 * @param list
	 */
	public void setInputTabelle(final List<IKonfigAssDatenModell> list) {

		if ((tvZufluesse1 != null) && (list != null)) {
			// _tvZufluesse1.setInput(list);

			for (final IKonfigAssDatenModell kadm : list) {

				final TableItem[] items = tvZufluesse1.getTable().getItems();

				boolean vorhanden = false;

				for (final TableItem ti : items) {
					final String id = (String) ti.getData("id");
					if (id.equals(kadm.holePluginId())) {
						vorhanden = true;
					}

				}

				if (!vorhanden) {
					final TableItem ti = new TableItem(tvZufluesse1.getTable(), SWT.NONE);
					ti.setText(1, kadm.holeName());
					ti.setText(2, kadm.holeBeschreibung());
					ti.setData("id", kadm.holePluginId());
				}

			}

			tvZufluesse1.getTable().addListener(SWT.Selection, new Listener() {
				@Override
				public void handleEvent(final Event event) {
					updateButton();
				}
			});
		}
	}

	/**
	 * @author Pittner
	 *
	 */
	private static class TabelleTeilkonfigurationsassistentenInhaltsVersorger implements IStructuredContentProvider {

		@Override
		public Object[] getElements(final Object inputElement) {
			final List<?> list = (List<?>) inputElement;

			final Object[] re = new Object[list.size()];

			for (int i = 0; i < list.size(); i++) {
				re[i] = list.get(i);
			}

			return re;
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput, final Object newInput) {
			// übergebener Input wird nicht betrachtet
		}
	}

	/**
	 * {@link LabelProvider} für die Tabelle.
	 *
	 * @author HofmannJ
	 *
	 */
	private static class TabelleTeilkonfigurationsassistentenDarstellungsVersorger extends LabelProvider
			implements ITableLabelProvider {

		@Override
		public Image getColumnImage(final Object element, final int columnIndex) {
			return null;
		}

		@Override
		public String getColumnText(final Object element, final int columnIndex) {
			if (columnIndex == 0) {
				return (String) element;
			}
			return "default";
		}

		@Override
		public boolean isLabelProperty(final Object element, final String property) {
			return false;
		}
	}

	public TableViewer holeTableViewerTeilKonfigurationsAssistenten() {
		return tvZufluesse1;
	}

	/**
	 * This method initializes composite22
	 *
	 */
	private void erzeugeCompositeOben(final Composite parent) {
		final GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.FILL;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.grabExcessVerticalSpace = true;
		gridData4.verticalAlignment = GridData.FILL;
		final Composite composite22 = new Composite(parent, SWT.NONE);
		composite22.setLayout(new GridLayout());
		erzeugeCompositeGroupOben(composite22);
		composite22.setLayoutData(gridData4);
	}

	/**
	 * This method initializes composite25
	 *
	 */
	private void erzeugeCompositeOeffnen(final Group group) {
		final GridData gridData13 = new GridData();
		gridData13.horizontalAlignment = GridData.END;
		gridData13.grabExcessHorizontalSpace = true;
		gridData13.verticalAlignment = GridData.CENTER;
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		final Composite composite25 = new Composite(group, SWT.NONE);
		composite25.setLayout(new GridLayout());
		composite25.setLayoutData(gridData);
		button2 = new Button(composite25, SWT.NONE);
		button2.setText("Teilkonfigurationsassistenten öffnen...");
		button2.setLayoutData(gridData13);
		// button2.setEnabled(false);
		button2.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				final TableItem[] items = tvZufluesse1.getTable().getItems();
				final List<TableItem> openItems = new LinkedList<>();

				for (final TableItem item : items) {
					final TableItem ti = item;

					if (ti.getChecked() && (!oeffneExtension(ti))) {
						openItems.add(ti);
					}
				}
				behandleGeoeffneteTKA(openItems);
			}
		});
		button2.setEnabled(false);
	}

	/**
	 * This method initializes composite
	 *
	 */
	private void erzeugeCompositeGroupOben(final Composite parent) {
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		final Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayout(new GridLayout());
		erzeugeGroupOben(composite);
		composite.setLayoutData(gridData);
	}

	/**
	 * Holt die registrierten Teilkonfigurationsassistenten
	 *
	 * @return die Liste der TKA
	 */
	public List<IKonfigAssDatenModell> holeAngemeldeteAssistenten() {
		final SortedMap<String, IKonfigAssDatenModell> sortMap = new TreeMap<>();

		final IConfigurationElement[] configurationElementsFor = Platform.getExtensionRegistry()
				.getConfigurationElementsFor("de.bsvrz.buv.plugin.konfigass.konfigAssDatenModell");

		for (final IConfigurationElement ce : configurationElementsFor) {
			final IKonfigAssDatenModell operation;
			try {
				operation = (IKonfigAssDatenModell) ce.createExecutableExtension("konfigAssDatenModellKlasse");

				final String name = operation.holeName();

				if (name != null) {
					sortMap.put(name, operation);
				}
			} catch (final CoreException e) {
				KonfigAssDarstellungAssistenten.LOGGER.error(e.getLocalizedMessage(), e);
			}
		}

		return new ArrayList<>(sortMap.values());
	}

	/**
	 * This method initializes composite2
	 *
	 */
	private void erzeugeCompositeUnten(final Composite parent) {
		final GridData gridData6 = new GridData();
		gridData6.horizontalAlignment = GridData.FILL;
		gridData6.grabExcessHorizontalSpace = true;
		gridData6.heightHint = -1;
		gridData6.verticalAlignment = GridData.CENTER;
		final Composite composite2 = new Composite(parent, SWT.NONE);
		composite2.setLayout(new GridLayout());
		erzeugeCompositeGroupUnten(composite2);
		composite2.setLayoutData(gridData6);
	}

	/**
	 * This method initializes composite3
	 *
	 */
	private void erzeugeCompositeGroupUnten(final Composite parent) {
		final GridData gridData7 = new GridData();
		gridData7.horizontalAlignment = GridData.FILL;
		gridData7.grabExcessHorizontalSpace = true;
		gridData7.verticalAlignment = GridData.CENTER;
		final Composite composite3 = new Composite(parent, SWT.NONE);
		composite3.setLayout(new GridLayout());
		erzeugeGroupUnten(composite3);
		composite3.setLayoutData(gridData7);
	}

	/**
	 * This method initializes group
	 *
	 */
	private void erzeugeGroupUnten(final Composite parent) {
		final GridData gridData8 = new GridData();
		gridData8.horizontalAlignment = GridData.FILL;
		gridData8.grabExcessHorizontalSpace = true;
		gridData8.verticalAlignment = GridData.CENTER;
		final Group group = new Group(parent, SWT.NONE);
		group.setLayout(new GridLayout());
		erzeugeCompositeOeffnen(group);
		// erzeugeCompositeGroupUnten2();
		group.setLayoutData(gridData8);
	}

	/**
	 * This method initializes group1
	 *
	 */
	private void erzeugeGroupOben(final Composite parent) {
		final GridData gridData11 = new GridData();
		gridData11.horizontalAlignment = GridData.FILL;
		gridData11.grabExcessHorizontalSpace = true;
		gridData11.grabExcessVerticalSpace = true;
		gridData11.verticalAlignment = GridData.FILL;
		final Group group1 = new Group(parent, SWT.NONE);
		group1.setLayout(new GridLayout());
		group1.setText("Registrierte Teilkonfigurationsassistenten");
		final Composite composite = erzeugeCompositeTabelle(group1);
		group1.setLayoutData(gridData11);

		erzeugeTabelleTeilkonfigurationsassistenten(composite);
	}

	/**
	 * This method initializes composite4
	 *
	 */
	private Composite erzeugeCompositeTabelle(final Composite parent) {
		final GridData gridData12 = new GridData();
		gridData12.horizontalAlignment = GridData.FILL;
		gridData12.grabExcessHorizontalSpace = true;
		gridData12.grabExcessVerticalSpace = true;
		gridData12.verticalAlignment = GridData.FILL;
		final Composite composite4 = new Composite(parent, SWT.NONE);
		composite4.setLayout(new FillLayout());
		composite4.setLayoutData(gridData12);

		return composite4;
	}

	private void oeffneTkaAusSelektion(final TableItem[] selection) {
		final List<TableItem> openItems = new LinkedList<>();

		for (final TableItem ti : selection) {
			ti.setChecked(true);
			if (!oeffneExtension(ti)) {
				openItems.add(ti);
			}
		}

		behandleGeoeffneteTKA(openItems);
	}

	private boolean ermittleObTkaGeoeffnet(final String pluginId) {
		final IWorkbench wb = PlatformUI.getWorkbench();
		final IWorkbenchWindow ww = wb.getActiveWorkbenchWindow();
		final IWorkbenchPage page = ww.getActivePage();

		final IViewReference[] vrs = page.getViewReferences();

		for (final IViewReference ref : vrs) {
			final String refId = ref.getId();
			if (!"de.bsvrz.buv.plugin.tkabasis.tkaBasisView".equals(refId)) {
				continue;
			}

			if (pluginId.startsWith("de.bsvrz.buv.plugin.tka")) {
				final String secId = pluginId.substring(pluginId.indexOf("tka") + 3);
				if ((ref.getSecondaryId() != null) && ref.getSecondaryId().toLowerCase().equals(secId)) {
					return true;
				}
			}
		}

		return false;
	}

	public Button getButtonOeffnen() {
		return button2;
	}

	private boolean oeffneExtension(final TableItem ti) {
		final String id = (String) ti.getData("id");

		if (ermittleObTkaGeoeffnet(id)) {
			return false;
		}
		IKonfigAssDatenModell kadm = null;
		final IConfigurationElement[] configurationElementsFor = Platform.getExtensionRegistry()
				.getConfigurationElementsFor("de.bsvrz.buv.plugin.konfigass.konfigAssDatenModell");

		for (final IConfigurationElement ce : configurationElementsFor) {
			final IKonfigAssDatenModell operation;
			try {
				operation = (IKonfigAssDatenModell) ce.createExecutableExtension("konfigAssDatenModellKlasse");
				if (operation.holePluginId().equals(id)) {
					kadm = operation;
					break;
				}
			} catch (final CoreException e) {
				KonfigAssDarstellungAssistenten.LOGGER.error(e.getLocalizedMessage(), e);
			}
		}

		if (kadm != null) {
			final KonfigAssSicht lokalView = KonfigAssSicht.getView(secondId);

			if (!lokalView.getDarstellungKonfigurationComposite().getZugeordnetePluginIds().contains(id)) {
				// Falls Plugin noch nicht zugewiesen
				final ConfigDataModel dm = lokalView.getDarstellungKonfigurationComposite().getConfig();
				final String pfad = lokalView.getDarstellungKonfigurationComposite().getDateiPfad();
				kadm.setzeKonfigDatenModell(dm, pfad);
				lokalView.addZugeordnetePluginId(id);

				// Entferne zugeordnete PluginId aus allen andern KonfigAss
				// Views
				final IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
				final IWorkbenchPage page = dw.getActivePage();
				for (final IViewReference ref : page.getViewReferences()) {
					if (ref.getId().equals(KonfigAssSicht.ID)) {
						final String secId = ref.getSecondaryId();
						if (((secId != null) && !secId.equals(secondId)) || ((secId == null) && (secondId != null))) {
							final KonfigAssSicht kav = (KonfigAssSicht) ref.getView(false);
							if ((kav != null) && (kav.getDarstellungKonfigurationComposite().getZugeordnetePluginIds()
									.contains(id))) {
								kav.removeZugeordnetePluginId(id);
								for (final TableItem item : kav.getDarstellungAssisitentenComposite().tvZufluesse1
										.getTable().getItems()) {
									if (id.equals(item.getData("id"))) {
										item.setChecked(false);
									}
								}
							}
						}
					}
				}
			}

			// Zugeordnete Aktion ausführen
			final IAction aktion = kadm.holeAktion();

			if (aktion != null) {
				aktion.run();
			}
		}
		return true;
	}

	private void behandleGeoeffneteTKA(final List<TableItem> items) {
		if ((items != null) && !items.isEmpty()) {
			// Öffne Listdialog
			final KonfigAssListDialog listDialog = new KonfigAssListDialog(
					PlatformUI.getWorkbench().getDisplay().getActiveShell());
			listDialog.setTitle("TKA kann nur einmal geöffnet werden");
			listDialog.setMessage("Folgende TKA sind bereits geöffnet" + System.getProperty("line.separator")
					+ "evtl. im Kontext einer anderen Konfiguration:");
			listDialog.setAddCancelButton(false);
			listDialog.setContentProvider(new ArrayContentProvider());
			listDialog.setLabelProvider(new LabelProvider());
			listDialog.setHelpContext(KonfigurationsAssistent.PLUGIN_ID + ".ass_limit");

			final List<String> liste = new LinkedList<>();
			for (final TableItem ti : items) {
				liste.add(ti.getText(1));
			}
			listDialog.setInput(liste);
			listDialog.open();

			// Aktualisiere Checkboxen
			for (final TableItem ti : items) {
				if (ti.getChecked()) {
					final String id = (String) ti.getData("id");
					final KonfigAssSicht lokalView = KonfigAssSicht.getView(secondId);
					if (!lokalView.getDarstellungKonfigurationComposite().getZugeordnetePluginIds().contains(id)) {
						ti.setChecked(false);
					}
				}
			}
			updateButton();
		}
	}

	private void updateButton() {
		final TableItem[] items = tvZufluesse1.getTable().getItems();

		boolean oeffnenMoeglich = false;

		for (final TableItem ti : items) {
			if (ti.getChecked()) {
				oeffnenMoeglich = true;
			}
		}

		if (oeffnenMoeglich) {
			button2.setEnabled(true);
		} else {
			button2.setEnabled(false);
		}
	}

} // @jve:decl-index=0:visual-constraint="10,10"
