/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.UIJob;

import de.bsvrz.buv.plugin.konfigass.KonfigBereichDaten.EnumStatusKonfigBereich;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbAnlegenAssistent;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbAnlegenModell;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbAuswahlAssistent;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbAuswahlModell;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbEntfernenHinzufuegenAssistent;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbEntfernenHinzufuegenModell;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbImportAssistent;
import de.bsvrz.buv.plugin.konfigass.assistenten.KbImportModell;
import de.bsvrz.buv.plugin.konfigass.assistenten.KvAoeNeuAssistent;
import de.bsvrz.buv.plugin.konfigass.assistenten.KvAoeNeuModell;
import de.bsvrz.buv.plugin.konfigass.assistenten.TkaWizardDialog;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeBaseValueDataFactory;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeHelper;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationAuthority;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.dav.daf.main.config.management.ConfigAreaAndVersion;
import de.bsvrz.puk.config.configFile.datamodel.AreaDependencyChecker;
import de.bsvrz.puk.config.configFile.datamodel.ConfigConfigurationArea;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.puk.config.main.managementfile.ConfigurationAreaManagementInfo;
import de.bsvrz.puk.config.main.managementfile.ConfigurationManagementFile;
import de.bsvrz.puk.config.main.managementfile.ManagementFile;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Erzeugt die Darstellung die im Reiter Konfiguration angezeigt wird
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, Pittner (Dambach)
 */
public class KonfigAssDarstellungKonfiguration extends Composite implements DisposeListener {

	/** Logger für Debugausgaben. */
	private static final Debug LOGGER = Debug.getLogger();

	private static final Color COLOR2 = new Color(null, 165, 165, 165);

	private TableViewer tvZufluesse1;

	private Button buttonLadeKonfiguration;
	private Button buttonEntladeKonfiguration;
	private Button buttonNeuerKvAoe;
	private Button buttonKbNeu;
	private Button buttonKbEntfernen;
	private Button buttonKbHinzufuegen;
	private Button buttonImport;
	private Button buttonKonsistenzPruefung;
	private Button buttonUebepruefeAktivierung;
	private Button buttonAktivierung;
	private Button buttonFreigabeZurUebernahme;
	private Button buttonFreigabeZurAktivierung;
	private Button buttonFreigabeZurAktivierungOhne;
	private Button buttonExport;

	private Text text11;
	private Text text21;

	private Composite composite11;

	private ConfigDataModel config;

	private String pidAoeKonfig;

	private SortedMap<String, KonfigBereichDaten> mapKonfigBereichKapsel;

	private List<KonfigBereichDaten> listKonfigBereichKapsel;

	private String dateiPfad;

	// Liste der für diesen KA geöffneten Extensions
	private final List<String> zugeordnetePluginIds = new LinkedList<>();

	private Thread tAutoCloser;

	private IStructuredSelection globalSelection;

	private MultiStatus multiStatus;

	private ISelectionChangedListener selChangedlistener;

	// FIXME Check, warum hier ein explizites Layout aufgerufen wird
	private Composite ladeComposite;

	public KonfigAssDarstellungKonfiguration(final Composite parent, final int style) {
		super(parent, style);
		initialisiere();
		addDisposeListener(this);
	}

	/**
	 * Führt die Basis-Initialisierung des Composites durch
	 */
	private void initialisiere() {
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.heightHint = -1;
		gridData1.widthHint = -1;
		gridData1.horizontalIndent = 0;
		gridData1.verticalAlignment = GridData.FILL;
		setLayout(new GridLayout(1, false));
		this.setSize(new Point(770, 611));
		setLayoutData(gridData1);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.makeColumnsEqualWidth = false;
		gridLayout.numColumns = 1;
		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.heightHint = -1;
		gridData2.widthHint = -1;
		gridData2.horizontalIndent = 0;
		gridData2.verticalAlignment = GridData.FILL;

		final Composite panel = new Composite(this, SWT.NONE);
		panel.setLayoutData(gridData2);
		panel.setLayout(gridLayout);

		erzeugeCompositeOben(panel);
		erzeugeCompositeMitte(panel);

		// Für die F1 Taste
		PlatformUI.getWorkbench().getHelpSystem().setHelp(this, KonfigurationsAssistent.PLUGIN_ID + ".konfiguration");

		updateButtonKeinDatenModell();
	}

	public ConfigDataModel getConfig() {
		return config;
	}

	/**
	 * This method initializes composite
	 *
	 */
	private void erzeugeCompositeMitte(final Composite parent) {
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;

		final Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayout(new GridLayout(2, false));
		composite.setLayoutData(gridData);

		erzeugeGroupLinks(composite);
		erzeugeCompositeGroupRechts(composite);
	}

	/** This method initializes group11. */
	private void erzeugeGroupLinks(final Composite parent) {
		final GridLayout gridLayout2 = new GridLayout();
		gridLayout2.numColumns = 1;
		final GridData gridData8 = new GridData();
		gridData8.horizontalAlignment = GridData.FILL;
		gridData8.grabExcessHorizontalSpace = true;
		gridData8.grabExcessVerticalSpace = true;
		gridData8.verticalAlignment = GridData.FILL;
		final GridData gridData6 = new GridData();
		gridData6.grabExcessHorizontalSpace = true;
		gridData6.grabExcessVerticalSpace = true;
		gridData6.horizontalAlignment = GridData.FILL;
		gridData6.verticalAlignment = GridData.FILL;
		final Composite composite6 = new Composite(parent, SWT.NONE);
		composite6.setLayout(gridLayout2);
		composite6.setLayoutData(gridData8);

		final Group group11 = new Group(composite6, SWT.NONE);
		group11.setLayout(new GridLayout());
		group11.setText("Konfigurationsbereiche");
		group11.setLayoutData(gridData6);

		erzeugeCompositeGroupLinks(group11);
	}

	/**
	 * This method initializes group11
	 *
	 */
	private void erzeugeCompositeGroupRechts(final Composite parent) {
		final GridLayout gridLayout2 = new GridLayout();
		gridLayout2.marginWidth = 5;
		gridLayout2.horizontalSpacing = 5;
		final GridData gridData7 = new GridData();
		gridData7.widthHint = 252;
		gridData7.horizontalAlignment = SWT.FILL;
		gridData7.grabExcessVerticalSpace = true;
		gridData7.verticalAlignment = GridData.FILL;
		final Composite composite7 = new Composite(parent, SWT.NONE);
		composite7.setLayout(gridLayout2);
		composite7.setLayoutData(gridData7);

		erzeugeGroupOperationen(composite7);

		erzeugeTabelleKonfigurationsbereiche();
	}

	/**
	 * This method initializes group511
	 *
	 */
	private void erzeugeGroupOperationen(final Composite parent) {
		final GridLayout gridLayout4 = new GridLayout();
		gridLayout4.marginWidth = 5;
		final GridData gridData8 = new GridData();
		gridData8.widthHint = 241;
		gridData8.heightHint = 373;
		gridData8.horizontalAlignment = SWT.FILL;
		gridData8.grabExcessVerticalSpace = true;
		gridData8.grabExcessHorizontalSpace = true;
		gridData8.verticalAlignment = GridData.FILL;
		final Group group511 = new Group(parent, SWT.NONE);
		group511.setText("Operationen");
		group511.setLayoutData(gridData8);
		createCompositeOperationen1(group511);
		createCompositeOperationen2(group511);
		createCompositeOperationen3(group511);
		createCompositeOperationen4(group511);
		group511.setLayout(gridLayout4);
		createCompositeOperationen5(group511);
	}

	/**
	 * This method initializes composite22
	 *
	 */
	private void createCompositeOperationen1(final Composite parent) {
		final GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.FILL;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.verticalAlignment = GridData.CENTER;
		final GridData gridData17 = new GridData();
		gridData17.horizontalAlignment = SWT.FILL;
		gridData17.widthHint = 214;
		gridData17.verticalAlignment = SWT.FILL;
		final GridData gridData10 = new GridData();
		gridData10.widthHint = 215;
		gridData10.horizontalAlignment = GridData.FILL;
		gridData10.grabExcessHorizontalSpace = true;
		gridData10.grabExcessVerticalSpace = true;
		gridData10.verticalAlignment = GridData.FILL;
		final Composite composite22 = new Composite(parent, SWT.NONE);
		composite22.setLayout(new GridLayout());
		composite22.setLayoutData(gridData10);
		buttonNeuerKvAoe = new Button(composite22, SWT.NONE);
		buttonNeuerKvAoe.setText("Neuer KV/AOE...");
		buttonNeuerKvAoe.setAlignment(SWT.LEFT);
		buttonNeuerKvAoe.setLayoutData(gridData4);
		buttonNeuerKvAoe.setLayoutData(gridData17);
		buttonNeuerKvAoe.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				updateButtonJobAktiv();
				aktionNeuerKvAoe();
			}
		});
	}

	/**
	 * This method initializes composite23
	 *
	 */
	private void createCompositeOperationen2(final Composite parent) {
		final GridData gridData25 = new GridData();
		gridData25.widthHint = 204;
		gridData25.grabExcessHorizontalSpace = true;
		gridData25.verticalAlignment = GridData.CENTER;
		gridData25.horizontalAlignment = GridData.FILL;
		final GridData gridData16 = new GridData();
		gridData16.horizontalAlignment = GridData.FILL;
		gridData16.verticalAlignment = GridData.CENTER;
		final GridData gridData15 = new GridData();
		gridData15.widthHint = 203;
		gridData15.horizontalAlignment = SWT.FILL;
		gridData15.verticalAlignment = GridData.CENTER;
		final GridData gridData11 = new GridData();
		gridData11.horizontalAlignment = GridData.FILL;
		gridData11.grabExcessHorizontalSpace = true;
		gridData11.grabExcessVerticalSpace = true;
		gridData11.verticalAlignment = GridData.FILL;
		final Composite composite23 = new Composite(parent, SWT.NONE);
		composite23.setLayout(new GridLayout());
		composite23.setLayoutData(gridData11);
		buttonKbNeu = new Button(composite23, SWT.NONE);
		buttonKbNeu.setText("KB Neu...");
		buttonKbNeu.setAlignment(SWT.LEFT);
		buttonKbNeu.setLayoutData(gridData15);
		buttonKbNeu.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionKbNeu();

			}
		});
		buttonKbEntfernen = new Button(composite23, SWT.NONE);
		buttonKbEntfernen.setText("KB Entfernen...");
		buttonKbEntfernen.setAlignment(SWT.LEFT);
		buttonKbEntfernen.setLayoutData(gridData16);
		buttonKbEntfernen.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionKbEntfernen();
			}
		});
		buttonKbHinzufuegen = new Button(composite23, SWT.NONE);
		buttonKbHinzufuegen.setText("KB Hinzufügen...");
		buttonKbHinzufuegen.setAlignment(SWT.LEFT);
		buttonKbHinzufuegen.setLayoutData(gridData25);
		buttonKbHinzufuegen.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionKbHinzufuegen();
			}
		});
	}

	/**
	 * This method initializes composite24
	 *
	 */
	private void createCompositeOperationen3(final Composite parent) {
		final GridData gridData21 = new GridData();
		gridData21.horizontalAlignment = GridData.FILL;
		gridData21.grabExcessHorizontalSpace = true;
		gridData21.verticalAlignment = GridData.CENTER;
		final GridData gridData20 = new GridData();
		gridData20.horizontalAlignment = GridData.FILL;
		gridData20.grabExcessHorizontalSpace = true;
		gridData20.verticalAlignment = GridData.CENTER;
		final GridData gridData19 = new GridData();
		gridData19.horizontalAlignment = GridData.FILL;
		gridData19.grabExcessHorizontalSpace = true;
		gridData19.verticalAlignment = GridData.CENTER;
		final GridData gridData18 = new GridData();
		gridData18.widthHint = 222;
		gridData18.horizontalAlignment = GridData.FILL;
		gridData18.grabExcessHorizontalSpace = true;
		gridData18.verticalAlignment = GridData.CENTER;
		final GridData gridData12 = new GridData();
		gridData12.widthHint = 215;
		gridData12.horizontalAlignment = GridData.FILL;
		gridData12.grabExcessHorizontalSpace = true;
		gridData12.grabExcessVerticalSpace = true;
		gridData12.verticalAlignment = GridData.FILL;
		final Composite composite24 = new Composite(parent, SWT.NONE);
		composite24.setLayout(new GridLayout());
		composite24.setLayoutData(gridData12);
		buttonImport = new Button(composite24, SWT.NONE);
		buttonImport.setText("Import...");
		buttonImport.setAlignment(SWT.LEFT);
		buttonImport.setLayoutData(gridData18);
		buttonImport.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				updateButtonJobAktiv();
				aktionImport();
			}
		});
		buttonKonsistenzPruefung = new Button(composite24, SWT.NONE);
		buttonKonsistenzPruefung.setText("Konsistenzprüfung");
		buttonKonsistenzPruefung.setAlignment(SWT.LEFT);
		buttonKonsistenzPruefung.setLayoutData(gridData19);
		buttonKonsistenzPruefung.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				updateButtonJobAktiv();
				aktionKonsistenzPruefung();
			}
		});

		buttonUebepruefeAktivierung = new Button(composite24, SWT.NONE);
		buttonUebepruefeAktivierung.setText("\u00DCberpr\u00FCfe Aktivierung");
		buttonUebepruefeAktivierung.setAlignment(SWT.LEFT);
		buttonUebepruefeAktivierung.setLayoutData(gridData21);
		buttonUebepruefeAktivierung.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				updateButtonJobAktiv();
				aktionPruefungAktivierung();
			}
		});

		buttonAktivierung = new Button(composite24, SWT.NONE);
		buttonAktivierung.setText("Aktivierung...");
		buttonAktivierung.setAlignment(SWT.LEFT);
		buttonAktivierung.setLayoutData(gridData20);
		buttonAktivierung.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				updateButtonJobAktiv();
				aktionAktivierung();
			}
		});

	}

	/**
	 * This method initializes composite25
	 *
	 */
	private void createCompositeOperationen4(final Composite parent) {
		final GridData gridData23 = new GridData();
		gridData23.widthHint = 205;
		gridData23.horizontalAlignment = GridData.FILL;
		final GridData gridData22 = new GridData();
		gridData22.horizontalAlignment = GridData.FILL;
		gridData22.grabExcessHorizontalSpace = true;
		gridData22.verticalAlignment = GridData.CENTER;
		final GridData gridData21 = new GridData();
		gridData21.horizontalAlignment = GridData.FILL;
		gridData21.grabExcessHorizontalSpace = true;
		gridData21.verticalAlignment = GridData.CENTER;
		final GridData gridData13 = new GridData();
		gridData13.widthHint = 212;
		gridData13.horizontalAlignment = GridData.FILL;
		gridData13.grabExcessVerticalSpace = true;
		gridData13.verticalAlignment = GridData.FILL;
		final Composite composite25 = new Composite(parent, SWT.NONE);
		composite25.setLayout(new GridLayout());
		composite25.setLayoutData(gridData13);
		buttonFreigabeZurUebernahme = new Button(composite25, SWT.NONE);
		buttonFreigabeZurUebernahme.setText("Freigabe zur Übernahme...");
		buttonFreigabeZurUebernahme.setAlignment(SWT.LEFT);
		buttonFreigabeZurUebernahme.setLayoutData(gridData21);
		buttonFreigabeZurUebernahme.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				updateButtonJobAktiv();
				aktionFreigabeZurUebernahme();
			}
		});
		buttonFreigabeZurAktivierung = new Button(composite25, SWT.NONE);
		buttonFreigabeZurAktivierung.setText("Freigabe zur Aktivierung...");
		buttonFreigabeZurAktivierung.setAlignment(SWT.LEFT);
		buttonFreigabeZurAktivierung.setLayoutData(gridData22);
		buttonFreigabeZurAktivierung.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				updateButtonJobAktiv();
				aktionFreigabeZurAktivierung();
			}
		});
		buttonFreigabeZurAktivierungOhne = new Button(composite25, SWT.NONE);
		buttonFreigabeZurAktivierungOhne.setText("F.z.A. ohne lokale Aktivierung durch KV");
		buttonFreigabeZurAktivierungOhne.setAlignment(SWT.LEFT);
		buttonFreigabeZurAktivierungOhne.setLayoutData(gridData23);
		buttonFreigabeZurAktivierungOhne.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				updateButtonJobAktiv();
				aktionFreigabeZurAktivierungKv();
			}
		});

	}

	/**
	 * This method initializes composite26
	 *
	 */
	private void createCompositeOperationen5(final Composite parent) {
		final GridData gridData24 = new GridData();
		gridData24.grabExcessHorizontalSpace = true;
		gridData24.widthHint = 208;
		gridData24.horizontalAlignment = GridData.FILL;
		final GridData gridData14 = new GridData();
		gridData14.horizontalAlignment = SWT.FILL;
		gridData14.grabExcessVerticalSpace = true;
		gridData14.verticalAlignment = SWT.FILL;
		final Composite composite26 = new Composite(parent, SWT.NONE);
		composite26.setLayout(new GridLayout());
		composite26.setLayoutData(gridData14);
		buttonExport = new Button(composite26, SWT.NONE);
		buttonExport.setText("Export");
		buttonExport.setAlignment(SWT.LEFT);
		buttonExport.setLayoutData(gridData24);
		buttonExport.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				updateButtonJobAktiv();
				aktionExport();
			}
		});
	}

	/**
	 * This method initializes composite27
	 *
	 */
	private void erzeugeCompositeOben(final Composite parent) {
		final GridData gridData28 = new GridData();
		gridData28.horizontalAlignment = GridData.FILL;
		gridData28.grabExcessHorizontalSpace = true;
		gridData28.verticalAlignment = GridData.CENTER;
		final Composite composite27 = new Composite(parent, SWT.NONE);
		composite27.setLayout(new GridLayout());
		erzeugeCompositeGroupKonfiguration(composite27);
		composite27.setLayoutData(gridData28);
	}

	/**
	 * This method initializes composite28
	 *
	 * @param parent
	 *
	 */
	private void erzeugeCompositeGroupKonfiguration(final Composite parent) {
		final GridData gridData29 = new GridData();
		gridData29.horizontalAlignment = GridData.FILL;
		gridData29.grabExcessHorizontalSpace = true;
		gridData29.verticalAlignment = GridData.CENTER;
		final Composite composite28 = new Composite(parent, SWT.NONE);
		composite28.setLayout(new GridLayout());
		erzeugeGroupKonfiguration(composite28);
		composite28.setLayoutData(gridData29);
	}

	/**
	 * This method initializes group01
	 *
	 * @param parent
	 *
	 */
	private void erzeugeGroupKonfiguration(final Composite parent) {
		final GridData gridData30 = new GridData();
		gridData30.horizontalAlignment = GridData.FILL;
		gridData30.grabExcessHorizontalSpace = true;
		gridData30.verticalAlignment = GridData.CENTER;
		final GridData gridData51 = new GridData();
		gridData51.horizontalAlignment = GridData.FILL;
		gridData51.grabExcessHorizontalSpace = true;
		gridData51.verticalAlignment = GridData.CENTER;
		final GridData gridData41 = new GridData();
		gridData41.horizontalAlignment = GridData.FILL;
		gridData41.grabExcessHorizontalSpace = true;
		gridData41.verticalAlignment = GridData.CENTER;
		final GridLayout gridLayout11 = new GridLayout();
		gridLayout11.numColumns = 4;
		gridLayout11.makeColumnsEqualWidth = false;
		gridLayout11.marginHeight = 5;
		gridLayout11.marginWidth = 5;
		gridLayout11.verticalSpacing = 5;
		gridLayout11.horizontalSpacing = 5;
		final Group group01 = new Group(parent, SWT.NONE);
		group01.setText("Konfiguration");
		group01.setLayoutData(gridData30);
		group01.setLayout(gridLayout11);
		final Label label11 = new Label(group01, SWT.NONE);
		label11.setText("Konfiguration (Verwaltungsdatei)");
		text11 = new Text(group01, SWT.BORDER);
		text11.setLayoutData(gridData41);
		buttonLadeKonfiguration = new Button(group01, SWT.NONE);
		buttonLadeKonfiguration.setText("...");
		buttonLadeKonfiguration.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {

				ladeComposite.layout(true);

				final FileDialog dialog = new FileDialog(Display.getCurrent().getActiveShell(),
						SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM | SWT.RESIZE | SWT.OPEN);
				dialog.setFilterNames(new String[] { "XML-Dateien" });
				dialog.setFilterExtensions(new String[] { "*.xml" });

				if (dateiPfad != null) {
					File temp = new File(dateiPfad);
					String path = temp.getParent();
					if (path != null) {
						temp = new File(path + "/../" + "config");
						if (temp.exists()) {
							path += "/../" + "config";
						} else {
							temp = new File(path + "/../" + "konfiguration");
							if (temp.exists()) {
								path += "/../" + "konfiguration";
							}
						}
						dialog.setFilterPath(path);
					}
				}

				final String s = dialog.open();

				if (s != null) {
					dateiPfad = s.replace("\\", "/");
					updateButtonJobAktiv();
					aktionLadeDatenModell(true, null);
				} else {
					if (config != null) {
						updateButtonKeineSelektion();
					} else {
						updateButtonKeinDatenModell();
					}
				}

			}
		});
		final GridData gridData = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gridData.widthHint = 24;
		buttonEntladeKonfiguration = new Button(group01, SWT.NONE);
		buttonEntladeKonfiguration.setLayoutData(gridData);
		buttonEntladeKonfiguration.setText("X");
		buttonEntladeKonfiguration.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionEntladeDatenModell();
			}
		});

		final Label label31 = new Label(group01, SWT.NONE);
		label31.setText("Konfigurationsverantwortlicher");
		text21 = new Text(group01, SWT.BORDER);
		text21.setLayoutData(gridData51);
		text21.setEditable(false);
	}

	public List<KonfigBereichDaten> getListKonfigBereichKapsel() {
		return listKonfigBereichKapsel;
	}

	/**
	 * Erzeugt das Offline-Datenmodell der Konfiguration aus der
	 * XML-Verwaltungsdatei
	 *
	 * @param configurationFile die Verwaltungsdatei
	 * @return das Offline-Datenmodell
	 */
	private ConfigDataModel erzeugeKonfigDatenModellAusDatei(final File configurationFile) {
		try {
			final ConfigDataModel configuration = new ConfigDataModel(configurationFile);

			final List<ConfigAreaAndVersion> configAreasAndVersions = new ArrayList<>();
			for (final ConfigurationArea configurationArea : configuration.getAllConfigurationAreas().values()) {
				configAreasAndVersions
						.add(new ConfigAreaAndVersion(configurationArea, configurationArea.getActivatableVersion()));
			}

			KonfigAssistentErgebnis ergebnis = new KonfigAssistentErgebnis();
			ergebnis.setAreaDependencyCheckResult(new AreaDependencyChecker().checkAreas(configAreasAndVersions));
			ergebnis.ausgebenInMultiStatus(multiStatus);

			final Runnable autoCloser = new Runnable() {
				@Override
				public void run() {
					configuration.close();
				}
			};

			if (tAutoCloser != null) {
				Runtime.getRuntime().removeShutdownHook(tAutoCloser);
			}

			tAutoCloser = new Thread(autoCloser);

			Runtime.getRuntime().addShutdownHook(tAutoCloser);

			return configuration;
		} catch (final RuntimeException e) {
			if (tAutoCloser != null) {
				Runtime.getRuntime().removeShutdownHook(tAutoCloser);
				tAutoCloser = null;
			}

			multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID, "Fehler beim Laden des Datenmodells", e));
		}

		return null;

	}

	private void erzeugeTabelleKonfigurationsbereiche() {
		final GridData gridData111 = new GridData();
		gridData111.grabExcessVerticalSpace = true;
		gridData111.heightHint = 321;
		gridData111.horizontalAlignment = GridData.FILL;
		gridData111.grabExcessHorizontalSpace = true;
		gridData111.verticalAlignment = GridData.FILL;
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		final GridLayout gridLayout = new GridLayout();
		ladeComposite = new Composite(composite11, SWT.None);
		ladeComposite.setLayout(gridLayout);
		ladeComposite.setLayoutData(gridData);

		tvZufluesse1 = new TableViewer(ladeComposite, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		tvZufluesse1.setContentProvider(new TabelleKonfigurationsbereicheInhaltsVersorger());

		final TableViewerColumn tc1 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc1.getColumn().setText("Modelle");
		tc1.getColumn().setToolTipText("Modelldefinitionen vorhanden");

		final TableViewerColumn tc2 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc2.getColumn().setText("Objekte");
		tc2.getColumn().setToolTipText("Objektdefinitionen vorhanden");

		final TableViewerColumn tc3 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc3.getColumn().setText("KV");
		tc3.getColumn().setToolTipText("Konfigurationsverantworlicher");

		final TableViewerColumn tc4 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc4.getColumn().setText("KB");
		tc4.getColumn().setToolTipText("Konfigurationsbereich");

		final TableViewerColumn tc5 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc5.getColumn().setText("Zeitp. aktiv. Vers.");
		tc5.getColumn().setToolTipText("Zeitpunkt aktivierte Version");

		final TableViewerColumn tc6 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc6.getColumn().setText("Aktivierbare Vers.");
		tc6.getColumn().setToolTipText("Aktivierbare Version");

		final TableViewerColumn tc7 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc7.getColumn().setText("Aktuelle Vers.");
		tc7.getColumn().setToolTipText("Aktuelle Version");

		final TableViewerColumn tc8 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc8.getColumn().setText("Vers. F.z.Übern.");
		tc8.getColumn().setToolTipText("Version der Freigabe zur Übernahme");

		final TableViewerColumn tc9 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc9.getColumn().setText("Vers. F.z.Aktiv.");
		tc9.getColumn().setToolTipText("Version der Freigabe zur Aktivierung");

		final TableViewerColumn tc10 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc10.getColumn().setText("Vers. F.z.Akt.o.l.A");
		tc10.getColumn().setToolTipText(
				"Version der Freigabe zur Aktivierung ohne lokale Aktivierung durch den Konfigurationsverantwortlichen");

		final TableViewerColumn tc12 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc12.getColumn().setText("Pfad");
		tc12.getColumn().setToolTipText("Pfadangabe zur *.config-Datei");

		tvZufluesse1.getTable().setLinesVisible(true);
		tvZufluesse1.getTable().setLayoutData(gridData111);
		tvZufluesse1.getTable().setHeaderVisible(true);

		selChangedlistener = new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {

				final IStructuredSelection selection = (IStructuredSelection) tvZufluesse1.getSelection();

				if ((selection != null) && !selection.isEmpty()) {
					globalSelection = selection;
					final Object obj = globalSelection.getFirstElement();
					if (obj instanceof KonfigBereichDaten) {
						final Iterator<?> it = globalSelection.iterator();
						boolean selektionMitRechte = true;

						while (it.hasNext()) {
							final KonfigBereichDaten kbd = (KonfigBereichDaten) it.next();
							if (!kbd.getBearbeitung()) {
								selektionMitRechte = false;
							}
						}
						updateButtonSelektion(selektionMitRechte);
					} else {
						updateButtonKeineSelektion();
					}
				}

			}
		};

		tvZufluesse1.addSelectionChangedListener(selChangedlistener);

		final TableColumn[] columns = tvZufluesse1.getTable().getColumns();
		for (final TableColumn tc : columns) {
			tc.pack();
		}
	}

	/**
	 * * {@link LabelProvider} für die Tabelle.
	 *
	 * @author Pittner
	 *
	 */
	private static class TabelleKonfigurationsbereicheInhaltsVersorger implements IStructuredContentProvider {

		@Override
		public Object[] getElements(final Object inputElement) {

			final List<Object> result = new ArrayList<>();

			if (inputElement instanceof List<?>) {
				result.addAll((Collection<?>) inputElement);
			}

			return result.toArray();
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput, final Object newInput) {
			// der übergebene Input wird hier nicht betrachtet
		}
	}

	/**
	 * This method initializes composite11
	 *
	 */
	private void erzeugeCompositeGroupLinks(final Composite parent) {
		final GridData gridData9 = new GridData();
		gridData9.heightHint = 355;
		gridData9.horizontalAlignment = GridData.FILL;
		gridData9.grabExcessHorizontalSpace = true;
		gridData9.grabExcessVerticalSpace = true;
		gridData9.verticalAlignment = GridData.FILL;
		composite11 = new Composite(parent, SWT.NONE);
		composite11.setLayout(new GridLayout());
		composite11.setLayoutData(gridData9);
	}

	/**
	 * Setzt das Konfig-Datenmodell für alle registrierten
	 * Teilkonfigurationsassistenten
	 *
	 * @param dataModel das Datenmodell
	 * @param pfad      der Pfad zur Verwaltungsdatei, wird bei den
	 *                  Teilkonfigurationsassistenten angezeigt
	 */
	public void setzeKonfigDatenModellFuerTka(final ConfigDataModel dataModel, final String pfad) {
		final IExtensionRegistry er = Platform.getExtensionRegistry();

		final IConfigurationElement[] configurationElementsFor = er
				.getExtensionPoint("de.bsvrz.buv.plugin.konfigass.konfigAssDatenModell").getConfigurationElements();

		for (final IConfigurationElement ce : configurationElementsFor) {
			try {
				final IKonfigAssDatenModell operation = (IKonfigAssDatenModell) ce
						.createExecutableExtension("konfigAssDatenModellKlasse");

				final String s = operation.holePluginId();

				if (zugeordnetePluginIds.contains(s)) {
					if (dataModel != null) {
						operation.setzeKonfigDatenModell(dataModel, pfad);
					} else {
						operation.setzeKonfigDatenModell(null, null);
					}
				}
			} catch (final CoreException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Setzt das Konfig-Datenmodell für den Teilkonfigurationsassistenten mit der
	 * übergebenen Id
	 *
	 * @param pluginId  Die Id des TKA-Plugins, für das das Datenmodel gesetzt
	 *                  werden soll
	 *
	 * @param dataModel das Datenmodell
	 * @param pfad      der Pfad zur Verwaltungsdatei, wird bei den
	 *                  Teilkonfigurationsassistenten angezeigt
	 */
	public void setzeKonfigDatenModellFuerTka(final String pluginId, final ConfigDataModel dataModel,
			final String pfad) {
		final IExtensionRegistry er = Platform.getExtensionRegistry();

		final IConfigurationElement[] configurationElementsFor = er
				.getExtensionPoint("de.bsvrz.buv.plugin.konfigass.konfigAssDatenModell").getConfigurationElements();

		for (final IConfigurationElement ce : configurationElementsFor) {
			try {
				final IKonfigAssDatenModell operation = (IKonfigAssDatenModell) ce
						.createExecutableExtension("konfigAssDatenModellKlasse");

				final String s = operation.holePluginId();

				if (s.equals(pluginId)) {
					if (dataModel != null) {
						operation.setzeKonfigDatenModell(dataModel, pfad);
					} else {
						operation.setzeKonfigDatenModell(null, null);
					}

					break;
				}
			} catch (final CoreException e) {
				KonfigAssDarstellungKonfiguration.LOGGER.error(e.getLocalizedMessage(), e);
			}
		}
	}

	/**
	 * Setzt das Konfig-Datenmodell für alle registrierten
	 * Teilkonfigurationsassistenten
	 *
	 * @param pfad der Pfad zur Verwaltungsdatei, wird bei den
	 *             Teilkonfigurationsassistenten angezeigt
	 */
	private void informiereUeberLadenDatenmodell(final String pfad) {
		final IExtensionRegistry er = Platform.getExtensionRegistry();

		final IConfigurationElement[] configurationElementsFor = er
				.getExtensionPoint("de.bsvrz.buv.plugin.konfigass.konfigAssDatenModell").getConfigurationElements();

		for (final IConfigurationElement ce : configurationElementsFor) {
			try {
				final IKonfigAssDatenModell operation = (IKonfigAssDatenModell) ce
						.createExecutableExtension("konfigAssDatenModellKlasse");

				Display.getDefault().asyncExec(new Runnable() {

					@Override
					public void run() {
						operation.ladeKonfigDatenModell(dateiPfad);
					}
				});

			} catch (final CoreException e) {
				KonfigAssDarstellungKonfiguration.LOGGER.error(e.getLocalizedMessage(), e);
			}
		}
	}

	private void aktuelleInfoKbAnzeigen() {
		pidAoeKonfig = config.getConfigurationAuthorityPid();

		mapKonfigBereichKapsel = new TreeMap<>();

		final List<ConfigurationAreaManagementInfo> liste = config.getManagementFile()
				.getAllConfigurationAreaManagementInfos();

		for (int i = 0; i < liste.size(); i++) {
			final ConfigurationAreaManagementInfo cami = liste.get(i);

			final String pid = cami.getPid();

			final ConfigurationArea ca = config.getAllConfigurationAreas().get(pid);

			if (ca != null) {
				final String name = ca.getName();
				ConfigurationAuthority kv = null;
				try {
					kv = ca.getConfigurationAuthority();
					if (kv != null) {
						final String pidAoeKb = kv.getPid();

						boolean bearbeiten = false;
						if (pidAoeKonfig.equals(pidAoeKb)) {
							bearbeiten = true;
						}
						final long z2 = ca.getTimeOfLastActiveConfigurationChange();

						final short v2 = ca.getModifiableVersion();
						final short v3 = ca.getActiveVersion();
						final short v4 = ca.getTransferableVersion();
						final short v6 = ca.getActivatableVersion();
						final short v7 = ca.getActivatableVersion();

						final String pfad = cami.getDirectory().getAbsolutePath();

						final KonfigBereichBestimmeModellObjekt kbBestimmeModellObjekt = new KonfigBereichBestimmeModellObjekt(
								config, ca);

						final Boolean isModell = kbBestimmeModellObjekt.getIsModell();
						final Boolean isObjekt = kbBestimmeModellObjekt.getIsObjekt();

						final KonfigBereichDaten kbk = new KonfigBereichDaten(pid, name, bearbeiten, isModell, isObjekt,
								pidAoeKb, z2, v2, v3, v4, v6, v7, EnumStatusKonfigBereich.IMPORT, pfad);

						final String key = kbk.getPidAOE() + kbk.getPid();

						mapKonfigBereichKapsel.put(key, kbk);
					}
				} catch (final IllegalStateException e) {
					KonfigAssDarstellungKonfiguration.LOGGER.error(e.getLocalizedMessage(), e);
				}
			}
			final Collection<KonfigBereichDaten> col = mapKonfigBereichKapsel.values();

			listKonfigBereichKapsel = new ArrayList<>();
			for (final KonfigBereichDaten kbk : col) {
				listKonfigBereichKapsel.add(kbk);
			}
		}
	}

	@Override
	public void widgetDisposed(final DisposeEvent e) {
		if (config != null) {
			config.close();
			config = null;
		}
	}

	private void erzeugeInhaltTabelle(final List<KonfigBereichDaten> liste) {
		// System.out.println("KonfigAssDarstellungKonfiguration.erzeugeInhaltTabelle()");

		tvZufluesse1.setInput(new ArrayList<KonfigBereichDaten>());
		final FontRegistry registry = new FontRegistry();
		final Font fontBold = registry.getBold(Display.getCurrent().getSystemFont().getFontData()[0].getName());

		final List<KonfigBereichDaten> listSel = new ArrayList<>();

		if (globalSelection != null) {
			final Object obj = globalSelection.getFirstElement();
			if (obj instanceof KonfigBereichDaten) {
				final Iterator<?> it = globalSelection.iterator();

				while (it.hasNext()) {
					final KonfigBereichDaten kbd = (KonfigBereichDaten) it.next();
					listSel.add(kbd);
				}
			}
		}

		final List<TableItem> listItem = new ArrayList<>();

		for (final KonfigBereichDaten kapsel : liste) {
			final TableItem ti = new TableItem(tvZufluesse1.getTable(), SWT.NONE);

			final Image a1 = ImageFactory.getInstanz().getImage("icons/a1.png");
			final Image c1 = ImageFactory.getInstanz().getImage("icons/c1.png");

			if (kapsel.getModelle()) {
				ti.setImage(0, a1);
			} else {
				ti.setImage(0, c1);
			}

			if (kapsel.getObjekte()) {
				ti.setImage(1, a1);
			} else {
				ti.setImage(1, c1);
			}

			ti.setText(2, kapsel.getPidAOE());
			ti.setText(3, kapsel.getPid());
			final Date d = new Date(kapsel.getDatumAktivierung());
			final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyy HH:mm:ss");
			ti.setText(4, sdf.format(d));
			ti.setText(5, kapsel.getVersionAktivierung().toString());
			ti.setText(6, kapsel.getVersionAktuell().toString());
			ti.setText(7, kapsel.getVersionFreigabeZurUebernahme().toString());
			ti.setText(8, kapsel.getVersionFreigabeZurAktivierung().toString());
			ti.setText(9, kapsel.getVersionFreigabeZurAktivierungOhne().toString());
			// ti.setText(10,
			// kapsel.getStatusKonfigBereich().toString().toString());
			ti.setText(10, kapsel.getPfad());

			ti.setData(kapsel);

			if ((kapsel.getAktvieren() != null) && kapsel.getAktvieren()) {
				final Color color = Display.getDefault().getSystemColor(SWT.COLOR_RED);
				ti.setBackground(color);
			} else {
				final Color color = Display.getDefault().getSystemColor(SWT.COLOR_WHITE);
				ti.setBackground(color);
			}

			if ((kapsel.getBearbeitung() != null) && !kapsel.getBearbeitung()) {
				// Color color =
				// Display.getDefault().getSystemColor(SWT.COLOR_GRAY);
				ti.setForeground(KonfigAssDarstellungKonfiguration.COLOR2);
			} else {
				ti.setFont(fontBold);
			}

			if (listSel.contains(kapsel)) {
				// System.out.println("gefunden: " + kapsel.getPid());
				listItem.add(ti);
			}

		}
		if (listItem.size() > 0) {
			final TableItem[] items = listItem.toArray(new TableItem[listItem.size()]);
			// System.out.println("items: " + items.length);
			tvZufluesse1.getTable().setSelection(items);

			selChangedlistener.selectionChanged(new SelectionChangedEvent(tvZufluesse1, tvZufluesse1.getSelection()));

		} else {
			updateButtonKeineSelektion();
		}
	}

	private void aktionLadeDatenModell(final boolean benutzerInfo, final List<String> list) {
		if (dateiPfad != null) {

			final File file = new File(dateiPfad);

			if (config != null) {
				if (benutzerInfo) {
					final String meldung = KonfigurationsAssistent.PLUGIN_ID + " Zugriff auf das bisherige Datenmodell "
							+ "ist nicht mehr möglich, da ein neues Datenmodell geladen wird!";

					final IWorkbench wb = PlatformUI.getWorkbench();
					if (wb != null) {
						final Display display = wb.getDisplay();

						if (display != null) {
							final Shell shell = display.getActiveShell();
							if (shell != null) {
								final MessageBox msg = new MessageBox(shell, SWT.OK | SWT.CANCEL | SWT.ICON_WARNING);
								msg.setText("Neues Datenmodell wird geladen");
								msg.setMessage(meldung);
								final int open = msg.open();

								if (open != SWT.OK) {
									updateButtonKeineSelektion();
									return;
								}
							}
						}
					}
				}
				informiereUeberLadenDatenmodell("Test");
				tAutoCloser.start();
			}

			multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion Laden Datenmodell", null);

			final Job job = new Job("KonfigAss: Lade Datenmodell") {

				@Override
				protected IStatus run(final IProgressMonitor monitor) {
					monitor.beginTask("Start mit Laden...", 5);
					monitor.worked(1);

					try {
						if (tAutoCloser != null) {
							tAutoCloser.join();
						}
						MeldungenSpeicher.getInstanz().setKonfiguration(dateiPfad);
						config = erzeugeKonfigDatenModellAusDatei(file);
						monitor.worked(1);
					} catch (final Exception e) {
						MeldungenSpeicher.getInstanz().setKonfiguration("");
						multiStatus.merge(
								new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID, "Fehler Aktion Laden Datenmodell", e));

						return Status.CANCEL_STATUS;
					} finally {
						monitor.worked(3);
						monitor.done();
					}

					if (config == null) {
						return Status.CANCEL_STATUS;
					}
					aktuelleInfoKbAnzeigen();

					boolean fehler = false;

					if (list != null) {
						for (final String pidKb : list) {
							final ConfigurationArea ca1 = config.getAllConfigurationAreas().get(pidKb);

							if (ca1 != null) {
								final AttributeGroup atg = config
										.getAttributeGroup("atg.konfigurationsBereichAbhängigkeiten");
								final ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(config);
								final Data data = ((ConfigSystemObject) ca1).getConfigurationData(atg, lookup);
								if (data != null) {
									Data item = data.getItem("KonfigurationsAbhängigkeiten");
									final Array array = item.asArray();
									for (int i = 0; i < array.getLength(); i++) {
										item = array.getItem(i);

										final String pid = item.getItem("BereichNotwendig").asTextValue()
												.getValueText();
										if (pid != null) {
											final ConfigurationArea kb = config.getAllConfigurationAreas().get(pid);
											if (kb != null) {
												final String text2 = item.getItem("Kennung").asTextValue()
														.getValueText();

												if ("notwendig".equals(text2)) {
													final Map<String, ConfigurationArea> map = config
															.getAllConfigurationAreas();
													if (!map.containsKey(kb.getPid())) {
														final String meldung = "Fehler bei den Abhängigkeiten des KB: "
																+ kb.getPid() + " ist nicht vorhanden in: " + pidKb;
														multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
																meldung));
														fehler = true;
													}
												}
											} else {
												final String meldung = "Fehler bei den Abhängigkeiten des KB: " + pid
														+ " es ist kein SystemObject für den KB vorhanden!";
												multiStatus
														.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID, meldung));
												fehler = true;
											}
										}
									}
								}
							}
							// else{
							// String meldung =
							// "Fehler bei den Abhängigkeiten des KB: " +
							// pidKb +
							// " es ist kein SystemObject für den KB
							// vorhanden!";
							// _status.merge(new Status(Status.ERROR,
							// Activator.PLUGIN_ID, meldung));
							// fehler = true;
							// }
						}
					}

					if (!fehler) {
						return Status.OK_STATUS;
					}
					return Status.CANCEL_STATUS;

				}
			};

			job.addJobChangeListener(new JobChangeAdapter() {

				@Override
				public void done(final IJobChangeEvent event) {
					if (event.getResult().isOK()) {
						multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
								"Aktion Laden Datenmodell beendet, bitte beachten Sie die Ausgaben der Konfiguration"));

						final UIJob uiJob = new UIJob("aktualisiere GUI") {

							@Override
							public IStatus runInUIThread(final IProgressMonitor monitor) {
								try {
									aktuelleInfoKbAnzeigen();
									erzeugeInhaltTabelle(listKonfigBereichKapsel);
									updateTextKonfiguration(dateiPfad, pidAoeKonfig);
									setzeKonfigDatenModellFuerTka(config, dateiPfad);
								} catch (final Exception e) {
									multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
											"Fehler beim aktualisieren des GUI, es sind Ausnahmen aufgetreten. ", e));

									if (config != null) {
										config.close();
										config = null;
									}
									erzeugeInhaltTabelle(new ArrayList<KonfigBereichDaten>());
									updateTextKonfiguration("", "");
									updateButtonKeinDatenModell();
									// setzeKonfigDatenModellFuerTka(null, "");
									setzeKonfigDatenModellFuerTka(null, dateiPfad);

									if (list != null) {
										final XmlElementKonfigVerwaltung xmlElementKonfigVerwaltung = new XmlElementKonfigVerwaltung();
										xmlElementKonfigVerwaltung.einlesenXmlAusDatei(dateiPfad);

										final List<ElementKonfig> listeKbXml = xmlElementKonfigVerwaltung
												.getElementKonfigBereich();

										final List<ElementKonfig> delList = new ArrayList<>();

										if (listeKbXml != null) {
											for (final ElementKonfig ek : listeKbXml) {
												if (ek != null) {
													final String b = ek.getBezeichnung();
													for (final String s : list) {
														if (s.equals(b)) {
															delList.add(ek);
														}
													}
												}
											}

										}
										if (delList.size() > 0) {
											for (final ElementKonfig ek2 : delList) {
												final String meldung = "Entferne KB wegen Fehler aus den Veraltungdaten: "
														+ ek2.getBezeichnung();
												multiStatus
														.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID, meldung));
												xmlElementKonfigVerwaltung.entferneElementKonfig(ek2);
											}

											final String s = xmlElementKonfigVerwaltung.ausgabeXmlFormatiert();
											xmlElementKonfigVerwaltung.schreibeXMLStringInDatei(dateiPfad, s, true);
										}
									}

									return Status.CANCEL_STATUS;
								}
								return Status.OK_STATUS;
							}
						};
						uiJob.setUser(false);
						uiJob.setSystem(true);
						uiJob.schedule();

					} else
					// Aufräumen
					{
						multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
								"Aktion Laden Datenmodell abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. Die evt. geladene Konfiguration wurde geschlossen. "));

						final UIJob uiJob = new UIJob("aktualisiere GUI") {

							@Override
							public IStatus runInUIThread(final IProgressMonitor monitor) {
								try {
									if (config != null) {
										config.close();
										config = null;
									}
									erzeugeInhaltTabelle(new ArrayList<KonfigBereichDaten>());
									updateTextKonfiguration("", "");
									updateButtonKeinDatenModell();
									setzeKonfigDatenModellFuerTka(null, dateiPfad);

									if (list != null) {
										final XmlElementKonfigVerwaltung xmlElementKonfigVerwaltung = new XmlElementKonfigVerwaltung();
										xmlElementKonfigVerwaltung.einlesenXmlAusDatei(dateiPfad);

										final List<ElementKonfig> listeKbXml = xmlElementKonfigVerwaltung
												.getElementKonfigBereich();

										final List<ElementKonfig> delList = new ArrayList<>();

										if (listeKbXml != null) {
											for (final ElementKonfig ek : listeKbXml) {
												if (ek != null) {
													final String b = ek.getBezeichnung();
													for (final String s : list) {
														if (s.equals(b)) {
															delList.add(ek);
														}
													}
												}
											}

										}
										if (delList.size() > 0) {
											for (final ElementKonfig ek2 : delList) {
												final String meldung = "Entferne KB aus den Veraltungdaten: "
														+ ek2.getBezeichnung();
												multiStatus
														.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID, meldung));
												xmlElementKonfigVerwaltung.entferneElementKonfig(ek2);
											}

											final String s = xmlElementKonfigVerwaltung.ausgabeXmlFormatiert();
											xmlElementKonfigVerwaltung.schreibeXMLStringInDatei(dateiPfad, s, true);
										}
									}

								} catch (final Exception e) {
									multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
											"Fehler beim aktualisieren des GUI, es sind Ausnahmen aufgetreten. ", e));
									return Status.CANCEL_STATUS;
								}
								return Status.OK_STATUS;
							}
						};
						uiJob.setUser(false);
						uiJob.setSystem(true);
						uiJob.schedule();
					}
					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
				}
			});
			job.setSystem(false);
			job.setUser(true);
			job.schedule();
		}
	}

	/**
	 * Speichert die Info als konfigurierenden Datensatz an einem Objekt.
	 *
	 * @param systemObject Objekt, an dem die Info gespeichert werden soll
	 * @param model        das verwendete Modell
	 *
	 * @throws ConfigurationChangeException Falls der konfigurierende Datensatz
	 *                                      nicht am Objekt gespeichert werden
	 *                                      konnte.
	 */
	private void setInfo(final SystemObject systemObject, final KvAoeNeuModell model)
			throws ConfigurationChangeException {
		// wenn Info nicht definiert ist, braucht es auch nicht abgespeichert zu
		// werden
		if (model != null) {
			// wenn der Info-Datensatz bereits existiert, kann dieser geändert
			// werden
			final AttributeGroup atg = config.getAttributeGroup("atg.info");
			final Data data = AttributeBaseValueDataFactory.createAdapter(atg,
					AttributeHelper.getAttributesValues(atg));
			data.getTextValue("kurzinfo").setText(model.getKurzInfo());
			data.getTextValue("beschreibung").setText(model.getBeschreibung());
			systemObject.setConfigurationData(atg, data);
		}
	}

	/**
	 * Speichert die Info als konfigurierenden Datensatz an einem Objekt.
	 *
	 * @param systemObject Objekt, an dem die Info gespeichert werden soll
	 * @param model        das verwendete Modell
	 *
	 * @throws ConfigurationChangeException Falls der konfigurierende Datensatz
	 *                                      nicht am Objekt gespeichert werden
	 *                                      konnte.
	 */
	private void setInfoDef(final SystemObject systemObject, final KvAoeNeuModell model)
			throws ConfigurationChangeException {
		// wenn Info nicht definiert ist, braucht es auch nicht abgespeichert zu
		// werden
		if (model != null) {
			// wenn der Info-Datensatz bereits existiert, kann dieser geändert
			// werden
			final AttributeGroup atg = config.getAttributeGroup("atg.info");
			final Data data = AttributeBaseValueDataFactory.createAdapter(atg,
					AttributeHelper.getAttributesValues(atg));
			data.getTextValue("kurzinfo").setText("");
			data.getTextValue("beschreibung").setText(model.getBeschreibungDef());
			systemObject.setConfigurationData(atg, data);
		}
	}

	private void aktionKonsistenzPruefung() {
		multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion Konsistenzprüfung", null);

		final Job job = new Job("KonfigAss: Pruefe Konsistenz") {

			@Override
			protected IStatus run(final IProgressMonitor monitor) {
				monitor.beginTask("Starte Konsistenzpruefung...", 5);
				monitor.worked(1);

				try {

					final KonfigApplikation app = new KonfigApplikation(config);
					KonfigAssistentErgebnis ergebnis = app.pruefeKonsistenz(false, false);

					ergebnis.ausgebenInMultiStatus(multiStatus);

					monitor.worked(1);
				} catch (final Exception e2) {
					multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
							"Aktion Konsistenzprüfung abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. "
									+ e2));
					return Status.CANCEL_STATUS;
				} finally {
					monitor.worked(3);
					monitor.done();
				}

				return Status.OK_STATUS;
			}
		};

		job.addJobChangeListener(new JobChangeAdapter() {

			@Override
			public void running(final IJobChangeEvent event) {
				final UIJob uiJob = new UIJob("aktualisiere GUI") {

					@Override
					public IStatus runInUIThread(final IProgressMonitor monitor) {
						return Status.OK_STATUS;
					}
				};
				uiJob.setUser(false);
				uiJob.setSystem(true);
				uiJob.schedule();
			}

			@Override
			public void done(final IJobChangeEvent event) {
				if (event.getResult().isOK()) {
					multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
							"Aktion Konsistenzprüfung beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
					final UIJob uiJob = new UIJob("aktualisiere GUI") {

						@Override
						public IStatus runInUIThread(final IProgressMonitor monitor) {
							aktuelleInfoKbAnzeigen();
							erzeugeInhaltTabelle(listKonfigBereichKapsel);
							return Status.OK_STATUS;
						}
					};
					uiJob.setUser(false);
					uiJob.setSystem(true);
					uiJob.schedule();

				} else {
					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
					final UIJob uiJob = new UIJob("aktualisiere GUI") {

						@Override
						public IStatus runInUIThread(final IProgressMonitor monitor) {
							updateButtonKeineSelektion();
							return Status.OK_STATUS;
						}
					};
					uiJob.setUser(false);
					uiJob.setSystem(true);
					uiJob.schedule();
				}

			}
		});
		job.setSystem(false);
		job.setUser(true);
		job.schedule();
	}

	private void aktionPruefungAktivierung() {
		multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion Ueberprüfung Aktivierung", null);
//		MeldungenSpeicher.getInstanz().setStatus(multiStatus);

		final Job job = new Job("KonfigAss: Pruefe Aktivierung") {

			@Override
			protected IStatus run(final IProgressMonitor monitor) {
				final Map<String, ConfigurationArea> areas = config.getAllConfigurationAreas();

				monitor.beginTask("Starte Aktivierungsprüfung...", areas.size() + 2);
				monitor.worked(1);

				try {
					final List<ConfigurationArea> list = new ArrayList<>();

					for (final Map.Entry<String, ConfigurationArea> me : areas.entrySet()) {
						boolean aktivieren = false;
						final ConfigurationArea ca = me.getValue();
						if (ca != null) {

							if (!ca.isValid()) {
								aktivieren = true;
							} else {
								final Collection<SystemObject> objects = ca.getObjects(null,
										ObjectTimeSpecification.valid());

								for (final SystemObject so : objects) {
									if (!so.isValid()) {
										aktivieren = true;
										break;
									}
								}

								final Collection<SystemObject> newObjects = ca.getNewObjects();

								for (final SystemObject so : newObjects) {
									if (!so.isValid()) {
										aktivieren = true;
										break;
									}
								}
							}

							if (aktivieren) {
								list.add(ca);
							}
						}
						monitor.worked(1);
					}

					aktuelleInfoKbAnzeigen();
					for (final ConfigurationArea ca : list) {
						for (final KonfigBereichDaten kbd : listKonfigBereichKapsel) {
							if (ca.getPid().equals(kbd.getPid())) {
								kbd.setAktvieren(true);
							}
						}
					}

					monitor.worked(1);
				} catch (final Exception e2) {
					multiStatus.add(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
							"Aktion Aktivierungsprüfung abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. "
									+ e2));
					return Status.CANCEL_STATUS;
				} finally {
					monitor.done();
				}

				return Status.OK_STATUS;
			}
		};

		job.addJobChangeListener(new JobChangeAdapter() {

			@Override
			public void running(final IJobChangeEvent event) {
				final UIJob uiJob = new UIJob("aktualisiere GUI") {

					@Override
					public IStatus runInUIThread(final IProgressMonitor monitor) {
						return Status.OK_STATUS;
					}
				};
				uiJob.setUser(false);
				uiJob.setSystem(true);
				uiJob.schedule();
			}

			@Override
			public void done(final IJobChangeEvent event) {
				if (event.getResult().isOK()) {
					multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
							"Aktion Aktivierungsprüfung beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
					final UIJob uiJob = new UIJob("aktualisiere GUI") {

						@Override
						public IStatus runInUIThread(final IProgressMonitor monitor) {
							erzeugeInhaltTabelle(listKonfigBereichKapsel);
							return Status.OK_STATUS;
						}
					};
					uiJob.setUser(false);
					uiJob.setSystem(true);
					uiJob.schedule();

				} else {
					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
					final UIJob uiJob = new UIJob("aktualisiere GUI") {

						@Override
						public IStatus runInUIThread(final IProgressMonitor monitor) {
							updateButtonKeineSelektion();
							return Status.OK_STATUS;
						}
					};
					uiJob.setUser(false);
					uiJob.setSystem(true);
					uiJob.schedule();
				}

			}
		});
		job.setSystem(false);
		job.setUser(true);
		job.schedule();
	}

	private void aktionAktivierung() {
		if (globalSelection != null) {
			final Object obj = globalSelection.getFirstElement();
			if (obj instanceof KonfigBereichDaten) {
				final Iterator<?> it = globalSelection.iterator();

				final Map<String, File> list = new HashMap<>();

				while (it.hasNext()) {
					final KonfigBereichDaten kbd = (KonfigBereichDaten) it.next();
					final String pid = kbd.getPid();
					final Map<String, ConfigurationArea> map = config.getAllConfigurationAreas();
					if (map.containsKey(pid)) {
						final ConfigurationArea ca = map.get(pid);
						list.put(ca.getPid(), new File(""));
					}

				}
				if (list.size() > 0) {
					final KbAuswahlAssistent wizard = new KbAuswahlAssistent("Aktivieren", "aktiviert", list);

					wizard.init(PlatformUI.getWorkbench(), null);
					// Instantiates the wizard container with the wizard and
					// opens it
					final WizardDialog dialog = new WizardDialog(
							PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
					dialog.create();
					final int ret = dialog.open();

					if (ret == 0) {
						multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion Aktivierung",
								null);

						final KbAuswahlModell model = wizard.getModel();
						final Map<String, File> list2 = model.getList();

						final Job job = new Job("KonfigAss: Aktiviere Änderungen") {

							@Override
							protected IStatus run(final IProgressMonitor monitor) {
								monitor.beginTask("Starte Aktivierung...", 5);
								monitor.worked(1);

								try {

									final List<ConfigurationArea> listCa = new ArrayList<>();

									for (final Map.Entry<String, File> me : list2.entrySet()) {
										final String pid = me.getKey();
										final ConfigurationArea ca = config.getAllConfigurationAreas().get(pid);
										if (ca != null) {
											listCa.add(ca);
										}
									}

									for (final ConfigurationArea ca : listCa) {
										multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
												"Aktiviere KB: " + ca.getPid()));
									}

									final KonfigApplikation app = new KonfigApplikation(config);
									KonfigAssistentErgebnis ergebnis = app.starteAktivierung(listCa, false, false);
									ergebnis.ausgebenInMultiStatus(multiStatus);

									monitor.worked(1);
								} catch (final Exception e2) {
									multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
											"Aktion Aktivierung abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. "
													+ e2));
									return Status.CANCEL_STATUS;
								} finally {
									monitor.worked(3);
									monitor.done();
								}

								return Status.OK_STATUS;
							}
						};

						job.addJobChangeListener(new JobChangeAdapter() {

							@Override
							public void running(final IJobChangeEvent event) {
								final UIJob uiJob = new UIJob("aktualisiere GUI") {

									@Override
									public IStatus runInUIThread(final IProgressMonitor monitor) {
										return Status.OK_STATUS;
									}
								};
								uiJob.setUser(false);
								uiJob.setSystem(true);
								uiJob.schedule();
							}

							@Override
							public void done(final IJobChangeEvent event) {
								if (event.getResult().isOK()) {
									multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
											"Aktion Aktivierung beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
									MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);

									updateButtonJobAktiv();
									aktionLadeDatenModell(false, null);
								} else {
									MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
									final UIJob uiJob = new UIJob("aktualisiere GUI") {

										@Override
										public IStatus runInUIThread(final IProgressMonitor monitor) {
											updateButtonKeineSelektion();
											return Status.OK_STATUS;
										}
									};
									uiJob.setUser(false);
									uiJob.setSystem(true);
									uiJob.schedule();
								}

							}
						});
						job.setSystem(false);
						job.setUser(true);
						job.schedule();

					} else {
						selChangedlistener
								.selectionChanged(new SelectionChangedEvent(tvZufluesse1, tvZufluesse1.getSelection()));
					}
				}
			}
		}
	}

	private void aktionAktivierung(final List<ConfigurationArea> list) {
		multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion Aktivierung", null);

		final Job job = new Job("KonfigAss: Aktiviere Änderungen") {

			@Override
			protected IStatus run(final IProgressMonitor monitor) {
				monitor.beginTask("Starte Aktivierung...", 5);
				monitor.worked(1);

				try {
					if (list.size() > 0) {
						final KonfigApplikation app = new KonfigApplikation(config);

						for (final ConfigurationArea ca : list) {
							multiStatus
									.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID, "Aktiviere KB: " + ca.getPid()));
						}

						KonfigAssistentErgebnis ergebnis = app.starteAktivierung(list, false, false);
						ergebnis.ausgebenInMultiStatus(multiStatus);

						monitor.worked(1);

					}
				} catch (final Exception e2) {
					multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
							"Aktion Aktivierung abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. "
									+ e2));
					return Status.CANCEL_STATUS;
				} finally {
					monitor.worked(3);
					monitor.done();
				}

				return Status.OK_STATUS;
			}
		};

		job.addJobChangeListener(new JobChangeAdapter() {

			@Override
			public void running(final IJobChangeEvent event) {
				final UIJob uiJob = new UIJob("aktualisiere GUI") {

					@Override
					public IStatus runInUIThread(final IProgressMonitor monitor) {
						return Status.OK_STATUS;
					}
				};
				uiJob.setUser(false);
				uiJob.setSystem(true);
				uiJob.schedule();
			}

			@Override
			public void done(final IJobChangeEvent event) {
				if (event.getResult().isOK()) {
					multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
							"Aktion Aktivierung beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);

					updateButtonJobAktiv();
					aktionLadeDatenModell(false, null);
				} else {
					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
					final UIJob uiJob = new UIJob("aktualisiere GUI") {

						@Override
						public IStatus runInUIThread(final IProgressMonitor monitor) {
							updateButtonKeineSelektion();
							return Status.OK_STATUS;
						}
					};
					uiJob.setUser(false);
					uiJob.setSystem(true);
					uiJob.schedule();
				}

			}
		});
		job.setSystem(false);
		job.setUser(true);
		job.schedule();
	}

	private void aktionImport() {
		if (config != null) {

			final KbImportAssistent wizard = new KbImportAssistent(dateiPfad, true);
			wizard.init(PlatformUI.getWorkbench(), null);
			// Instantiates the wizard container with the wizard and opens it
			final WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(),
					wizard);
			dialog.create();

			final int ret = dialog.open();

			if (ret == 0) {
				multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion Import", null);

				final KbImportModell model = wizard.getModel();

				final Map<String, File> files = model.getList();

				final List<String> list = new ArrayList<>();

				final Collection<String> values = files.keySet();

				for (final String s : values) {
					if (!list.contains(s)) {

						if (s.contains(".xml")) {
							final int i = s.indexOf(".xml");
							final String neu = s.subSequence(0, i).toString();
							list.add(neu);
						}

					}
				}

				if (list.size() > 0) {

					final List<String> flist = list;

					final Job job = new Job("KonfigAss: Importiere") {
						@Override
						protected IStatus run(final IProgressMonitor monitor) {
							monitor.beginTask("Starte Import...", 5);
							monitor.worked(1);

							try {

								final KonfigApplikation app = new KonfigApplikation(config);

								final File[] ff = files.values().toArray(new File[files.values().size()]);

								final File file = ff[0];

								final String path = file.getParent();
								final File folder = new File(path);

								for (final String string : flist) {
									multiStatus.merge(
											new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID, "Importiere KB : " + string));
								}
								app.startImport(folder, flist);
								monitor.worked(1);
							} catch (final Exception e2) {
								multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
										"Aktion Import abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. ",
										e2));
								return Status.CANCEL_STATUS;
							} finally {
								monitor.worked(3);
								monitor.done();
							}

							return Status.OK_STATUS;
						}
					};

					job.addJobChangeListener(new JobChangeAdapter() {

						@Override
						public void running(final IJobChangeEvent event) {
							final UIJob uiJob = new UIJob("aktualisiere GUI") {

								@Override
								public IStatus runInUIThread(final IProgressMonitor monitor) {
									return Status.OK_STATUS;
								}
							};
							uiJob.setUser(false);
							uiJob.setSystem(true);
							uiJob.schedule();
						}

						@Override
						public void done(final IJobChangeEvent event) {
							if (event.getResult().isOK()) {
								multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
										"Aktion Import beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
								MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
								final UIJob uiJob = new UIJob("aktualisiere GUI") {

									@Override
									public IStatus runInUIThread(final IProgressMonitor monitor) {
										aktuelleInfoKbAnzeigen();
										erzeugeInhaltTabelle(listKonfigBereichKapsel);

										updateTextKonfiguration(dateiPfad, pidAoeKonfig);
										setzeKonfigDatenModellFuerTka(config, dateiPfad);
										return Status.OK_STATUS;
									}
								};
								uiJob.setUser(false);
								uiJob.setSystem(true);
								uiJob.schedule();
							} else {
								MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
								final UIJob uiJob = new UIJob("aktualisiere GUI") {

									@Override
									public IStatus runInUIThread(final IProgressMonitor monitor) {
										updateButtonKeineSelektion();
										return Status.OK_STATUS;
									}
								};
								uiJob.setUser(false);
								uiJob.setSystem(true);
								uiJob.schedule();
							}

						}
					});
					job.setSystem(false);
					job.setUser(true);
					job.schedule();
				}
			} else {
				selChangedlistener
						.selectionChanged(new SelectionChangedEvent(tvZufluesse1, tvZufluesse1.getSelection()));
			}
		}
	}

	private void aktionKbHinzufuegen() {
		final KbEntfernenHinzufuegenAssistent wizard = new KbEntfernenHinzufuegenAssistent(dateiPfad, true);

		wizard.init(PlatformUI.getWorkbench(), null);
		// Instantiates the wizard container with the wizard and opens it
		final WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
		dialog.create();

		final int ret = dialog.open();

		if (ret == 0) {
			multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion KB Hinzufügen. ", null);

			final KbEntfernenHinzufuegenModell model = wizard.getModel();

			final Map<String, File> list = model.getList();

			if (tAutoCloser != null) {
				tAutoCloser.start();
			}

			final List<ConfigurationAreaManagementInfo> list3 = config.getManagementFile()
					.getAllConfigurationAreaManagementInfos();

			final List<String> listPidsCa = new ArrayList<>();

			for (final ConfigurationAreaManagementInfo cami : list3) {
				final String pid = cami.getPid();
				listPidsCa.add(pid);
			}

			// Warten, bis der Thread beendet ist: finish
			try {
				if (tAutoCloser != null) {
					tAutoCloser.join();
				}

				config = null;

				final ManagementFile managementFile = new ManagementFile(new File(dateiPfad));

				final Set<String> keySet = list.keySet();
				final List<String> pids = new ArrayList<>();
				for (final String s : keySet) {
					if (s.contains(".config")) {
						final int i = s.indexOf(".config");
						final String neu = s.subSequence(0, i).toString();

						if (listPidsCa.contains(neu)) {
							multiStatus.merge(new Status(IStatus.WARNING, KonfigurationsAssistent.PLUGIN_ID,
									"KB ist schon in der aktuellen Konfiguration vorhanden: " + neu));
						} else {
							final ConfigurationAreaManagementInfo cami = managementFile
									.addConfigurationAreaManagementInfo(neu);
							final String pfadParent = list.get(s).getParent();
							cami.setDirectory(new File(pfadParent));
							pids.add(neu);
							multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
									"Füge KB hinzu: " + neu + " - Pfad: " + pfadParent));
						}

					} else {
						multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
								"Fehler beim Hinzufügen der Datei: " + s));
					}

				}

				if (pids.size() > 0) {
					managementFile.save();
				}

				managementFile.close();

				multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
						"Aktion Laden Datenmodell beendet, bitte beachten Sie evtl. Ausgaben der Konfiguration"));
				MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);

				updateButtonJobAktiv();

				aktionLadeDatenModell(false, pids);
			} catch (final Exception ex) {
				multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
						"Aktion KB Hinzufügen abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. ",
						ex));
				MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
			}
		} else {
			selChangedlistener.selectionChanged(new SelectionChangedEvent(tvZufluesse1, tvZufluesse1.getSelection()));
		}
	}

	private void aktionKbEntfernen() {
		if (config != null) {
			final Map<String, File> list = new HashMap<>();

			final List<String> listFehler = new ArrayList<>();

			final IStructuredSelection selection = (IStructuredSelection) tvZufluesse1.getSelection();

			if (selection.size() > 0) {
				final Object firstElement = selection.getFirstElement();

				if (firstElement instanceof KonfigBereichDaten) {
					final List<KonfigBereichDaten> listSel = new ArrayList<>();

					final Iterator<?> it = selection.iterator();
					while (it.hasNext()) {
						final KonfigBereichDaten kbd = (KonfigBereichDaten) it.next();
						listSel.add(kbd);
					}

					boolean fehler = false;
					for (final KonfigBereichDaten kbd : listSel) {
						for (final KonfigBereichDaten kbd2 : listKonfigBereichKapsel) {
							final ConfigurationArea ca = config.getAllConfigurationAreas().get(kbd2.getPid());
							if (ca != null) {
								final AttributeGroup atg = config
										.getAttributeGroup("atg.konfigurationsBereichAbhängigkeiten");
								final ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(config);
								final Data data = ((ConfigSystemObject) ca).getConfigurationData(atg, lookup);
								if (data != null) {
									Data item = data.getItem("KonfigurationsAbhängigkeiten");
									final Array array = item.asArray();
									for (int i = 0; i < array.getLength(); i++) {
										item = array.getItem(i);

										final String pid = item.getItem("BereichNotwendig").asTextValue()
												.getValueText();
										if (pid != null) {
											final ConfigurationArea kb = config.getConfigurationArea(pid);
											if (kb != null) {
												final String text2 = item.getItem("Kennung").asTextValue()
														.getValueText();

												if ("notwendig".equals(text2)) {

													if (kbd.getPid().equals(kb.getPid())) {

														if (!listFehler.contains(kbd2.getPid())
																&& !listSel.contains(kbd2)) {
															fehler = true;
															listFehler.add(kbd2.getPid());
														}
													}
												}
											}
										}
									}
								}

							} else {
								final String meldung = "Fehler bei den Abhängigkeiten des KB: " + kbd2.getPid()
										+ " es ist kein SystemObject für den KB vorhanden!";
								multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID, meldung));
								fehler = true;
							}
						}
						list.put(kbd.getPid(), null);
					}
					if (fehler) {
						final StringBuilder meldung = new StringBuilder();
						final String separator = System.getProperty("line.separator");
						meldung.append("Das Entfernen des/der KB führt zu einem inkonsistenten Kernsystem,  ");
						meldung.append("da dieser/diese von folgenden KB benötigt werden:");
						meldung.append(separator);
						meldung.append(separator);

						for (int j = 0; j < listFehler.size(); j++) {
							final String s = listFehler.get(j);
							if (j < (listFehler.size() - 1)) {
								meldung.append(", ");
							}
							meldung.append(s);
						}

						meldung.append(separator);
						meldung.append(separator);
						meldung.append("Soll die Aktion trotzdem fortgeführt werden?");

						final MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(),
								SWT.OK | SWT.CANCEL | SWT.ERROR);
						msg.setText("Fehler KB: Abhängigkeiten");
						msg.setMessage(meldung.toString());
						final int open = msg.open();

						if (open != SWT.OK) {
							return;
						}

					}
				}
			}

			final KbEntfernenHinzufuegenAssistent wizard = new KbEntfernenHinzufuegenAssistent(pidAoeKonfig, false,
					list);

			wizard.init(PlatformUI.getWorkbench(), null);
			// Instantiates the wizard container with the wizard and opens it
			final WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(),
					wizard);
			dialog.create();
			final int ret = dialog.open();

			if (ret == 0) {
				multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion KB Entfernen. ", null);

				final XmlElementKonfigVerwaltung xmlElementKonfigVerwaltung = new XmlElementKonfigVerwaltung();

				xmlElementKonfigVerwaltung.einlesenXmlAusDatei(dateiPfad);

				final List<ElementKonfig> listeKbXml = xmlElementKonfigVerwaltung.getElementKonfigBereich();

				final KbEntfernenHinzufuegenModell model = wizard.getModel();
				final Map<String, File> list2 = model.getList();

				final List<ElementKonfig> delList = new ArrayList<>();

				if (listeKbXml != null) {
					for (final ElementKonfig ek : listeKbXml) {
						if (ek != null) {
							final String b = ek.getBezeichnung();
							final Set<String> keySet = list2.keySet();
							for (final String s : keySet) {
								if (s.equals(b)) {
									delList.add(ek);
								}
							}
						}
					}
				}

				if (tAutoCloser != null) {
					tAutoCloser.start();
				}

				// Warten, bis der Thread beendet ist: finish
				try {
					if (tAutoCloser != null) {
						tAutoCloser.join();
					}

					config = null;

					if (delList.size() > 0) {
						for (final ElementKonfig ek2 : delList) {
							xmlElementKonfigVerwaltung.entferneElementKonfig(ek2);
							multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
									"Entferne KB: " + ek2.getBezeichnung()));
						}

						final String s = xmlElementKonfigVerwaltung.ausgabeXmlFormatiert();
						xmlElementKonfigVerwaltung.schreibeXMLStringInDatei(dateiPfad, s, true);

						multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
								"Aktion KB Entfernen beendet, bitte beachten Sie evtl. die Ausgaben der Konfiguration"));
						MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);

						updateButtonJobAktiv();
						aktionLadeDatenModell(false, null);

					}
				} catch (final Exception ex) {
					multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
							"Aktion KB Entfernen abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. ",
							ex));
					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
				}
			} else {
				selChangedlistener
						.selectionChanged(new SelectionChangedEvent(tvZufluesse1, tvZufluesse1.getSelection()));
			}
		}
	}

	private void aktionKbNeu() {
		if (config != null) {
			final KbAnlegenAssistent wizard = new KbAnlegenAssistent();

			final WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(),
					wizard);
			final int ret = dialog.open();

			if (ret == Window.OK) {
				final KbAnlegenModell model = wizard.getModel();
				final String pid = model.getPid();
				final String name = model.getName();
				final long stand = model.getStand();
				final String autor = model.getAutor();
				final String grund = model.getGrund();
				final String text = model.getText();
				final String info = model.getInfo();
				final String beschreibung = model.getBeschreibung();

				multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion KB Neu", null);

				final String aoePid = config.getConfigurationAuthorityPid();

				try {
					multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
							"Erstelle KB: " + pid + " unter dem KV: " + aoePid));

					boolean vorhanden = false;
					final File verzeichnis = new File(dateiPfad).getParentFile();
					if (verzeichnis.exists() && verzeichnis.isDirectory()) {
						final String[] list = verzeichnis.list();
						if (list != null) {
							for (final String s : list) {
								if (s.contains(".config")) {
									final int i = s.indexOf(".config");
									final String neu = s.subSequence(0, i).toString();
									if (neu.equals(pid)) {
										vorhanden = true;
									}
								}
							}
						}
					}

					if (!vorhanden) {
						final ConfigurationArea ca = config.createConfigurationArea(name, pid, aoePid);

						AttributeGroup atg = config.getAttributeGroup("atg.konfigurationsÄnderungen");
						Data data = AttributeBaseValueDataFactory.createAdapter(atg,
								AttributeHelper.getAttributesValues(atg));

						final Array array = data.getItem("KonfigurationsÄnderung").asArray();
						array.setLength(1);
						array.getItem(0).getItem("Stand").asTimeValue().setMillis(stand);
						array.getItem(0).getItem("Autor").asTextValue().setText(autor);
						array.getItem(0).getItem("Version").asTextValue().setText("1");
						array.getItem(0).getItem("Grund").asTextValue().setText(grund);
						array.getItem(0).getItem("Text").asTextValue().setText(text);
						ca.setConfigurationData(atg, data);

						atg = config.getAttributeGroup("atg.info");
						data = AttributeBaseValueDataFactory.createAdapter(atg,
								AttributeHelper.getAttributesValues(atg));

						data.getTextValue("kurzinfo").setText(info);
						data.getTextValue("beschreibung").setText(beschreibung);
						ca.setConfigurationData(atg, data);

						final ConfigurationAreaManagementInfo cami = config.getManagementFile()
								.getConfigurationAreaManagementInfo(ca.getPid());
						cami.setDirectory(verzeichnis);

						multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
								"Aktion KB Neu beendet, bitte beachten Sie evtl. Ausgaben der Konfiguration"));

						final UIJob uiJob = new UIJob("aktualisiere GUI") {

							@Override
							public IStatus runInUIThread(final IProgressMonitor monitor) {
								aktuelleInfoKbAnzeigen();
								erzeugeInhaltTabelle(listKonfigBereichKapsel);

								updateTextKonfiguration(dateiPfad, pidAoeKonfig);

								setzeKonfigDatenModellFuerTka(config, dateiPfad);
								return Status.OK_STATUS;
							}
						};
						uiJob.setUser(false);
						uiJob.setSystem(true);
						uiJob.schedule();

					} else {
						multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
								"Aktion KB Neu abgebrochen! Grund: Es ist bereits ein Konfigurationsbereich vorhanden: "
										+ pid));
					}
					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);

				} catch (final Exception ex) {
					multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
							"Aktion KB Neu abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. ",
							ex));
					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
				}
			} else {
				selChangedlistener
						.selectionChanged(new SelectionChangedEvent(tvZufluesse1, tvZufluesse1.getSelection()));
			}
		}
	}

	private void aktionNeuerKvAoe() {
		final KvAoeNeuAssistent wizard = new KvAoeNeuAssistent();

		final WizardDialog dialog = new TkaWizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(),
				wizard);
		dialog.create();

		final int ret = dialog.open();

		if (ret == 0) {
			final KvAoeNeuModell model = wizard.getModel();
			final String ziel = model.getZielVerzeichnis();
			dateiPfad = ziel.replace("\\", "/") + "/config/verwaltungsdaten.xml";

			final File file = new File(dateiPfad);

			final String jobname = "KonfigAss: Neuer KV/AOE";

			final Job job = new Job(jobname) {
				@Override
				protected IStatus run(final IProgressMonitor monitor) {
					if (model.isUseKV()) {
						monitor.beginTask("Start Erstellung KV...", 4);

						multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion Neuer KV", null);
					} else {
						monitor.beginTask("Start Erstellung AOE...", 8);
						multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion Neuer AOE", null);
					}

					if ((model.getConfig() != null) && (model.getAutoCloser() != null)) {
						config = model.getConfig();
						tAutoCloser = model.getAutoCloser();
						config.setConfigurationAuthority(null);
					} else {
						config = erzeugeKonfigDatenModellAusDatei(file);
					}
					monitor.worked(1);

					if (config == null) {
						multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
								"Aktion Neuer KV/AOE abgebrochen! Grund: es konnte kein Datenmodell erzeugt werden."));
						return Status.CANCEL_STATUS;
					}

					final ConfigurationManagementFile managementFile = config.getManagementFile();

					if (managementFile != null) {

						final String pid = model.getPid();
						managementFile.setConfigurationAuthority(pid);

						if (model.isUseKV()) {
							try {
								final String name = "KB fuer " + model.getName();
								final String kb = "kb." + model.getPid();
								final String code = model.getKodierung();
								final long l = Long.parseLong(code);
								final String kv = managementFile.getConfigurationAuthority();

								// Bereich und
								// Konfigurationsverantwortlicher werden
								// erstellt
								final SystemObjectType type = config.getType("typ.konfigurationsVerantwortlicher");

								monitor.setTaskName("Erzeuge KB für KV");
								final ConfigurationArea ca = config.createConfigurationArea(name, kb, type, kv,
										model.getName(), l);
								monitor.worked(1);

								multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
										"Starte Erzeugung KB für KV", null));

								setInfo(ca, model);

								// die Zeiten, wann sich ein Datensatz
								// geändert oder ein Objekt erzeugt wurde,
								// müssen
								// initialisiert
								// werden
								((ConfigConfigurationArea) ca).initialiseTimeOfLastChanges();

								final AttributeGroup atg = config
										.getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften");
								final Data data = AttributeBaseValueDataFactory.createAdapter(atg,
										AttributeHelper.getAttributesValues(atg));
								data.getItem("kodierung").asTextValue().setText(code);

								final Collection<SystemObject> zuukuenftige = ca.getNewObjects();

								for (final SystemObject so : zuukuenftige) {
									if (so.getPid().equals(kv)) {

										final ConfigurationObject co = (ConfigurationObject) so;

										co.setConfigurationData(atg, data);

										multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
												"Erzeugung KB für KV erfolgreich beendet", null));
									}

								}

								final List<ConfigurationArea> list = new ArrayList<>();

								list.addAll(config.getAllConfigurationAreas().values());

								final KonfigApplikation app = new KonfigApplikation(config);

								monitor.setTaskName("Aktiviere KB: " + ca.getPid());

								multiStatus.merge(
										new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID, "Aktiviere Änderungen", null));

								KonfigAssistentErgebnis ergebnis = app.starteAktivierung(list, false, false);
								ergebnis.ausgebenInMultiStatus(multiStatus);

								monitor.worked(1);

								multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
										"Aktivierung  beendet, bitte beachten Sie evtl. die Ausgaben der Konfiguration",
										null));

							} catch (final ConfigurationChangeException e1) {
								multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
										"Fehler bei Erzeugung KB für KV", e1));
							}

						} else {

							try {
								final String name = "AOE fuer " + model.getName();
								final String kb = "kb." + model.getPid();
								final String code = model.getKodierung();
								final Long l = Long.parseLong(code);
								final String kv = managementFile.getConfigurationAuthority();

								multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
										"Starte Erzeugung KB für AOE", null));

								monitor.setTaskName("Erzeuge KB für AOE");
								// Bereich und AOE werden erstellt
								final SystemObjectType type = config.getType("typ.autarkeOrganisationsEinheit");
								final ConfigurationArea ca = config.createConfigurationArea(name, kb, type, kv,
										model.getName(), l);
								monitor.worked(1);

								setInfo(ca, model);

								// die Zeiten, wann sich ein Datensatz
								// geändert oder ein Objekt erzeugt wurde,
								// müssen
								// initialisiert
								// werden
								((ConfigConfigurationArea) ca).initialiseTimeOfLastChanges();

								multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
										"Erzeugung KB für AOE erfolgreich abgeschlossen", null));

								final AttributeGroup atg = config
										.getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften");
								final Data data = AttributeBaseValueDataFactory.createAdapter(atg,
										AttributeHelper.getAttributesValues(atg));
								data.getItem("kodierung").asTextValue().setText(code);

								multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
										"Starte Erzeugung Mengen für AOE", null));

								final Collection<SystemObject> zuukuenftige = ca.getNewObjects();
								monitor.setTaskName("Erzeuge Mengen");

								final Map<String, String> mengen = model.getMengen();

								for (final SystemObject so : zuukuenftige) {
									if (so.getPid().equals(kv)) {
										final ConfigurationObject co = (ConfigurationObject) so;

										co.setConfigurationData(atg, data);

										for (final Map.Entry<String, String> me : mengen.entrySet()) {

											final String value = me.getValue();
											final SystemObjectType typ = config.getType(value);

											if (typ != null) {
												final ObjectSet set = (ObjectSet) ca.createConfigurationObject(
														(ConfigurationObjectType) typ, "", me.getKey(), null);

												if (set != null) {
													co.addSet(set);
												}
											} else {
												multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
														"Konnte Typ der Menge nicht ermitteln: " + me.getKey(), null));
											}

										}
									}
								}
								monitor.worked(1);

								multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
										"Erzeugung Mengen für AOE erfolgreich beendet", null));

								final List<ConfigurationArea> list = new ArrayList<>();
								list.add(ca);

								final KonfigApplikation app = new KonfigApplikation(config);

								multiStatus
										.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID, "Starte Aktivierung", null));

								monitor.setTaskName("Aktiviere: " + ca.getPid());
								app.starteAktivierung(list, false, false);
								monitor.worked(1);

								multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
										"Aktivierung erfolgreich beendet", null));
								multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
										"Starte Entladen Datenmodell", null));

								monitor.setTaskName("Schliesse Konfiguration...");
								tAutoCloser.start();
								try {
									tAutoCloser.join();
									monitor.worked(1);
									config = null;
									monitor.setTaskName("Lade Datenmodell...");
									config = erzeugeKonfigDatenModellAusDatei(file);
									monitor.worked(1);

									multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
											"Entladen Datenmodell erfolgreich beendet", null));

									multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
											"Starte Erzeugung Default-KB für AOE", null));

									monitor.setTaskName("Erzeuge Default KB für AOE");

									final String nameDefaultKb = "Default-Bereich für: " + model.getName();
									final String pidDefaultKb = "kb.default." + model.getPid();

									final ConfigurationArea caDef = config.createConfigurationArea(nameDefaultKb,
											pidDefaultKb, kv);
									monitor.worked(1);

									setInfoDef(caDef, model);

									multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
											"Erzeugung Default-KB für AOE erfolgreich beendet", null));

									monitor.setTaskName("Aktualisiere KB für AOE");

									multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
											"Starte Aktualisierung KB für AOE ", null));
									final SystemObject so = config.getObject(kv);
									if (so != null) {
										final ConfigurationObject co = (ConfigurationObject) so;
										final Data cd = co.getConfigurationData(atg);
										cd.getItem("kodierung").asTextValue().setText(code);
										final Array array = cd.getItem("defaultBereich").asArray();
										array.setLength(1);
										array.getItem(0).asTextValue().setText("kb.default." + model.getPid());
										co.setConfigurationData(atg, cd);
									}

									multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
											"Aktualisierung KB für AOE erfolgreich beendet", null));

									monitor.worked(1);

									list.add(caDef);

								} catch (final InterruptedException e) {
									multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
											"Fehler beim Schliessen des Datenmodells!", e));
								}

							} catch (final ConfigurationChangeException e1) {
								multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
										"Fehler beim Ändern der Konfigurationsdaten für den Default-KB!", e1));
							}

						}

					}

					monitor.done();

					if (config == null) {
						return Status.CANCEL_STATUS;
					}
					return Status.OK_STATUS;
				}
			};

			job.addJobChangeListener(new JobChangeAdapter() {

				@Override
				public void running(final IJobChangeEvent event) {
					final UIJob uiJob = new UIJob("aktualisiere GUI") {

						@Override
						public IStatus runInUIThread(final IProgressMonitor monitor) {
							return Status.OK_STATUS;
						}
					};
					uiJob.setUser(false);
					uiJob.setSystem(true);
					uiJob.schedule();
				}

				@Override
				public void done(final IJobChangeEvent event) {
					if (event.getResult().isOK()) {
						if (model.isUseKV()) {
							tAutoCloser.start();

							// Warten, bis der Thread beendet ist: finish
							try {
								tAutoCloser.join();

								config = null;

								updateButtonJobAktiv();

								multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
										"Aktion Neuer KV beendet, bitte beachten Sie die Ausgaben der Konfiguration",
										null));
								MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);

								aktionLadeDatenModell(false, null);
							} catch (final InterruptedException ex) {
								multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
										"Fehler beim Schliessen des Datenmodells!", ex));
								MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
							}

						} else {
							multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
									"Aktion Neuer AOE beendet, bitte beachten Sie die Ausgaben der Konfiguration",
									null));
							MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);

							final Map<String, ConfigurationArea> areas = config.getAllConfigurationAreas();

							final List<ConfigurationArea> list = new ArrayList<>();

							for (final Map.Entry<String, ConfigurationArea> me : areas.entrySet()) {
								final ConfigurationArea ca = me.getValue();
								if (!ca.isValid()) {
									list.add(ca);
								}
							}

							aktionAktivierung(list);
						}
					} else {
						if (model.isUseKV()) {
							multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
									"Aktion Neuer KV abgebrochen! Grund: es ist mind. eine Ausnahme bei der Ausführung aufgetreten. ",
									null));
							MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
						} else {
							multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
									"Aktion Neuer AOE abgebrochen! Grund: es ist mind. eine Ausnahme bei der Ausführung aufgetreten. ",
									null));
							MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
						}

						final UIJob uiJob = new UIJob("aktualisiere GUI") {

							@Override
							public IStatus runInUIThread(final IProgressMonitor monitor) {
								erzeugeInhaltTabelle(new ArrayList<KonfigBereichDaten>());
								updateTextKonfiguration("", "");
								updateButtonKeinDatenModell();
								setzeKonfigDatenModellFuerTka(null, dateiPfad);

								// Löschen Ordnerstruktur
								KonfigAssDateiOperationen.loescheVerzeichnis(new File(ziel), monitor, 1);

								return Status.OK_STATUS;
							}
						};
						uiJob.setUser(false);
						uiJob.setSystem(true);
						uiJob.schedule();

					}

				}
			});
			job.setSystem(false);
			job.setUser(true);
			job.schedule();

		} else {
			updateButtonKeinDatenModell();
		}
	}

	private void aktionFreigabeZurUebernahme() {
		if (globalSelection != null) {
			final Object obj = globalSelection.getFirstElement();
			if (obj instanceof KonfigBereichDaten) {
				final Iterator<?> it = globalSelection.iterator();

				final Map<String, File> list = new HashMap<>();

				while (it.hasNext()) {
					final KonfigBereichDaten kbd = (KonfigBereichDaten) it.next();
					final String pid = kbd.getPid();
					final Map<String, ConfigurationArea> map = config.getAllConfigurationAreas();
					if (map.containsKey(pid)) {
						final ConfigurationArea ca = map.get(pid);
						list.put(ca.getPid(), new File(""));
					}

				}
				if (list.size() > 0) {
					final KbAuswahlAssistent wizard = new KbAuswahlAssistent("Freigabe zur Übernahme", "freigegeben",
							list);

					wizard.init(PlatformUI.getWorkbench(), null);
					// Instantiates the wizard container with the wizard and
					// opens it
					final WizardDialog dialog = new WizardDialog(
							PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
					dialog.create();
					final int ret = dialog.open();

					if (ret == 0) {
						multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK,
								"Starte Aktion Freigabe zur Übernahme", null);

						final KbAuswahlModell model = wizard.getModel();
						final Map<String, File> list2 = model.getList();

						final Job job = new Job("KonfigAss: Freigabe zur Übernahme") {

							@Override
							protected IStatus run(final IProgressMonitor monitor) {
								monitor.beginTask("Starte Freigabe...", 5);
								monitor.worked(1);

								try {

									final List<ConfigurationArea> listCa = new ArrayList<>();

									for (final Map.Entry<String, File> me : list2.entrySet()) {
										final String pid = me.getKey();

										final ConfigurationArea ca = config.getAllConfigurationAreas().get(pid);
										if (ca != null) {
											listCa.add(ca);
										}
									}

									for (final ConfigurationArea ca : listCa) {
										multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
												"Freigabe zur Übernahme von KB: " + ca.getPid()));
									}

									final KonfigApplikation app = new KonfigApplikation(config);
									KonfigAssistentErgebnis ergebnis = app.starteFreigabeZurUebernahme(listCa, false,
											false);
									ergebnis.ausgebenInMultiStatus(multiStatus);

									monitor.worked(1);

								} catch (final Exception e2) {
									multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
											"Aktion Freigabe zur Übernahme abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. "
													+ e2));
									return Status.CANCEL_STATUS;
								} finally {
									monitor.worked(3);
									monitor.done();
								}

								return Status.OK_STATUS;
							}
						};

						job.addJobChangeListener(new JobChangeAdapter() {

							@Override
							public void running(final IJobChangeEvent event) {
								final UIJob uiJob = new UIJob("aktualisiere GUI") {

									@Override
									public IStatus runInUIThread(final IProgressMonitor monitor) {
										return Status.OK_STATUS;
									}
								};
								uiJob.setUser(false);
								uiJob.setSystem(true);
								uiJob.schedule();
							}

							@Override
							public void done(final IJobChangeEvent event) {
								if (event.getResult().isOK()) {
									multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
											"Aktion Freigabe zur Übernahme beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
									MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);

									updateButtonJobAktiv();
									aktionLadeDatenModell(false, null);
								} else {
									MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
									final UIJob uiJob = new UIJob("aktualisiere GUI") {

										@Override
										public IStatus runInUIThread(final IProgressMonitor monitor) {
											updateButtonKeineSelektion();
											return Status.OK_STATUS;
										}
									};
									uiJob.setUser(false);
									uiJob.setSystem(true);
									uiJob.schedule();
								}

							}
						});
						job.setSystem(false);
						job.setUser(true);
						job.schedule();

					} else {
						selChangedlistener
								.selectionChanged(new SelectionChangedEvent(tvZufluesse1, tvZufluesse1.getSelection()));
					}
				}
			}
		}
	}

	private void aktionFreigabeZurAktivierung() {
		if (globalSelection != null) {
			final Object obj = globalSelection.getFirstElement();
			if (obj instanceof KonfigBereichDaten) {
				final Iterator<?> it = globalSelection.iterator();

				final Map<String, File> list = new HashMap<>();

				while (it.hasNext()) {
					final KonfigBereichDaten kbd = (KonfigBereichDaten) it.next();
					final String pid = kbd.getPid();
					final Map<String, ConfigurationArea> map = config.getAllConfigurationAreas();
					if (map.containsKey(pid)) {
						final ConfigurationArea ca = map.get(pid);
						list.put(ca.getPid(), new File(""));
					}

				}
				if (list.size() > 0) {
					final KbAuswahlAssistent wizard = new KbAuswahlAssistent("Freigabe zur Aktivierung", "freigegeben",
							list);

					wizard.init(PlatformUI.getWorkbench(), null);
					// Instantiates the wizard container with the wizard and
					// opens it
					final WizardDialog dialog = new WizardDialog(
							PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
					dialog.create();
					final int ret = dialog.open();

					if (ret == 0) {
						multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK,
								"Starte Aktion Freigabe zur Aktivierung", null);

						final KbAuswahlModell model = wizard.getModel();
						final Map<String, File> list2 = model.getList();

						final Job job = new Job("KonfigAss: Freigabe zur Aktivierung") {

							@Override
							protected IStatus run(final IProgressMonitor monitor) {
								monitor.beginTask("Starte Freigabe...", 5);
								monitor.worked(1);

								try {

									final List<ConfigurationArea> listCa = new ArrayList<>();

									for (final Map.Entry<String, File> me : list2.entrySet()) {
										final String pid = me.getKey();

										final ConfigurationArea ca = config.getAllConfigurationAreas().get(pid);
										if (ca != null) {
											listCa.add(ca);
										}
									}

									for (final ConfigurationArea ca : listCa) {
										multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
												"Aktiviere KB: " + ca.getPid()));
									}

									final KonfigApplikation app = new KonfigApplikation(config);
									KonfigAssistentErgebnis ergebnis = app.starteFreigabeZurAktivierung(listCa);
									ergebnis.ausgebenInMultiStatus(multiStatus);

									monitor.worked(1);

								} catch (final Exception e2) {
									multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
											"Aktion Freigabe zur Aktivierung abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. "
													+ e2));
									return Status.CANCEL_STATUS;
								} finally {
									monitor.worked(3);
									monitor.done();
								}

								return Status.OK_STATUS;
							}
						};

						job.addJobChangeListener(new JobChangeAdapter() {

							@Override
							public void running(final IJobChangeEvent event) {
								final UIJob uiJob = new UIJob("aktualisiere GUI") {

									@Override
									public IStatus runInUIThread(final IProgressMonitor monitor) {
										return Status.OK_STATUS;
									}
								};
								uiJob.setUser(false);
								uiJob.setSystem(true);
								uiJob.schedule();
							}

							@Override
							public void done(final IJobChangeEvent event) {
								if (event.getResult().isOK()) {
									multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
											"Aktion Freigabe zur Aktivierung beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
									MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);

									updateButtonJobAktiv();
									aktionLadeDatenModell(false, null);
								} else {
									MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
									final UIJob uiJob = new UIJob("aktualisiere GUI") {

										@Override
										public IStatus runInUIThread(final IProgressMonitor monitor) {
											updateButtonKeineSelektion();
											return Status.OK_STATUS;
										}
									};
									uiJob.setUser(false);
									uiJob.setSystem(true);
									uiJob.schedule();
								}

							}
						});
						job.setSystem(false);
						job.setUser(true);
						job.schedule();

					} else {
						selChangedlistener
								.selectionChanged(new SelectionChangedEvent(tvZufluesse1, tvZufluesse1.getSelection()));
					}
				}
			}
		}
	}

	private void aktionFreigabeZurAktivierungKv() {
		if (globalSelection != null) {
			final Object obj = globalSelection.getFirstElement();
			if (obj instanceof KonfigBereichDaten) {
				final Iterator<?> it = globalSelection.iterator();

				final Map<String, File> list = new HashMap<>();

				while (it.hasNext()) {
					final KonfigBereichDaten kbd = (KonfigBereichDaten) it.next();
					final String pid = kbd.getPid();
					final Map<String, ConfigurationArea> map = config.getAllConfigurationAreas();
					if (map.containsKey(pid)) {
						final ConfigurationArea ca = map.get(pid);
						list.put(ca.getPid(), new File(""));
					}

				}
				if (list.size() > 0) {
					final KbAuswahlAssistent wizard = new KbAuswahlAssistent("F.z.Akt.o.l.A.", "freigegeben", list);

					wizard.init(PlatformUI.getWorkbench(), null);
					// Instantiates the wizard container with the wizard and
					// opens it
					final WizardDialog dialog = new WizardDialog(
							PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
					dialog.create();
					final int ret = dialog.open();

					if (ret == 0) {
						multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion F.z.Akt.o.l.A.",
								null);

						final KbAuswahlModell model = wizard.getModel();
						final Map<String, File> list2 = model.getList();

						final Job job = new Job("KonfigAss: F.z.Akt.o.l.A.") {

							@Override
							protected IStatus run(final IProgressMonitor monitor) {
								monitor.beginTask("Starte Freigabe...", 5);
								monitor.worked(1);

								try {

									final List<ConfigurationArea> listCa = new ArrayList<>();

									for (final Map.Entry<String, File> me : list2.entrySet()) {
										final String pid = me.getKey();

										final ConfigurationArea ca = config.getAllConfigurationAreas().get(pid);
										if (ca != null) {
											listCa.add(ca);
										}
									}

									for (final ConfigurationArea ca : listCa) {
										multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
												"Freigabe zur Aktivierung von KB: " + ca.getPid()));
									}

									final KonfigApplikation app = new KonfigApplikation(config);
									KonfigAssistentErgebnis ergebnis = app
											.startFreigabeZurAktivierungOhneKVAktivierung(listCa, false, false);
									ergebnis.ausgebenInMultiStatus(multiStatus);

									monitor.worked(1);

								} catch (final Exception e2) {
									multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
											"Aktion Freigabe zur Aktivierung o.l.A.d.K abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. "
													+ e2));
									return Status.CANCEL_STATUS;
								} finally {
									monitor.worked(3);
									monitor.done();
								}

								return Status.OK_STATUS;
							}
						};

						job.addJobChangeListener(new JobChangeAdapter() {

							@Override
							public void running(final IJobChangeEvent event) {
								final UIJob uiJob = new UIJob("aktualisiere GUI") {

									@Override
									public IStatus runInUIThread(final IProgressMonitor monitor) {
										return Status.OK_STATUS;
									}
								};
								uiJob.setUser(false);
								uiJob.setSystem(true);
								uiJob.schedule();
							}

							@Override
							public void done(final IJobChangeEvent event) {
								if (event.getResult().isOK()) {
									multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
											"Aktion Freigabe zur Übernahme beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
									MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);

									updateButtonJobAktiv();
									aktionLadeDatenModell(false, null);
								} else {
									MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
									final UIJob uiJob = new UIJob("aktualisiere GUI") {

										@Override
										public IStatus runInUIThread(final IProgressMonitor monitor) {
											updateButtonKeineSelektion();
											return Status.OK_STATUS;
										}
									};
									uiJob.setUser(false);
									uiJob.setSystem(true);
									uiJob.schedule();
								}

							}
						});
						job.setSystem(false);
						job.setUser(true);
						job.schedule();

					} else {
						selChangedlistener
								.selectionChanged(new SelectionChangedEvent(tvZufluesse1, tvZufluesse1.getSelection()));
					}
				}
			}
		}
	}

	private void aktionExport() {
		if (globalSelection != null) {
			final Object obj = globalSelection.getFirstElement();
			if (obj instanceof KonfigBereichDaten) {
				final Iterator<?> it = globalSelection.iterator();

				final Map<String, File> list = new HashMap<>();

				while (it.hasNext()) {
					final KonfigBereichDaten kbd = (KonfigBereichDaten) it.next();
					final String pid = kbd.getPid();
					final Map<String, ConfigurationArea> map = config.getAllConfigurationAreas();
					if (map.containsKey(pid)) {
						final ConfigurationArea ca = map.get(pid);
						list.put(ca.getPid(), new File(""));
					}

				}
				if (list.size() > 0) {
					final KbAuswahlAssistent wizard = new KbAuswahlAssistent("Export", "exportiert", list);

					wizard.init(PlatformUI.getWorkbench(), null);
					// Instantiates the wizard container with the wizard and
					// opens it
					final WizardDialog dialog = new WizardDialog(
							PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
					dialog.create();
					final int ret = dialog.open();

					if (ret == 0) {
						multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Aktion Export", null);

						final KbAuswahlModell model = wizard.getModel();
						final Map<String, File> list2 = model.getList();

						final Job job = new Job("KonfigAss: Export") {

							@Override
							protected IStatus run(final IProgressMonitor monitor) {
								monitor.beginTask("Starte Export...", 5);
								monitor.worked(1);

								try {

									if (list2.size() > 0) {
										final List<String> lokaleListe = new ArrayList<>();

										for (final String s : list2.keySet()) {
											multiStatus.merge(
													new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID, "Export für KB: " + s));
											lokaleListe.add(s);
										}
										File file = new File(dateiPfad);

										final String configPfad = file.getParent();
										file = new File(configPfad);
										final String pfad = file.getParent();

										final String xmlPfad = pfad + "/xml";
										file = new File(xmlPfad);

										if (!file.exists()) {
											file.mkdir();
										}

										if (file.isDirectory()) {
											final KonfigApplikation app = new KonfigApplikation(config);
											app.startExport(file, lokaleListe);
											monitor.worked(1);
										}
									}

								} catch (final Exception e2) {
									multiStatus.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
											"Aktion Export abgebrochen! Grund: es ist eine Ausnahme bei der Ausführung aufgetreten. "
													+ e2));
									return Status.CANCEL_STATUS;
								} finally {
									monitor.worked(3);
									monitor.done();
								}

								return Status.OK_STATUS;
							}
						};

						job.addJobChangeListener(new JobChangeAdapter() {

							@Override
							public void running(final IJobChangeEvent event) {
								final UIJob uiJob = new UIJob("aktualisiere GUI") {

									@Override
									public IStatus runInUIThread(final IProgressMonitor monitor) {
										return Status.OK_STATUS;
									}
								};
								uiJob.setUser(false);
								uiJob.setSystem(true);
								uiJob.schedule();
							}

							@Override
							public void done(final IJobChangeEvent event) {
								if (event.getResult().isOK()) {
									multiStatus.merge(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
											"Aktion Export beendet, bitte beachten Sie die Ausgaben der Konfiguration"));
									MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);

									Display.getDefault().asyncExec(new Runnable() {

										@Override
										public void run() {
											selChangedlistener.selectionChanged(new SelectionChangedEvent(tvZufluesse1,
													tvZufluesse1.getSelection()));
										}
									});

								} else {
									MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
									final UIJob uiJob = new UIJob("aktualisiere GUI") {

										@Override
										public IStatus runInUIThread(final IProgressMonitor monitor) {
											updateButtonKeineSelektion();
											return Status.OK_STATUS;
										}
									};
									uiJob.setUser(false);
									uiJob.setSystem(true);
									uiJob.schedule();
								}

							}
						});
						job.setSystem(false);
						job.setUser(true);
						job.schedule();

					} else {
						selChangedlistener
								.selectionChanged(new SelectionChangedEvent(tvZufluesse1, tvZufluesse1.getSelection()));
					}
				}
			}
		}
	}

	private void aktionEntladeDatenModell() {
		if (config != null) {
			final String meldung = KonfigurationsAssistent.PLUGIN_ID + " Zugriff auf das bisherige Datenmodell "
					+ "ist nicht mehr möglich, da das Datenmodell entladen wird!";

			final MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(),
					SWT.OK | SWT.CANCEL | SWT.ICON_WARNING);
			msg.setText("Das Datenmodell wird entladen");
			msg.setMessage(meldung);
			final int open = msg.open();

			if (open == SWT.OK) {

				config.close();
				config = null;

				erzeugeInhaltTabelle(new ArrayList<KonfigBereichDaten>());

				updateTextKonfiguration("", "");

				updateButtonKeinDatenModell();

				setzeKonfigDatenModellFuerTka(null, dateiPfad);
			}

		}
	}

	private void updateButtonJobAktiv() {
		final UIJob uiJob = new UIJob("aktualisiere GUI") {

			@Override
			public IStatus runInUIThread(final IProgressMonitor monitor) {
				buttonLadeKonfiguration.setEnabled(false);
				buttonEntladeKonfiguration.setEnabled(false);

				buttonNeuerKvAoe.setEnabled(false);

				buttonKbNeu.setEnabled(false);
				buttonKbHinzufuegen.setEnabled(false);
				buttonKbEntfernen.setEnabled(false);

				buttonImport.setEnabled(false);
				buttonKonsistenzPruefung.setEnabled(false);
				buttonUebepruefeAktivierung.setEnabled(false);
				buttonAktivierung.setEnabled(false);

				buttonFreigabeZurUebernahme.setEnabled(false);
				buttonFreigabeZurAktivierung.setEnabled(false);
				buttonFreigabeZurAktivierungOhne.setEnabled(false);

				buttonExport.setEnabled(false);

				return Status.OK_STATUS;
			}
		};
		uiJob.setUser(false);
		uiJob.setSystem(true);
		uiJob.schedule();
	}

	private void updateButtonSelektion(final boolean selektionMitRechte) {
		buttonLadeKonfiguration.setEnabled(true);
		buttonEntladeKonfiguration.setEnabled(true);

		buttonNeuerKvAoe.setEnabled(false);

		buttonKbNeu.setEnabled(true);
		buttonKbHinzufuegen.setEnabled(true);
		buttonKbEntfernen.setEnabled(true);

		buttonImport.setEnabled(true);
		buttonKonsistenzPruefung.setEnabled(true);
		buttonUebepruefeAktivierung.setEnabled(true);
		buttonAktivierung.setEnabled(true);

		if (selektionMitRechte) {
			final Iterator<?> it = globalSelection.iterator();

			final Map<String, ConfigurationArea> areas = config.getAllConfigurationAreas();

			boolean selektionAktiviert = true;
			while (it.hasNext()) {
				final KonfigBereichDaten kbd = (KonfigBereichDaten) it.next();
				final String pid = kbd.getPid();
				if (areas.containsKey(pid)) {
					final ConfigurationArea ca = areas.get(pid);
					if (!ca.isValid()) {
						selektionAktiviert = false;
					}
				}
			}

			buttonFreigabeZurAktivierung.setEnabled(selektionAktiviert);
			buttonFreigabeZurUebernahme.setEnabled(true);
			buttonFreigabeZurAktivierungOhne.setEnabled(true);
		} else {
			buttonFreigabeZurAktivierung.setEnabled(false);
			buttonFreigabeZurUebernahme.setEnabled(false);
			buttonFreigabeZurAktivierungOhne.setEnabled(false);
		}

		buttonExport.setEnabled(true);
	}

	private void updateButtonKeineSelektion() {
		buttonLadeKonfiguration.setEnabled(true);
		buttonEntladeKonfiguration.setEnabled(true);

		buttonNeuerKvAoe.setEnabled(false);

		buttonKbNeu.setEnabled(true);
		buttonKbHinzufuegen.setEnabled(true);
		buttonKbEntfernen.setEnabled(false);

		buttonImport.setEnabled(true);
		buttonKonsistenzPruefung.setEnabled(true);
		buttonUebepruefeAktivierung.setEnabled(true);
		buttonAktivierung.setEnabled(false);

		buttonFreigabeZurUebernahme.setEnabled(false);
		buttonFreigabeZurAktivierung.setEnabled(false);
		buttonFreigabeZurAktivierungOhne.setEnabled(false);

		buttonExport.setEnabled(false);
	}

	private void updateButtonKeinDatenModell() {
		buttonLadeKonfiguration.setEnabled(true);
		buttonEntladeKonfiguration.setEnabled(false);

		buttonNeuerKvAoe.setEnabled(true);

		buttonKbNeu.setEnabled(false);
		buttonKbHinzufuegen.setEnabled(false);
		buttonKbEntfernen.setEnabled(false);

		buttonImport.setEnabled(false);
		buttonKonsistenzPruefung.setEnabled(false);
		buttonUebepruefeAktivierung.setEnabled(false);
		buttonAktivierung.setEnabled(false);

		buttonFreigabeZurUebernahme.setEnabled(false);
		buttonFreigabeZurAktivierung.setEnabled(false);
		buttonFreigabeZurAktivierungOhne.setEnabled(false);

		buttonExport.setEnabled(false);
	}

	private void updateTextKonfiguration(final String pfad, final String pidAoe) {
		if ((text11 != null) && (pfad != null)) {
			text11.setText(pfad);
		}

		if ((text21 != null) && (pidAoe != null)) {
			text21.setText(pidAoe);
		}
	}

	public String getPfad() {
		return dateiPfad;
	}

	List<String> getZugeordnetePluginIds() {
		return zugeordnetePluginIds;
	}

	String getDateiPfad() {
		return dateiPfad;
	}
}
