/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import org.eclipse.core.runtime.IProgressMonitor;

public final class KonfigAssDateiOperationen {

	private KonfigAssDateiOperationen() {
		// es soll keine Instanzen der Klasse geben
	}

	public static boolean loescheVerzeichnis(final File dir, final IProgressMonitor monitor, final int cnt) {
		if (dir.isDirectory()) {
			final String[] entries = dir.list();

			monitor.beginTask("Lösche Ordnerstruktur", cnt);

			if (entries != null) {
				for (int x = 0; x < entries.length; x++) {
					monitor.worked(x);

					final File aktFile = new File(dir.getPath(), entries[x]);
					KonfigAssDateiOperationen.loescheVerzeichnis(aktFile, monitor, x);
				}
			}

			if (dir.delete()) {
				return true;
			}
			return false;
		}
		monitor.worked(cnt);

		if (dir.delete()) {
			return true;
		}
		return false;
	}

	public static int copyDir(final File quelle, final File ziel, final IProgressMonitor monitor,
			final Integer initialCount, final Integer max, final boolean scharf) throws IOException {

		int cnt = initialCount;

		if (scharf) {
			monitor.beginTask("Erzeuge Ordnerstruktur", max);
			ziel.mkdirs();
		}

		final File[] files = quelle.listFiles();
		if (files != null) {
			for (int i = 0; i < files.length; i++) {
				final File file = files[i];

				final String fileName = file.getName();
				if (file.isDirectory()) {
					if ("config".equals(fileName) || "xml".equals(fileName)) {

						if (scharf) {
							monitor.worked(i);
						}

						cnt = KonfigAssDateiOperationen.copyDir(file,
								new File(ziel.getAbsolutePath() + System.getProperty("file.separator") + fileName),
								monitor, cnt, max, scharf);
						cnt++;

					}
				} else {
					if (fileName.endsWith("verwaltungsdaten.xml")) {
						if (scharf) {
							monitor.worked(i);
							KonfigAssDateiOperationen.copyFile(file,
									new File(ziel.getAbsolutePath() + System.getProperty("file.separator") + fileName));
						}

						cnt++;

					}
				}
			}
		}
		return cnt;
	}

	private static void copyFile(final File file, final File ziel) throws IOException {
		try (BufferedInputStream in = new BufferedInputStream(new FileInputStream(file));
				BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(ziel, true))) {
			int bytes = in.read();
			while (bytes != -1) {
				out.write(bytes);
				bytes = in.read();
			}
		}
	}

}
