/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISaveablePart;
import org.eclipse.ui.ISaveablePart2;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Nach Auswahl des Menüpunkts "Betriebsmeldungen..." in der Rahmenapplikation
 * muss sich der Dialog zur Anzeige der Betriebsmeldungen öffnen.
 *
 * <br>
 * <br>
 * Die Klasse erfüllt die technische Anforderungen des Segments Bedienung und
 * Visualisierung (BUV) Modul Konfigurationsassistent: <br>
 * <ul>
 * <li>TAnf-ÜVi-BW-C3-8</li>
 * </ul>
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, Sans (Dambach)
 *
 */
public class KonfigAssSicht extends ViewPart implements ISaveablePart2 {

	private static final Debug LOGGER = Debug.getLogger();

	public static final String ID = "de.bsvrz.buv.plugin.konfigass.konfigassView";

	private CTabFolder tabFolder;

	private String secondViewId;

	private boolean einmal;

	private KonfigAssDarstellungKonfiguration darstellungKonfigurationComposite;

	private KonfigAssDarstellungAssistenten darstellungAssistentenComposite;

	private boolean meldungDatenmodellEntladen = true;

	private IPartListener iPartListener;

	private IWorkbenchListener iWorkbenchListener;

	@Override
	public void createPartControl(final Composite parent) {

		// Für die F1 Taste
		// PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent,
		// Activator.PLUGIN_ID + ".auswahl" );
		PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, "de.bsvrz.buv.plugin.konfigass" + ".konfigass");

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		final Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayout(gridLayout);
		composite.setLayoutData(gridData);
		erzeugeCompositeTabFolder(composite);

		erzeugeTabItems();

		iPartListener = new IPartListener() {

			@Override
			public void partActivated(final IWorkbenchPart part) {
				erzeugeTabItemVonContributor(part.getSite().getPluginId());
			}

			@Override
			public void partBroughtToTop(final IWorkbenchPart part) {
				// wird nicht betrachtet
			}

			@Override
			public void partClosed(final IWorkbenchPart part) {
				// _config = _darstellungKonfigurationComposite.getConfig();
				if (part instanceof KonfigAssSicht) {
					final String sec = ((KonfigAssSicht) part).getViewSite().getSecondaryId();

					if (((sec != null) && sec.equals(secondViewId)) || ((sec == null) && (secondViewId == null))) {
						// String pfadKa =
						// ((KonfigAssSicht)part).getDarstellungKonfigurationComposite()._dateiPfad;
						final ConfigDataModel config = ((KonfigAssSicht) part).getDarstellungKonfigurationComposite()
								.getConfig();

						if (config != null) {
							if (meldungDatenmodellEntladen
									&& !getDarstellungKonfigurationComposite().getZugeordnetePluginIds().isEmpty()) {
								final String meldung = KonfigurationsAssistent.PLUGIN_ID
										+ " : Zugriff der TKA auf das bisherige Datenmodell "
										+ "ist nicht mehr möglich, da das Datenmodell entladen wird!";

								final MessageBox msg = new MessageBox(
										PlatformUI.getWorkbench().getDisplay().getActiveShell(),
										SWT.OK | SWT.ICON_WARNING);
								msg.setText("Das Datenmodell wird entladen");
								msg.setMessage(meldung);
								msg.open();
							}

							config.close();
							getDarstellungKonfigurationComposite().setzeKonfigDatenModellFuerTka(null, null);
							if (!((KonfigAssSicht) part).getDarstellungKonfigurationComposite().isDisposed()) {
								((KonfigAssSicht) part).getDarstellungKonfigurationComposite().dispose();
							}
						}
					}
				} else if (part instanceof IViewPart) {
					final IViewPart vp = (IViewPart) part;
					final IViewSite vs = vp.getViewSite();

					if ("de.bsvrz.buv.plugin.tkabasis.tkaBasisView".equals(vs.getId())) {
						final String sec = vs.getSecondaryId();

						if (sec != null) {
							final IWorkbench wb = PlatformUI.getWorkbench();
							final IWorkbenchWindow ww = wb.getActiveWorkbenchWindow();
							final IWorkbenchPage page = ww.getActivePage();

							for (final IViewReference ref : page.getViewReferences()) {
								if (ref.getId().equals(KonfigAssSicht.ID)) {
									final KonfigAssSicht view = (KonfigAssSicht) ref.getView(false);

									if (view != null) {
										final TableViewer tv = view.getDarstellungAssisitentenComposite()
												.holeTableViewerTeilKonfigurationsAssistenten();
										final TableItem[] items = tv.getTable().getItems();

										final String ss = sec.toLowerCase();
										final String id = "de.bsvrz.buv.plugin.tka" + ss;

										boolean oeffnenMoeglich = false;

										for (final TableItem ti : items) {
											final String tiId = (String) ti.getData("id");

											if (id.equals(tiId)) {
												ti.setChecked(false);
												removeZugeordnetePluginId(id);
												view.getDarstellungKonfigurationComposite()
														.setzeKonfigDatenModellFuerTka(id, null, null);
											}

											if (ti.getChecked()) {
												oeffnenMoeglich = true;
											}
										}

										final Button btn = view.getDarstellungAssisitentenComposite()
												.getButtonOeffnen();

										if ((btn != null) && !btn.isDisposed()) {
											if (oeffnenMoeglich) {
												btn.setEnabled(true);
											} else {
												btn.setEnabled(false);
											}
										}
									}
								}
							}
						}
					}
				}
			}

			@Override
			public void partDeactivated(final IWorkbenchPart part) {
				// wird nicht betrachtet
			}

			@Override
			public void partOpened(final IWorkbenchPart part) {
				// wird nicht betrachtet
			}

		};

		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService().addPartListener(iPartListener);

		iWorkbenchListener = new IWorkbenchListener() {

			@Override
			public boolean preShutdown(final IWorkbench workbench, final boolean forced) {
				meldungDatenmodellEntladen = false;

				return true;
			}

			@Override
			public void postShutdown(final IWorkbench workbench) {
				// wird nicht betrachtet
			}
		};

		PlatformUI.getWorkbench().addWorkbenchListener(iWorkbenchListener);

	}

	@Override
	public void setFocus() {
		if ((tabFolder != null) && (!tabFolder.isDisposed())) {
			tabFolder.setFocus();
		}
	}

	@Override
	public void dispose() {
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService().removePartListener(iPartListener);
		PlatformUI.getWorkbench().removeWorkbenchListener(iWorkbenchListener);
	}

	/**
	 * This method initializes composite
	 *
	 */
	private void erzeugeCompositeTabFolder(final Composite parent) {
		final GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.grabExcessVerticalSpace = true;
		gridData3.verticalAlignment = GridData.FILL;

		tabFolder = new CTabFolder(parent, SWT.NONE);
		tabFolder.setLayout(new GridLayout());
		tabFolder.setLayoutData(gridData3);
		tabFolder.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				final CTabFolder folder = (CTabFolder) e.getSource();
				aktualisiereTabelle(folder);
			}
		});
	}

	private void erzeugeTabItems() {
		final CTabItem tabItem = new CTabItem(tabFolder, SWT.NONE);
		tabItem.setText("Konfiguration");

		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;

		// Composite c1 = new Composite(_tabFolder, SWT.NONE);
		final ScrolledComposite c1 = new ScrolledComposite(tabFolder, SWT.H_SCROLL | SWT.V_SCROLL);
		c1.setLayout(new GridLayout());
		c1.setLayoutData(gridData2);

		darstellungKonfigurationComposite = new KonfigAssDarstellungKonfiguration(c1, SWT.NONE);

		// _config = _darstellungKonfigurationComposite.getConfig();

		tabItem.setControl(c1);

		c1.setContent(darstellungKonfigurationComposite);
		darstellungKonfigurationComposite.layout();
		c1.setExpandHorizontal(true);
		c1.setExpandVertical(true);
		c1.setMinSize(darstellungKonfigurationComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		darstellungKonfigurationComposite.layout();

		final CTabItem tabItem2 = new CTabItem(tabFolder, SWT.NONE);
		tabItem2.setText("Assistenten");

		final GridData gridData22 = new GridData();
		gridData22.horizontalAlignment = GridData.FILL;
		gridData22.grabExcessHorizontalSpace = true;
		gridData22.grabExcessVerticalSpace = true;
		gridData22.verticalAlignment = GridData.FILL;

		// Composite c12 = new Composite(_tabFolder, SWT.NONE);
		final ScrolledComposite c12 = new ScrolledComposite(tabFolder, SWT.H_SCROLL | SWT.V_SCROLL);
		c12.setLayout(new GridLayout());
		c12.setLayoutData(gridData22);

		darstellungAssistentenComposite = new KonfigAssDarstellungAssistenten(c12, SWT.NONE, secondViewId);
		tabItem2.setControl(c12);

		c12.setContent(darstellungAssistentenComposite);
		darstellungAssistentenComposite.layout();
		c12.setExpandHorizontal(true);
		c12.setExpandVertical(true);
		c12.setMinSize(darstellungAssistentenComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		darstellungAssistentenComposite.layout();

		tabFolder.setSelection(0);

	}

	@Override
	public void init(final IViewSite site, final IMemento memento) throws PartInitException {

		secondViewId = site.getSecondaryId();
		if (memento != null) {
			secondViewId = memento.getString("id");
		}

		super.init(site, memento);
	}

	@Override
	public void saveState(final IMemento memento) {
		memento.putString("id", secondViewId);
		super.saveState(memento);
	}

	/**
	 * @param folder
	 */
	private void aktualisiereTabelle(final CTabFolder folder) {
		final CTabItem item = folder.getSelection();

		final Composite c = (Composite) item.getControl();

		if (c != null) {
			if (c.getChildren() != null) {
				final Control control = c.getChildren()[0];

				if (control instanceof KonfigAssDarstellungKonfiguration) {
					darstellungKonfigurationComposite = (KonfigAssDarstellungKonfiguration) control;
				} else if (control instanceof KonfigAssDarstellungAssistenten) {
					darstellungAssistentenComposite = (KonfigAssDarstellungAssistenten) control;

					final List<IKonfigAssDatenModell> listeTKA = darstellungAssistentenComposite
							.holeAngemeldeteAssistenten();

					darstellungAssistentenComposite.setInputTabelle(listeTKA);

				}
			}
		}
	}

	public KonfigAssDarstellungKonfiguration getDarstellungKonfigurationComposite() {
		return darstellungKonfigurationComposite;
	}

	public KonfigAssDarstellungAssistenten getDarstellungAssisitentenComposite() {
		return darstellungAssistentenComposite;
	}

	private void erzeugeTabItemVonContributor(final String pluginId) {
		final IConfigurationElement[] configurationElementsFor = Platform.getExtensionRegistry()
				.getConfigurationElementsFor("de.bsvrz.buv.plugin.konfigass.konfigAssComposite");

		for (final IConfigurationElement ce : configurationElementsFor) {
			try {
				final IKonfigAssComposite operation = (IKonfigAssComposite) ce
						.createExecutableExtension("konfigAssCompositeKlasse");

				if (operation.holePluginId().equals(pluginId)) {
					if (!einmal) {

						final CTabItem tabItem = new CTabItem(tabFolder, SWT.NONE);
						tabItem.setText(operation.holeName());

						final GridData gridData2 = new GridData();
						gridData2.horizontalAlignment = GridData.FILL;
						gridData2.grabExcessHorizontalSpace = true;
						gridData2.grabExcessVerticalSpace = true;
						gridData2.verticalAlignment = GridData.FILL;

						final Composite c1 = new Composite(tabFolder, SWT.NONE);
						c1.setLayout(new GridLayout());
						c1.setLayoutData(gridData2);

						final Composite c = operation.holeContributorComposite(c1);
						tabItem.setControl(c);

						einmal = true;
					}
				}
			} catch (final CoreException e) {
				KonfigAssSicht.LOGGER.error(e.getLocalizedMessage(), e);
			}
		}
	}

	public boolean isMeldungDatenmodellEnladen() {
		return meldungDatenmodellEntladen;
	}

	public void setMeldungDatenmodellEnladen(final boolean meldungDatenmodellEnladen) {
		meldungDatenmodellEntladen = meldungDatenmodellEnladen;
	}

	@Override
	public void doSave(final IProgressMonitor monitor) {
		// wird nicht unterstützt
	}

	@Override
	public void doSaveAs() {
		// wird nicht unterstützt
	}

	@Override
	public boolean isDirty() {
		return !getSichtenZuPlugins().isEmpty();
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public boolean isSaveOnCloseNeeded() {
		return true;
	}

	@Override
	public int promptToSaveOnClose() {
		Map<String, IViewReference> tkaSichten = getSichtenZuPlugins();

		// Zuerst alle dirty Views schliessen
		for (final IViewReference ref : tkaSichten.values()) {

			if ((ref != null) && ref.isDirty()) {
				final IWorkbenchPage page = ref.getPage();
				final IWorkbenchPart part = ref.getPart(false);
				page.hideView(ref);
				if (page.isPartVisible(part)) {
					return ISaveablePart2.CANCEL;
				}
			}
		}

		// tkaSichten aktualisieren - enthält nur noch nicht-dirty View
		tkaSichten = getSichtenZuPlugins();

		// Alle nicht-dirty Views schliessen
		for (final IViewReference ref : tkaSichten.values()) {
			if (ref != null) {
				ref.getPage().hideView(ref);
			}
		}

		return ISaveablePart2.NO;
	}

	private Map<String, IViewReference> getSichtenZuPlugins() {
		final Map<String, IViewReference> result = new HashMap<>();

		final IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		if (dw != null) {
			final IWorkbenchPage page = dw.getActivePage();

			for (final String pluginId : getDarstellungKonfigurationComposite().getZugeordnetePluginIds()) {
				for (final IViewReference ref : page.getViewReferences()) {
					if (ref != null) {
						if ("de.bsvrz.buv.plugin.tkabasis.tkaBasisView".equals(ref.getId())) {
							if (pluginId.startsWith("de.bsvrz.buv.plugin.tka")) {
								final String secId = pluginId.substring(pluginId.indexOf("tka") + 3);
								if ((ref.getSecondaryId() != null)
										&& ref.getSecondaryId().toLowerCase().equals(secId)) {
									result.put(pluginId, ref);
								}
							}
						}
					}
				}
			}
		}

		return result;
	}

	void addZugeordnetePluginId(final String pluginId) {
		final boolean dirty = isDirty();

		if (getDarstellungKonfigurationComposite().getZugeordnetePluginIds().add(pluginId)) {
			if (isDirty() != dirty) {
				firePropertyChange(ISaveablePart.PROP_DIRTY);
			}
		}
	}

	void removeZugeordnetePluginId(final String pluginId) {
		final boolean dirty = isDirty();

		if (getDarstellungKonfigurationComposite().getZugeordnetePluginIds().remove(pluginId)) {
			if (isDirty() != dirty) {
				firePropertyChange(ISaveablePart.PROP_DIRTY);
			}
		}
	}

	static KonfigAssSicht getView(final String secondViewId) {
		// final KonfigAssSicht result = null;

		final IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		final IWorkbenchPage page = dw.getActivePage();

		// FIXME Eclipse-Bug: 420140
		// final IViewReference ref = page.findViewReference(KonfigAssSicht.ID,
		// secondViewId);
		//
		// if (ref != null) {
		// result = (KonfigAssSicht) ref.getView(false);
		// }

		try {
			final IViewPart view = page.showView(KonfigAssSicht.ID, secondViewId, IWorkbenchPage.VIEW_VISIBLE);
			if (view instanceof KonfigAssSicht) {
				return (KonfigAssSicht) view;
			}
		} catch (final PartInitException e) {
			e.printStackTrace();
		}

		return null;
	}

} // @jve:decl-index=0:visual-constraint="10,10,847,295"
