/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.management.ConfigAreaAndVersion;
import de.bsvrz.dav.daf.main.config.management.consistenycheck.ConsistencyCheckResultEntry;
import de.bsvrz.dav.daf.main.config.management.consistenycheck.ConsistencyCheckResultInterface;
import de.bsvrz.puk.config.configFile.datamodel.AreaDependencyCheck.AreaDependencyCheckResult;
import de.bsvrz.puk.config.configFile.datamodel.ConfigurationAreaDependency;

public class KonfigAssistentErgebnis {

	private AreaDependencyCheckResult areaDependencyCheckResult;
	private ConsistencyCheckResultInterface consistencyCheckResult;
	private final List<ConfigAreaAndVersion> betrachteteBereiche = new ArrayList<>();

	public AreaDependencyCheckResult getAreaDependencyCheckResult() {
		return areaDependencyCheckResult;
	}

	public void setAreaDependencyCheckResult(AreaDependencyCheckResult areaDependencyCheckResult) {
		this.areaDependencyCheckResult = areaDependencyCheckResult;
	}

	public ConsistencyCheckResultInterface getConsistencyCheckResult() {
		return consistencyCheckResult;
	}

	public void setConsistencyCheckResult(ConsistencyCheckResultInterface consistencyCheckResult) {
		this.consistencyCheckResult = consistencyCheckResult;
	}

	public void ausgebenInMultiStatus(MultiStatus multiStatus) {
		betrachteteBereicheAusgeben(multiStatus);
		areaDependencyCheckResultAusgeben(multiStatus);
		consistencyCheckResultAusgeben(multiStatus);
	}

	private void betrachteteBereicheAusgeben(MultiStatus multiStatus) {
		if( betrachteteBereiche.isEmpty()) {
			return;
		}

		MultiStatus status = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.INFO,
				betrachteteBereiche.size() + " Konfigurationsbereiche betrachtet", null);
		multiStatus.add(status);
		for( ConfigAreaAndVersion bereich : betrachteteBereiche) {
			status.add(new Status(IStatus.INFO, KonfigurationsAssistent.PLUGIN_ID, bereich.getConfigArea() + " in Version " + bereich.getVersion()));
		}
	}

	private void areaDependencyCheckResultAusgeben(MultiStatus multiStatus) {
		if( areaDependencyCheckResult == null) {
			return;
		}
		
		MultiStatus status = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.WARNING,
				areaDependencyCheckResult.getAreasWithUnknownDependencies().size() + " Konfigurationsbereiche, die nicht geprüft werden konnten", null);
		multiStatus.add(status);
		for( ConfigurationArea warnung : areaDependencyCheckResult.getAreasWithUnknownDependencies()) {
			status.add(new Status(IStatus.WARNING, KonfigurationsAssistent.PLUGIN_ID, warnung.toString()));
		}

		status = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.WARNING,
				areaDependencyCheckResult.getNeededDependencyErrors().size() + " Konfigurationsbereiche, die nicht in der geforderten Version vorliegen", null);
		multiStatus.add(status);
		for( Entry<ConfigurationArea, List<ConfigurationAreaDependency>> entry : areaDependencyCheckResult.getNeededDependencyErrors().entrySet()) {
			status.add(new Status(IStatus.WARNING, KonfigurationsAssistent.PLUGIN_ID, entry.toString()));
		}

		status = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.WARNING,
				areaDependencyCheckResult.getOptionalDependencyErrors().size() + " Optionale Konfigurationsbereiche, die nicht in der geforderten Version vorliegen", null);
		multiStatus.add(status);
		for( Entry<ConfigurationArea, List<ConfigurationAreaDependency>> entry : areaDependencyCheckResult.getOptionalDependencyErrors().entrySet()) {
			status.add(new Status(IStatus.WARNING, KonfigurationsAssistent.PLUGIN_ID, entry.toString()));
		}
	}

	private void consistencyCheckResultAusgeben(MultiStatus multiStatus) {
		
		if( consistencyCheckResult == null) {
			return;
		}
		
		MultiStatus status = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.WARNING,
				consistencyCheckResult.getWarnings().size() + " Warnungen", null);
		multiStatus.add(status);
		for( ConsistencyCheckResultEntry warnung : consistencyCheckResult.getWarnings()) {
			status.add(new Status(IStatus.WARNING, KonfigurationsAssistent.PLUGIN_ID, warnung.toString()));
		}
		
		status = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.ERROR,
				consistencyCheckResult.getLocalErrors().size() + " Lokale Fehler", null);
		multiStatus.add(status);
		for( ConsistencyCheckResultEntry warnung : consistencyCheckResult.getLocalErrors()) {
			status.add(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID, warnung.toString()));
		}

		status = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.ERROR,
				consistencyCheckResult.getInterferenceErrors().size() + " Interferenzfehler", null);
		multiStatus.add(status);
		for( ConsistencyCheckResultEntry warnung : consistencyCheckResult.getInterferenceErrors()) {
			status.add(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID, warnung.toString()));
		}
	}

	public void setBetrachteteBereiche(List<ConfigAreaAndVersion> betrachteteBereiche) {
		this.betrachteteBereiche.clear();
		this.betrachteteBereiche.addAll(betrachteteBereiche);
	}
}
