/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Klasse zur Ermittlung der Information, ob ein Konfigurationsbereich Modell-
 * und/oder Objekt-Definitionen besitzt. Die Bestimmung erfolgt ueber das
 * Datenmodell und den zum Konfigurationsbereich gehoerenden Objekte.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, Pittner (Dambach)
 *
 */
public class KonfigBereichBestimmeModellObjekt {
	/**
	 * Das Datenmodell
	 */
	private final DataModel dataModel;

	/**
	 * zu pruefender Konfigurationsbereich
	 */
	private final ConfigurationArea configArea;

	/**
	 * Liste von Pids aller Typen und Mengen des Meta-Modells
	 */
	private static final String[] PIDS_META_MODELL_TYPEN = new String[] { "typ.typ", "typ.attributListenDefinition",
			"typ.attributTyp", "typ.attributgruppe", "typ.attribut", "typ.attributgruppenVerwendung",
			"typ.werteZustand", "typ.mengenVerwendung", "typ.werteBereich", "typ.aspekt", "menge.attribute",
			"menge.mengenVerwendungen", "menge.attributgruppenVerwendungen", "menge.objektTypen",
			"menge.attributgruppen", "menge.werteZustaende" };

	/**
	 * Flagge, die anzeigt ob der Konfigurationsbereich Modell-Definitionen hat
	 */
	private Boolean isModell = false;

	/**
	 * Flagge, die anzeigt ob der Konfigurationsbereich Objekt-Definitionen hat
	 */
	private Boolean isObjekt = false;

	/**
	 * Konstruktor der Klasse. Die uebergebenen Parameter werden auf null geprueft.
	 *
	 * @param dataModel
	 *            das Datenmodell
	 * @param configArea
	 *            der zu pruefende Konfugurationbereich
	 */
	public KonfigBereichBestimmeModellObjekt(final DataModel dataModel, final ConfigurationArea configArea) {
		if ((dataModel != null) && (configArea != null)) {

			this.dataModel = dataModel;
			this.configArea = configArea;

			bestimmeModellObjektAusKonfigBereich();

		} else {
			throw new IllegalArgumentException("Argument ist null");
		}

	}

	/**
	 * Methode die den Konfigbereich auf Modell- sowie Objektdefinitioen prueft.
	 */
	private void bestimmeModellObjektAusKonfigBereich() {
		final List<SystemObjectType> listMetaModelTypes = new ArrayList<>();

		for (final String element : KonfigBereichBestimmeModellObjekt.PIDS_META_MODELL_TYPEN) {
			final SystemObjectType type = dataModel.getType(element);
			listMetaModelTypes.add(type);
		}

		final Collection<SystemObject> currentObjects = configArea.getCurrentObjects();

		final Collection<SystemObject> objectsFromMetaModelTypes = configArea.getObjects(listMetaModelTypes,
				ObjectTimeSpecification.valid());

		if (objectsFromMetaModelTypes.size() > 0) {
			isModell = true;
		}

		if (currentObjects.size() > (objectsFromMetaModelTypes.size() + 1)) {
			isObjekt = true;
		}

	}

	/**
	 * Holt die Info, ob der Konfigurationsbereich Modell-Definitionen besitzt.
	 *
	 * @return Info ob Modell-Definitionen vorhanden
	 */
	public Boolean getIsModell() {
		return isModell;
	}

	/**
	 * Holt die Info, ob der Konfigurationsbereich Objekt-Definitionen besitzt.
	 *
	 * @return Info ob Objekt-Definitionen vorhanden
	 */
	public Boolean getIsObjekt() {
		return isObjekt;
	}

}
