/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.main.dataview.VersionedView;

/**
 * Erzeugen von Instanzen vom Typ <code>ObjectLookup</code>
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, Sans (Dambach)
 *
 */
public final class LookupFactory {

	private LookupFactory() {
		// es soll keine Instanzen der Klasse geben
	}

	/**
	 * Erzeugt eine Instanz von <code>ObjectLookup</code>, die für alle aktuellen
	 * Konfigurationsbereiche als Version die jeweilige ModifiableVersion
	 * spezifiziert.
	 *
	 * @param dataModel
	 *            das Datenmodell der Konfiguration
	 * @return Instanz vom Typ <code>ObjectLookup</code>, die für alle aktuellen
	 *         Konfigurationsbereiche als Version die jeweilige ModifiableVersion
	 *         spezifiziert
	 */
	public static ObjectLookup createLookupForModifiableVersion(final ConfigDataModel dataModel) {
		final Collection<ConfigurationArea> areas = dataModel.getAllConfigurationAreas().values();

		return LookupFactory.createLookupForModifiableVersion(dataModel, areas);
	}

	/**
	 * Erzeugt eine Instanz von <code>ObjectLookup</code>, die für die übergebenen
	 * Konfigurationsbereiche als Version die jeweilige ModifiableVersion
	 * spezifiziert.
	 *
	 * @param dataModel
	 *            das Datenmodell der Konfiguration
	 * @param areas
	 *            Sammlung der zu berücksichtigenden Konfigurationsbereiche
	 * @return Instanz vom Typ <code>ObjectLookup</code>, die für alle übergebenen
	 *         Konfigurationsbereiche als Version die jeweilige ModifiableVersion
	 *         spezifiziert
	 */
	private static ObjectLookup createLookupForModifiableVersion(final ConfigDataModel dataModel,
			final Collection<ConfigurationArea> areas) {
		final Map<ConfigurationArea, Short> konfigVersionen = new HashMap<>();

		for (final ConfigurationArea kb : areas) {
			konfigVersionen.put(kb, kb.getModifiableVersion());
		}

		return new VersionedView(dataModel, konfigVersionen);
	}

}