/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.assistenten;

import java.util.Date;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.rw.basislib.kalender.DatumZeit;
import de.bsvrz.buv.rw.basislib.kalender.DatumZeit.Eingabetyp;

/**
 * Class representing the first page of the wizard
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class KbAnlegenStartSeite extends WizardPage implements ModifyListener, SelectionListener {

	private Text authorText;

	private DatumZeit datumZeit;

	private Text grundText;

	private Text text4;

	private Text textPid;

	private Text textName;

	private final KbAnlegenModell model;

	/**
	 * Constructor for HolidayMainPage.
	 *
	 * @param model
	 */
	public KbAnlegenStartSeite(final KbAnlegenModell model) {
		super("Page1");
		setTitle("KB Neu");
		setDescription("Eingabe der erforderlichen Daten");
		this.model = model;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	@Override
	public void createControl(final Composite parent) {

		final Composite composite = new Composite(parent, SWT.NULL);
		// create the desired layout for this wizard page
		final GridLayout gl = new GridLayout();
		final int ncol = 2;
		gl.numColumns = ncol;
		composite.setLayout(gl);

		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.verticalAlignment = GridData.CENTER;
		final GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.FILL;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.verticalAlignment = GridData.CENTER;
		final GridData gridData5 = new GridData();
		gridData5.horizontalAlignment = GridData.FILL;
		gridData5.grabExcessHorizontalSpace = true;
		gridData5.verticalAlignment = GridData.CENTER;

		final KbAnlegenAssistent wizard = (KbAnlegenAssistent) getWizard();
		final KbAnlegenModell wizardModel = wizard.getModel();

		final Label labelPid = new Label(composite, SWT.NONE);
		labelPid.setText("Pid");
		textPid = new Text(composite, SWT.BORDER);
		textPid.setLayoutData(gridData2);
		textPid.setText(wizardModel.getPid());
		textPid.addModifyListener(this);

		final Label labelName = new Label(composite, SWT.NONE);
		labelName.setText("Name");
		textName = new Text(composite, SWT.BORDER);
		textName.setLayoutData(gridData2);
		textName.setText(wizardModel.getName());
		textName.addModifyListener(this);

		final Label label = new Label(composite, SWT.NONE);
		label.setText("Stand");
		datumZeit = new DatumZeit(composite, SWT.BORDER, Eingabetyp.datumuhr, new Date(), false, false);
		datumZeit.setLayoutData(gridData1);
		datumZeit.hinzufuegenSelektionsZuhoerer(this);
		final Label label1 = new Label(composite, SWT.NONE);
		label1.setText("Autor");
		authorText = new Text(composite, SWT.BORDER);
		authorText.setLayoutData(gridData2);
		authorText.addModifyListener(this);

		final Label label3 = new Label(composite, SWT.NONE);
		label3.setText("Grund");
		grundText = new Text(composite, SWT.BORDER);
		grundText.setLayoutData(gridData4);
		grundText.addModifyListener(this);

		final Label label4 = new Label(composite, SWT.NONE);
		label4.setText("Text");
		text4 = new Text(composite, SWT.BORDER);
		text4.setLayoutData(gridData5);
		text4.addModifyListener(this);

		createLine(composite, ncol);

		// set the composite as the control for this page
		setControl(composite);
		setPageComplete(false);
	}

	@Override
	public boolean isPageComplete() {
		boolean result = true;
		result = result && KbAnlegenStartSeite.isTextNonEmpty(textPid);
		result = result && KbAnlegenStartSeite.isTextNonEmpty(textName);
		result = result && KbAnlegenStartSeite.isTextNonEmpty(authorText);
		result = result && KbAnlegenStartSeite.isTextNonEmpty(grundText);
		return result;
	}

	/*
	 * Saves the uses choices from this page to the model. Called on exit of the
	 * page
	 */
	private void saveDataToModel() {
		final Date d = datumZeit.getDatum();
		if (d != null) {
			model.setStand(d.getTime());
		}
		model.setPid(textPid.getText().trim());
		model.setName(textName.getText().trim());
		model.setAutor(authorText.getText().trim());
		model.setGrund(grundText.getText().trim());
		model.setText(text4.getText().trim());
	}

	private static boolean isTextNonEmpty(final Text t) {
		if (t != null) {
			final String s = t.getText();
			if ((s != null) && (s.trim().length() > 0)) {
				return true;
			}
		}
		return false;
	}

	private void createLine(final Composite parent, final int ncol) {
		final Label line = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = ncol;
		line.setLayoutData(gridData);
	}

	@Override
	public void widgetSelected(final SelectionEvent e) {
		saveDataToModel();
		setPageComplete(isPageComplete());
	}

	@Override
	public void widgetDefaultSelected(final SelectionEvent e) {
		widgetSelected(e);
	}

	@Override
	public void modifyText(final ModifyEvent e) {
		saveDataToModel();
		setPageComplete(isPageComplete());
	}
}
