/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.assistenten;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TableItem;

/**
 * Class representing the first page of the wizard
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 *
 */
public class KbImportStartSeite extends WizardPage implements Listener {

	private Button button;

	private final String dateiPfad;

	private TableViewer tvZufluesse1;

	private final boolean neu;

	/**
	 * Constructor for HolidayMainPage.
	 */
	public KbImportStartSeite(final String dateiPfad, final boolean neu) {
		super("Page1");
		this.neu = neu;

		if (neu) {
			setTitle("Import");
			setDescription("Folgende ausgewählte Versorgungsdateien werden importiert:");
		} else {
			setTitle("KB entfernen");
			setDescription("Folgende ausgewählte Konfigurationsbereiche werden entfernt:");
		}

		this.dateiPfad = dateiPfad;

	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	@Override
	public void createControl(final Composite parent) {

		// create the composite to hold the widgets
		final GridData gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		gd.verticalAlignment = GridData.BEGINNING;
		final Composite composite = new Composite(parent, SWT.NULL);

		composite.setLayout(new GridLayout(2, false));
		composite.setLayoutData(gd);

		tvZufluesse1 = new TableViewer(composite,
				SWT.BORDER | SWT.CHECK | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

		final TableViewerColumn tc1 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc1.getColumn().setWidth(20);
		tc1.getColumn().setText("Auswahl");
		final TableViewerColumn tc2 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc2.getColumn().setWidth(300);
		tc2.getColumn().setText("Menge");

		tvZufluesse1.setContentProvider(new ArrayContentProvider());
		tvZufluesse1.setLabelProvider(new TabelleTeilkonfigurationsassistentenDarstellungsVersorger());

		tvZufluesse1.getTable().setLayout(new FillLayout());
		final GridData gd2 = new GridData(GridData.FILL_BOTH);

		tvZufluesse1.getTable().setLayoutData(gd2);

		if (neu) {
			button = new Button(composite, SWT.NONE);
			button.setText("...");
			button.setLayoutData(new GridData(SWT.TOP, SWT.TOP, false, false));
			button.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(final SelectionEvent e) {
					final FileDialog dialog = new FileDialog(Display.getCurrent().getActiveShell(),
							SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM | SWT.RESIZE | SWT.MULTI | SWT.OPEN);
					dialog.setFilterNames(new String[] { "XML-Dateien" });
					dialog.setFilterExtensions(new String[] { "*.xml" });

					File temp = new File(dateiPfad);
					String path = temp.getParent();
					if (path != null) {
						temp = new File(path + "/../" + "xml");
						if (temp.exists()) {
							path += "/../" + "xml";
						}
						dialog.setFilterPath(path);
					}

					final String selectedDateiPfad = dialog.open();

					if (selectedDateiPfad != null) {
						final File datei = new File(selectedDateiPfad);
						final String folderName = datei.getParent();

						final String[] fileNames = dialog.getFileNames();

						final Map<String, File> map = new HashMap<>();

						for (final String s : fileNames) {
							final String ss = folderName + "/" + s;

							final File file = new File(ss);
							final TableItem ti = new TableItem(tvZufluesse1.getTable(), SWT.NONE);
							ti.setText(1, file.getName());
							ti.setChecked(true);
							map.put(file.getName(), file);
						}

						final KbImportAssistent wizard = (KbImportAssistent) getWizard();

						wizard.getModel().setList(map);
						setPageComplete(true);

					}
				}
			});
		}

		final KbImportAssistent wizard = (KbImportAssistent) getWizard();
		final KbImportModell model = wizard.getModel();

		tvZufluesse1.setInput(model.getList().keySet());

		createLine(composite, 2);

		// set the composite as the control for this page
		setControl(composite);
		addListeners();

		parent.layout(true);
	}

	private void addListeners() {

		tvZufluesse1.getTable().addListener(SWT.Selection, this);
	}

	/**
	 * @see Listener#handleEvent(Event)
	 */
	@Override
	public void handleEvent(final Event event) {
		setPageComplete(isPageComplete());
		getWizard().getContainer().updateButtons();
	}

	/**
	 * @see IWizardPage#canFlipToNextPage()
	 */
	@Override
	public boolean canFlipToNextPage() {
		return false;
	}

	/*
	 * Sets the completed field on the wizard class when all the information is
	 * entered and the wizard can be completed
	 */
	@Override
	public boolean isPageComplete() {
		final TableItem[] items = tvZufluesse1.getTable().getItems();

		// KbEntfernenHinzufuegenAssistent wizard =
		// (KbEntfernenHinzufuegenAssistent)getWizard();

		final KbImportAssistent wizard = (KbImportAssistent) getWizard();

		final Map<String, File> map = new HashMap<>();

		boolean canFinish = false;

		for (final TableItem ti : items) {
			if (ti.getChecked()) {

				final String filname = ti.getText(1);

				final File file = wizard.getModel().getList().get(filname);

				map.put(filname, file);
				canFinish = true;
			}
		}
		if (canFinish) {
			wizard.getModel().setList(map);
		}

		return canFinish;
	}

	private void createLine(final Composite parent, final int ncol) {
		final Label line = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = ncol;
		line.setLayoutData(gridData);
	}

	/**
	 * {@link LabelProvider} für die Tabelle.
	 *
	 * @author HofmannJ
	 *
	 */
	private static class TabelleTeilkonfigurationsassistentenDarstellungsVersorger extends LabelProvider
			implements ITableLabelProvider {

		@Override
		public Image getColumnImage(final Object element, final int columnIndex) {
			return null;
		}

		@Override
		public String getColumnText(final Object element, final int columnIndex) {

			String returnString = null;

			switch (columnIndex) {
			case 0:
				returnString = "";
				break;
			case 1:
				returnString = (String) element;
				break;
			default:
				returnString = "default";
			}
			return returnString;
		}

		@Override
		public boolean isLabelProperty(final Object element, final String property) {
			return false;
		}
	}
}
