/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.assistenten;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

/**
 * Model class containing the data for the holiday wizard
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class KvAoeNeuModell {

	private String pid = "";
	private String name = "";
	private String kodierung = "";
	private String quellVerzeichnis = "";
	private String zielVerzeichnis = "";
	private boolean useKV = true;
	private boolean useAOE;
	private String info = "";
	private String beschreibung = "";
	private String beschreibungDef = "";
	private final Map<String, String> mengen = new HashMap<>();

	private ConfigDataModel config;

	private Thread autoCloser;

	public Thread getAutoCloser() {
		return autoCloser;
	}

	public String getBeschreibung() {
		return beschreibung;
	}

	public String getBeschreibungDef() {
		return beschreibungDef;
	}

	public ConfigDataModel getConfig() {
		return config;
	}

	public String getKodierung() {
		return kodierung;
	}

	public String getKurzInfo() {
		return info;
	}

	public Map<String, String> getMengen() {
		return mengen;
	}

	public String getName() {
		return name;
	}

	public String getPid() {
		return pid;
	}

	public String getQuellVerzeichnis() {
		return quellVerzeichnis;
	}

	public String getZielVerzeichnis() {
		return zielVerzeichnis;
	}

	public boolean isUseAOE() {
		return useAOE;
	}

	public boolean isUseKV() {
		return useKV;
	}

	@Override
	public String toString() {
		final String separator = System.getProperty("line.separator");
		final StringBuilder s = new StringBuilder();
		s.append("KV/AOE neu erstellen:");
		s.append(separator);
		if (useKV) {
			s.append("KV neu: ");
		} else {
			s.append("AOE neu: ");
		}

		if (useKV) {
			s.append(separator);
			s.append("pid: ");
			s.append(pid);
			s.append(separator);
			s.append("name: ");
			s.append(name);
			s.append(separator);
			s.append("ziel: ");

			s.append(zielVerzeichnis);
			s.append(separator);
			s.append("quelle: ");
			s.append(quellVerzeichnis);
			s.append(separator);
			s.append("config: ");
			s.append(separator);
			s.append("info: ");
			s.append(info);
		} else {
			s.append(separator);
			s.append("pid: ");
			s.append(pid);
			s.append(separator);
			s.append("name: ");
			s.append(name);
			s.append(separator);
			s.append("kodierung: ");
			s.append(kodierung);
			s.append(separator);
			s.append("ziel: ");
			s.append(zielVerzeichnis);
			s.append(separator);
			s.append("quelle: ");
			s.append(quellVerzeichnis);
			s.append(separator);
			s.append("Verzeichnis: ");
			s.append(separator);
			s.append("info: ");
			s.append(info);
			s.append(separator);
			s.append("config: ");
			s.append(config);

			for (final Entry<String, String> menge : mengen.entrySet()) {
				s.append(separator);
				s.append(" menge: ");
				s.append(menge.getKey());
				s.append(':');
				s.append(menge.getValue());
			}
		}
		return s.toString();
	}

	void setUseKV(final boolean useKV) {
		this.useKV = useKV;
	}

	public void setUseAOE(final boolean useAOE) {
		this.useAOE = useAOE;
	}

	void setPid(final String pid) {
		this.pid = pid;
	}

	void setZielVerzeichnis(final String zielVerzeichnis) {
		this.zielVerzeichnis = zielVerzeichnis;
	}

	void setQuellVerzeichnis(final String quellVerzeichnis) {
		this.quellVerzeichnis = quellVerzeichnis;
	}

	void setInfo(final String info) {
		this.info = info;
	}

	void setBeschreibung(final String beschreibung) {
		this.beschreibung = beschreibung;
	}

	void setName(final String name) {
		this.name = name;
	}

	void setKodierung(final String kodierung) {
		this.kodierung = kodierung;
	}

	void setConfig(final ConfigDataModel config) {
		this.config = config;
	}

	void setBeschreibungDef(final String beschreibungDef) {
		this.beschreibungDef = beschreibungDef;
	}

	void setAutoCloser(final Thread autoCloser) {
		this.autoCloser = autoCloser;
	}

	void addMenge(final String key, final String value) {
		mengen.put(key, value);
	}

}
