/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.assistenten;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.plugin.konfigass.KonfigAssDateiOperationen;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Class representing the first page of the wizard
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 *
 */
public class KvAoeNeuSeite1 extends WizardPage implements Listener {

	/** Logger für Debugausgaben. */
	private static final Debug LOGGER = Debug.getLogger();

	private Button kvButton;

	private Button aoeButton;

	private Text textName;

	private Text textPid;

	private Text textKodierung;

	private Button backButton;

	private Button cancelButton;

	/**
	 * Constructor for HolidayMainPage.
	 */
	public KvAoeNeuSeite1() {
		super("Seit1");
		setTitle("Neuer KV/AOE anlegen");
		setDescription("Schritt 2 : Konfigurationsbereich erstellen");
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	@Override
	public void createControl(final Composite parent) {
		GridData gd;
		final Composite composite = new Composite(parent, SWT.NULL);

		final GridLayout gl = new GridLayout();
		final int ncol = 2;
		gl.numColumns = ncol;
		composite.setLayout(gl);

		final GridData gridData110 = new GridData();
		gridData110.horizontalAlignment = GridData.FILL;
		gridData110.grabExcessHorizontalSpace = true;
		gridData110.verticalAlignment = GridData.CENTER;
		final GridData gridData111 = new GridData();
		gridData111.horizontalAlignment = GridData.FILL;
		gridData111.grabExcessHorizontalSpace = true;
		gridData111.verticalAlignment = GridData.CENTER;
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;

		final Label labelKodierung = new Label(composite, SWT.NONE);
		labelKodierung.setText("Kodierung");
		textKodierung = new Text(composite, SWT.BORDER);
		textKodierung.setLayoutData(gridData111);
		textKodierung.setTextLimit(4);
		textKodierung.addListener(SWT.Verify, new Listener() {
			@Override
			public void handleEvent(final Event e) {
				final String string = e.text;
				final char[] chars = new char[string.length()];
				string.getChars(0, chars.length, chars, 0);
				for (int i = 0; i < chars.length; i++) {
					if (!(('0' <= chars[i]) && (chars[i] <= '9'))) {
						e.doit = false;
						return;
					}
				}
			}
		});

		final Label labelBeschreibung = new Label(composite, SWT.NONE);
		labelBeschreibung.setText("Name");
		textName = new Text(composite, SWT.BORDER);
		textName.setLayoutData(gridData110);
		final Label labelQuelle = new Label(composite, SWT.NONE);
		labelQuelle.setText("Pid");
		textPid = new Text(composite, SWT.BORDER);
		textPid.setLayoutData(gridData1);
		textPid.setEnabled(false);

		createLine(composite, ncol);

		kvButton = new Button(composite, SWT.RADIO);
		kvButton.setText("Konfigurationsverantwortlicher");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol;
		kvButton.setLayoutData(gd);
		kvButton.setEnabled(false);

		aoeButton = new Button(composite, SWT.RADIO);
		aoeButton.setText("Autarke Organisations Einheit");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol;
		aoeButton.setLayoutData(gd);
		aoeButton.setEnabled(false);

		// set the composite as the control for this page
		final TkaWizardDialog container = (TkaWizardDialog) getContainer();
		backButton = container.getButton(IDialogConstants.BACK_ID);
		cancelButton = container.getButton(IDialogConstants.CANCEL_ID);

		setControl(composite);
		addListeners();
	}

	private void addListeners() {
		textKodierung.addListener(SWT.Modify, this);
		textName.addListener(SWT.Modify, this);
		textPid.addListener(SWT.Modify, this);
		backButton.addListener(SWT.Selection, this);
		cancelButton.addListener(SWT.Selection, this);

	}

	/**
	 * @see Listener#handleEvent(Event)
	 */
	@Override
	public void handleEvent(final Event event) {
		final TkaWizardDialog container = (TkaWizardDialog) getContainer();
		if (container != null) {
			final Object object = container.getSelectedPage();

			if (object instanceof KvAoeNeuStartSeite) {
				if (event.widget == backButton) {
					loeschenVerzeichnisMitFortschrittsAnzeige(container);
				}
			} else if ((object instanceof KvAoeNeuSeite1) || (object instanceof KvAoeNeuSeite2)
					|| (object instanceof KvAoeNeuSeite3)) {
				if (event.widget == cancelButton) {
					final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
					final KvAoeNeuModell model = wizard.getModel();
					if (model.getConfig() != null) {
						model.getConfig().close();
						model.setConfig(null);
					}

					loeschenVerzeichnisMitFortschrittsAnzeige(container);
				}
			}
		}
		// <<<<<<< KvAoeNeuSeite1.java
		//
		// if (getWizard() != null && getWizard().getContainer() != null)
		// getWizard().getContainer().updateButtons();
		// =======
		if (container != null) {
			container.updateButtons();
			// >>>>>>> 1.11
		}
	}

	private void loeschenVerzeichnisMitFortschrittsAnzeige(final TkaWizardDialog container) {
		final IRunnableWithProgress op2 = new IRunnableWithProgress() {

			@Override
			public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
				final KvAoeNeuModell model = wizard.getModel();

				final File file = new File(model.getZielVerzeichnis());
				final File file2 = new File(model.getQuellVerzeichnis());

				try {
					final int i = 0;

					final int cnt = KonfigAssDateiOperationen.copyDir(file2, file, null, i, null, false);

					KonfigAssDateiOperationen.loescheVerzeichnis(file, monitor, cnt);

				} catch (final IOException e) {
					KvAoeNeuSeite1.LOGGER.warning(e.getLocalizedMessage(), e);
				}

			}
		};
		try {
			container.run(true, true, op2);
		} catch (final InvocationTargetException e) {
			KvAoeNeuSeite1.LOGGER.error(e.getLocalizedMessage(), e);
		} catch (final InterruptedException e) {
			KvAoeNeuSeite1.LOGGER.error(e.getLocalizedMessage(), e);
		}
	}

	// /*
	// * Returns the next page. Saves the values from this page in the model
	// associated with the wizard. Initializes the
	// * widgets on the next page.
	// */
	//
	@Override
	public IWizardPage getNextPage() {
		saveDataToModel();

		final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
		final KvAoeNeuModell model = wizard.getModel();

		if (model.isUseAOE()) {
			final KvAoeNeuSeite2 page2 = ((KvAoeNeuAssistent) getWizard()).getSeite2();
			page2.onEnterPage();
			return page2;
		}

		final KvAoeNeuSeite3 page3 = ((KvAoeNeuAssistent) getWizard()).getSeite3();
		page3.onEnterPage();
		return page3;
	}

	/**
	 * @see IWizardPage#canFlipToNextPage()
	 */
	@Override
	public boolean canFlipToNextPage() {
		if (getErrorMessage() != null) {
			return false;
		}

		if ((textKodierung.getText().length() > 0) && (textPid.getText().length() > 0)
				&& (textName.getText().length() > 0)) {
			return true;

		}

		return false;
	}

	/*
	 * Saves the uses choices from this page to the model. Called on exit of the
	 * page
	 */
	private void saveDataToModel() {

		final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
		final KvAoeNeuModell model = wizard.getModel();

		model.setKodierung(textKodierung.getText());
		model.setPid(textPid.getText());
		model.setName(textName.getText());

	}

	/*
	 * Sets the completed field on the wizard class when all the information is
	 * entered and the wizard can be completed
	 */
	@Override
	public boolean isPageComplete() {

		if ((textKodierung.getText().length() > 0) && (textPid.getText().length() > 0)
				&& (textName.getText().length() > 0)) {

			saveDataToModel();
			return true;
		}
		return false;
	}

	private void createLine(final Composite parent, final int ncol) {
		final Label line = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.BOLD);
		final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = ncol;
		line.setLayoutData(gridData);
	}

	void onEnterPage() {
		// Gets the model
		final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
		final KvAoeNeuModell model = wizard.getModel();

		if (model.isUseAOE()) {
			// textKodierung.setEnabled(true);
			aoeButton.setSelection(true);
			kvButton.setSelection(false);
		} else {
			// textKodierung.setEnabled(false);
			aoeButton.setSelection(false);
			kvButton.setSelection(true);
		}

		if (model.getPid() != null) {
			textPid.setText(model.getPid());
		}
	}

}
