/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.assistenten;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.buv.plugin.konfigass.KonfigurationsAssistent;
import de.bsvrz.buv.plugin.konfigass.MeldungenSpeicher;
import de.bsvrz.buv.plugin.konfigass.managementfile.MyConfigurationAreaManagementInfo;
import de.bsvrz.buv.plugin.konfigass.managementfile.MyConfigurationManagementFile;
import de.bsvrz.buv.plugin.konfigass.managementfile.MyManagementFile;
import de.bsvrz.dav.daf.main.config.ObjectSetUse;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Class representing the first page of the wizard
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 *
 */
public class KvAoeNeuSeite2 extends WizardPage {

	/** Logger für Debugausgaben. */
	private static final Debug LOGGER = Debug.getLogger();

	private Button kvButton;

	private Button aoeButton;

	private TableViewer tvZufluesse1;

	private SortedMap<String, Object[]> mapMengen;

	private Button cancelButton;

	/**
	 * Constructor for HolidayMainPage.
	 */
	public KvAoeNeuSeite2() {
		super("Page2");
		setTitle("Neuer KV/AOE anlegen");
		setDescription("Schritt 3 : Auswahl der Mengen");
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	@Override
	public void createControl(final Composite parent) {

		// create the composite to hold the widgets
		GridData gd;
		final Composite composite = new Composite(parent, SWT.NULL);

		// create the desired layout for this wizard page
		final int ncol = 1;
		composite.setLayout(new GridLayout());

		erzeugeTabelleTeilkonfigurationsassistenten(composite);

		createLine(composite, ncol);

		// Choice of transport
		kvButton = new Button(composite, SWT.RADIO);
		kvButton.setText("Konfigurationsverantwortlicher");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol;
		kvButton.setLayoutData(gd);
		// kvButton.setSelection(true);
		kvButton.setEnabled(false);

		aoeButton = new Button(composite, SWT.RADIO);
		aoeButton.setText("Autarke Organisations Einheit");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol;
		aoeButton.setLayoutData(gd);
		aoeButton.setEnabled(false);

		// set the composite as the control for this page
		setControl(composite);
		// addListeners();

		final TkaWizardDialog container = (TkaWizardDialog) getContainer();
		cancelButton = container.getButton(IDialogConstants.CANCEL_ID);

		parent.layout(true);
	}

	private void erzeugeTabelleTeilkonfigurationsassistenten(final Composite parent) {
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		gridData.grabExcessVerticalSpace = true;

		final Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayout(new GridLayout());
		composite.setLayoutData(gridData);

		tvZufluesse1 = new TableViewer(composite,
				SWT.CHECK | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		tvZufluesse1.setContentProvider(new TabelleTeilkonfigurationsassistentenInhaltsVersorger());

		final TableViewerColumn tc1 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc1.getColumn().setWidth(20);
		tc1.getColumn().setText("Auswahl");

		final TableViewerColumn tc2 = new TableViewerColumn(tvZufluesse1, SWT.NONE);
		tc2.getColumn().setWidth(200);
		tc2.getColumn().setText("Menge");

		tvZufluesse1.getTable().setLayout(new FillLayout());
		final GridData gd2 = new GridData(GridData.FILL_BOTH);
		tvZufluesse1.getTable().setLayoutData(gd2);

	}

	@Override
	public IWizardPage getNextPage() {
		saveDataToModel();
		final KvAoeNeuSeite3 page3 = ((KvAoeNeuAssistent) getWizard()).getSeite3();
		page3.onEnterPage();
		return page3;
	}

	/**
	 * @see IWizardPage#canFlipToNextPage()
	 */
	@Override
	public boolean canFlipToNextPage() {

		if (getErrorMessage() != null) {
			return false;
		}
		return true;
	}

	/*
	 * Saves the uses choices from this page to the model. Called on exit of the
	 * page
	 */
	private void saveDataToModel() {
		final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
		final KvAoeNeuModell model = wizard.getModel();

		for (final Map.Entry<String, Object[]> me : mapMengen.entrySet()) {
			final String set = me.getKey();
			final Object[] oo = me.getValue();
			final String typ = (String) oo[0];
			final Boolean required = (Boolean) oo[1];

			if (required) {
				model.addMenge(set, typ);
			}

		}
	}

	/*
	 * Sets the completed field on the wizard class when all the information is
	 * entered and the wizard can be completed
	 */
	@Override
	public boolean isPageComplete() {
		return true;
	}

	private void createLine(final Composite parent, final int ncol) {
		final Label line = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = ncol;
		line.setLayoutData(gridData);
	}

	void onEnterPage() {
		// Gets the model
		final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
		final KvAoeNeuModell model = wizard.getModel();

		final String ziel = model.getZielVerzeichnis();
		final String pfad = ziel + "/config/verwaltungsdaten.xml";

		final MyConfigurationManagementFile managementFile = new MyManagementFile(new File(pfad));

		final List<MyConfigurationAreaManagementInfo> camis = managementFile.getAllMyConfigurationAreaManagementInfos();
		final String q = model.getQuellVerzeichnis();
		final String z = model.getZielVerzeichnis();

		for (final MyConfigurationAreaManagementInfo cami : camis) {
			final File dir = cami.getDirectory();

			final String s1 = dir.getPath();

			if (s1.contains(z)) {
				final String s2 = s1.replace(z, q);

				final URI uriBase = new File(s1.replace("\\", "/")).toURI();
				final URI uriRelative = new File(s2.replace("\\", "/")).toURI();

				final URI uri = uriBase.relativize(uriRelative);
				try {
					cami.setDirectory(new File(uri).getCanonicalFile());
					cami.delAllVersionInfo();
				} catch (final IOException e) {
					KvAoeNeuSeite2.LOGGER.error(e.getLocalizedMessage(), e);
				}

			}
		}

		try {
			managementFile.save();
			managementFile.close();

			mapMengen = new TreeMap<>();
			mapMengen = bestimmeMengen(pfad);

			for (final Map.Entry<String, Object[]> me : mapMengen.entrySet()) {

				final TableItem ti = new TableItem(tvZufluesse1.getTable(), SWT.NONE);
				ti.setText(1, me.getKey());
				ti.setData("required", me.getValue()[1]);

				final Boolean r = (Boolean) ti.getData("required");

				ti.setChecked(r);
				if ((Boolean) me.getValue()[1]) {
					ti.setForeground(new Color(null, 209, 209, 209));
				}
			}

			final Table table = tvZufluesse1.getTable();

			table.addListener(SWT.Selection, new Listener() {
				@Override
				public void handleEvent(final Event event) {
					if (event.detail == SWT.CHECK) {

						if (event.item.getData("required") instanceof Boolean) {

							final Boolean b = (Boolean) event.item.getData("required");

							if (b) {
								event.detail = SWT.NONE;
								event.type = SWT.None;
								event.doit = false;
								final TableItem item = (TableItem) event.item;
								item.setChecked(!item.getChecked());
							} else {
								final TableItem item = (TableItem) event.item;
								final String key = item.getText(1);

								final Object[] oo = mapMengen.get(key);
								oo[1] = item.getChecked();
								mapMengen.put(key, oo);

							}
						}
					}
				}
			});

		} catch (final IOException e) {
			KvAoeNeuSeite2.LOGGER.error(e.getLocalizedMessage(), e);
		}

		if (model.isUseAOE()) {
			aoeButton.setSelection(true);
			kvButton.setSelection(false);
		} else {
			aoeButton.setSelection(false);
			kvButton.setSelection(true);
		}

	}

	/**
	 * @author Pittner
	 *
	 */
	private static class TabelleTeilkonfigurationsassistentenInhaltsVersorger implements IStructuredContentProvider {

		@Override
		public Object[] getElements(final Object inputElement) {

			final List<Object> result = new ArrayList<>();
			if (inputElement instanceof List<?>) {
				result.addAll((List<?>) inputElement);
			}

			return result.toArray();
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput, final Object newInput) {
			// der Input wird nicht betrachtet
		}
	}

	private SortedMap<String, Object[]> bestimmeMengen(final String pfad) {
		ladeDatenmodellAssistent(pfad);

		final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
		final KvAoeNeuModell model = wizard.getModel();

		if (model.getConfig() != null) {
			final SystemObjectType type = model.getConfig().getType("typ.autarkeOrganisationsEinheit");

			final List<ObjectSetUse> sets = type.getObjectSetUses();

			for (final ObjectSetUse osu : sets) {

				final Object[] o = new Object[2];

				final String typ = osu.getObjectSetType().getPidOrNameOrId();
				final String name = osu.getObjectSetName();

				o[0] = typ;
				o[1] = osu.isRequired();
				mapMengen.put(name, o);

			}
		}

		return mapMengen;
	}

	private void ladeDatenmodellAssistent(final String pfad) {
		final IRunnableWithProgress op2 = new IRunnableWithProgress() {

			private MultiStatus status;

			@Override
			public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {

				final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
				final KvAoeNeuModell model = wizard.getModel();

				if (model.isUseKV()) {
					monitor.beginTask("Lade Datenmodell", 5);
				} else {
					monitor.beginTask("Lade Datenmodell / Bestimme Mengen", 5);
				}

				monitor.worked(1);

				try {
					status = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Laden des Datenmodells", null);

					final File file = new File(pfad);

					Display.getDefault().asyncExec(new Runnable() {

						@Override
						public void run() {
							cancelButton.setEnabled(false);
						}
					});

					model.setConfig(erzeugeKonfigDatenModellAusDatei(file));

					monitor.worked(1);

					if (model.getConfig() != null) {
						status.add(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
								"Laden Datenmodell erfolgreich abgeschlossen. "));
					}
				} catch (final Exception e) {
					status.add(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
							"Aktion Laden Datenmodell fehlgeschlagen, die evt. geladene Konfiguration wurde geschlossen. ",
							e));
				} finally {
					Display.getDefault().asyncExec(new Runnable() {

						@Override
						public void run() {
							cancelButton.setEnabled(true);
						}
					});

					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(status);
					monitor.worked(3);
					monitor.done();
				}
			}
		};
		try {
			final TkaWizardDialog container = (TkaWizardDialog) getContainer();
			container.run(true, true, op2);
		} catch (final InvocationTargetException e) {
			KvAoeNeuSeite2.LOGGER.error(e.getLocalizedMessage(), e);
		} catch (final InterruptedException e) {
			KvAoeNeuSeite2.LOGGER.error(e.getLocalizedMessage(), e);
		}
	}

	/**
	 * Erzeugt das Offline-Datenmodell der Konfiguration aus der
	 * XML-Verwaltungsdatei
	 *
	 * @param configurationFile die Verwaltungsdatei
	 * @return das Offline-Datenmodell
	 */
	private ConfigDataModel erzeugeKonfigDatenModellAusDatei(final File configurationFile) {
		try {
			final ConfigDataModel configuration = new ConfigDataModel(configurationFile);

			final Runnable autoCloser = new Runnable() {
				@Override
				public void run() {
					configuration.close();
				}
			};

			final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
			final KvAoeNeuModell model = wizard.getModel();

			if (model.getAutoCloser() != null) {
				Runtime.getRuntime().removeShutdownHook(model.getAutoCloser());
			}

			model.setAutoCloser(new Thread(autoCloser));
			Runtime.getRuntime().addShutdownHook(model.getAutoCloser());

			return configuration;
		} catch (final Exception e) {
			final MultiStatus multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK,
					"Fehler beim Laden des Datenmodells", e);
			MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
		}
		return null;

	}

}
