/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.assistenten;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.plugin.konfigass.KonfigurationsAssistent;
import de.bsvrz.buv.plugin.konfigass.MeldungenSpeicher;
import de.bsvrz.buv.plugin.konfigass.managementfile.MyConfigurationAreaManagementInfo;
import de.bsvrz.buv.plugin.konfigass.managementfile.MyConfigurationManagementFile;
import de.bsvrz.buv.plugin.konfigass.managementfile.MyManagementFile;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Class representing the first page of the wizard
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 *
 */
public class KvAoeNeuSeite3 extends WizardPage implements Listener {

	/** Logger für Debugausgaben. */
	private static final Debug LOGGER = Debug.getLogger();

	private Button kvButton;

	private Button aoeButton;

	private Text textInfo;

	private StyledText textInfo2;

	private Text textInfoAOE;

	private Text textInfo2AOE;

	private Button cancelButton;

	private Button backButton;

	/**
	 * Constructor for HolidayMainPage.
	 */
	public KvAoeNeuSeite3() {
		super("Seite2");
		setTitle("Neuer KV/AOE anlegen");
		setDescription("Zusätzliche Informationen");
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	@Override
	public void createControl(final Composite parent) {

		// create the composite to hold the widgets
		GridData gd;
		final Composite composite = new Composite(parent, SWT.NULL);

		// create the desired layout for this wizard page
		final GridLayout gl = new GridLayout();
		final int ncol = 2;
		gl.numColumns = ncol;
		composite.setLayout(gl);

		final Label labelInfo = new Label(composite, SWT.NONE);
		labelInfo.setText("Kurzinfo");
		textInfo = new Text(composite, SWT.BORDER);
		textInfo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL));
		final Label labelInfo2 = new Label(composite, SWT.NONE);
		labelInfo2.setText("Beschreibung");
		labelInfo2.setLayoutData(new GridData(GridData.FILL_VERTICAL | GridData.VERTICAL_ALIGN_BEGINNING));
		textInfo2 = new StyledText(composite, SWT.BORDER | SWT.MULTI);
		textInfo2.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));

		final Label labelInfoAOE = new Label(composite, SWT.NONE);
		labelInfoAOE.setText("Kurzinfo Def.-Bereich");
		textInfoAOE = new Text(composite, SWT.BORDER);
		textInfoAOE.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL));
		final Label labelInfo2AOE = new Label(composite, SWT.NONE);
		labelInfo2AOE.setText("Beschr. Def.-Bereich");
		labelInfo2AOE.setLayoutData(new GridData(GridData.FILL_VERTICAL | GridData.VERTICAL_ALIGN_BEGINNING));
		textInfo2AOE = new Text(composite, SWT.BORDER | SWT.MULTI);
		textInfo2AOE
				.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));

		createLine(composite, ncol);

		// Choice of transport
		kvButton = new Button(composite, SWT.RADIO);
		kvButton.setText("Konfigurationsverantwortlicher");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol;
		kvButton.setLayoutData(gd);
		// kvButton.setSelection(true);
		kvButton.setEnabled(false);

		aoeButton = new Button(composite, SWT.RADIO);
		aoeButton.setText("Autarke Organisations Einheit");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol;
		aoeButton.setLayoutData(gd);
		aoeButton.setEnabled(false);

		final TkaWizardDialog container = (TkaWizardDialog) getContainer();
		cancelButton = container.getButton(IDialogConstants.CANCEL_ID);
		backButton = container.getButton(IDialogConstants.BACK_ID);

		// set the composite as the control for this page
		setControl(composite);
		addListeners();

	}

	private void addListeners() {
		textInfo.addListener(SWT.Modify, this);
		textInfo2.addListener(SWT.Modify, this);
		textInfoAOE.addListener(SWT.Modify, this);
		textInfo2AOE.addListener(SWT.Modify, this);
		backButton.addListener(SWT.Selection, this);
	}

	/**
	 * @see Listener#handleEvent(Event)
	 */
	@Override
	public void handleEvent(final Event event) {
		final TkaWizardDialog container = (TkaWizardDialog) getContainer();
		if (container != null) {
			final Object object = container.getSelectedPage();

			if (object instanceof KvAoeNeuSeite1) {
				if (event.widget == backButton) {

					final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
					final KvAoeNeuModell model = wizard.getModel();
					if (model.getAutoCloser() != null) {
						model.getConfig().close();
						model.setConfig(null);
					}
				}
			}
		}

		setPageComplete(isPageComplete());
		getWizard().getContainer().updateButtons();
	}

	/**
	 * @see IWizardPage#canFlipToNextPage()
	 */
	@Override
	public boolean canFlipToNextPage() {
		saveDataToModel();
		return false;
	}

	/*
	 * Saves the uses choices from this page to the model. Called on exit of the
	 * page
	 */
	private void saveDataToModel() {
		final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
		final KvAoeNeuModell model = wizard.getModel();

		model.setInfo(textInfo.getText());
		model.setBeschreibung(textInfo2.getText());
		model.setInfo(textInfoAOE.getText());
		model.setBeschreibungDef(textInfo2AOE.getText());
	}

	/*
	 * Sets the completed field on the wizard class when all the information is
	 * entered and the wizard can be completed
	 */
	@Override
	public boolean isPageComplete() {
		saveDataToModel();
		return true;
	}

	private void createLine(final Composite parent, final int ncol) {
		final Label line = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = ncol;
		line.setLayoutData(gridData);
	}

	void onEnterPage() {
		final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
		final KvAoeNeuModell model = wizard.getModel();

		if (model.isUseKV() && (model.getConfig() == null)) {
			final String ziel = model.getZielVerzeichnis();
			final String pfad = ziel + "/config/verwaltungsdaten.xml";

			final MyConfigurationManagementFile managementFile = new MyManagementFile(new File(pfad));

			final List<MyConfigurationAreaManagementInfo> camis = managementFile
					.getAllMyConfigurationAreaManagementInfos();
			final String q = model.getQuellVerzeichnis();
			final String z = model.getZielVerzeichnis();

			for (final MyConfigurationAreaManagementInfo cami : camis) {
				final File dir = cami.getDirectory();

				final String s1 = dir.getPath();

				if (s1.contains(z)) {
					final String s2 = s1.replace(z, q);

					final URI uriBase = new File(s1.replace("\\", "/")).toURI();
					final URI uriRelative = new File(s2.replace("\\", "/")).toURI();

					final URI uri = uriBase.relativize(uriRelative);
					try {
						cami.setDirectory(new File(uri).getCanonicalFile());
						cami.delAllVersionInfo();
					} catch (final IOException e) {
						KvAoeNeuSeite3.LOGGER.error(e.getLocalizedMessage(), e);
					}

				}
			}

			try {
				managementFile.save();
				managementFile.close();

				ladeDatenmodellAssistent(pfad);

			} catch (final IOException e) {
				KvAoeNeuSeite3.LOGGER.error(e.getLocalizedMessage(), e);
			}
		}

		setPageComplete(isPageComplete());

		if (!model.isUseAOE()) {
			aoeButton.setSelection(false);
			kvButton.setSelection(true);
			textInfo2AOE.setEnabled(false);
			textInfoAOE.setEnabled(false);
		} else {
			aoeButton.setSelection(true);
			kvButton.setSelection(false);
			textInfo2AOE.setEnabled(true);
			textInfoAOE.setEnabled(true);
		}
	}

	private void ladeDatenmodellAssistent(final String pfad) {
		final IRunnableWithProgress op2 = new IRunnableWithProgress() {

			private MultiStatus status;

			@Override
			public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {

				final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
				final KvAoeNeuModell model = wizard.getModel();

				if (model.isUseKV()) {
					monitor.beginTask("Lade Datenmodell", 1);
				} else {
					monitor.beginTask("Lade Datenmodell / Bestimme Mengen", 1);
				}

				try {
					status = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK, "Starte Laden des Datenmodells", null);

					final File file = new File(pfad);

					Display.getDefault().asyncExec(new Runnable() {

						@Override
						public void run() {
							cancelButton.setEnabled(false);
						}
					});

					model.setConfig(erzeugeKonfigDatenModellAusDatei(file));

					monitor.worked(1);

					if (model.getConfig() != null) {
						status.add(new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID,
								"Laden Datenmodell erfolgreich abgeschlossen. "));
					}
				} catch (final Exception e) {
					status.add(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
							"Aktion Laden Datenmodell fehlgeschlagen, die evt. geladene Konfiguration wurde geschlossen. ", e));
				} finally {
					Display.getDefault().asyncExec(new Runnable() {

						@Override
						public void run() {
							cancelButton.setEnabled(true);
						}
					});

					MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(status);
					monitor.done();
				}
			}
		};
		try {
			final TkaWizardDialog container = (TkaWizardDialog) getContainer();
			container.run(true, true, op2);
		} catch (final InvocationTargetException e) {
			KvAoeNeuSeite3.LOGGER.error(e.getLocalizedMessage(), e);
		} catch (final InterruptedException e) {
			KvAoeNeuSeite3.LOGGER.error(e.getLocalizedMessage(), e);
		}
	}

	/**
	 * Erzeugt das Offline-Datenmodell der Konfiguration aus der
	 * XML-Verwaltungsdatei
	 *
	 * @param configurationFile
	 *            die Verwaltungsdatei
	 * @return das Offline-Datenmodell
	 */
	private ConfigDataModel erzeugeKonfigDatenModellAusDatei(final File configurationFile) {
		try {
			final ConfigDataModel configuration = new ConfigDataModel(configurationFile);

			final Runnable autoCloser = new Runnable() {
				@Override
				public void run() {
					configuration.close();
				}
			};

			final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
			final KvAoeNeuModell model = wizard.getModel();

			if (model.getAutoCloser() != null) {
				Runtime.getRuntime().removeShutdownHook(model.getAutoCloser());
			}

			model.setAutoCloser(new Thread(autoCloser));
			Runtime.getRuntime().addShutdownHook(model.getAutoCloser());

			return configuration;
		} catch (final Exception e) {
			final MultiStatus multiStatus = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK,
					"Fehler beim Laden des Datenmodells", e);
			MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus);
		}
		return null;

	}

}
