/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.assistenten;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.konfigass.KonfigurationsAssistent;
import de.bsvrz.buv.plugin.konfigass.KonfigAssDateiOperationen;
import de.bsvrz.buv.plugin.konfigass.MeldungenSpeicher;
import de.bsvrz.puk.config.main.managementfile.ManagementFile;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Class representing the first page of the wizard
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 *
 */
public class KvAoeNeuStartSeite extends WizardPage implements Listener {

	/** Logger für Debugausgaben. */
	private static final Debug LOGGER = Debug.getLogger();

	private Button kvButton;

	private Button aoeButton;

	private Text textName;

	private Text textOrtOrdner;

	private Shell shell;

	private File source;

	/**
	 * Constructor for HolidayMainPage.
	 */
	public KvAoeNeuStartSeite() {
		super("Page1");
		setTitle("Neuer KV/AOE anlegen");
		setDescription("Schritt 1 : Ordnerstruktur kopieren");
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	@Override
	public void createControl(final Composite parent) {
		shell = parent.getShell();

		// create the composite to hold the widgets
		GridData gd;
		final Composite composite = new Composite(parent, SWT.NULL);

		// create the desired layout for this wizard page
		final GridLayout gl = new GridLayout();
		final int ncol = 3;
		gl.numColumns = ncol;
		composite.setLayout(gl);

		final Label labelName = new Label(composite, SWT.NONE);
		labelName.setText("Pid");
		textName = new Text(composite, SWT.BORDER);
		GridDataFactory.fillDefaults().span(2, 1).grab(true, false).applyTo(textName);

		final Label labelQuelle2 = new Label(composite, SWT.NONE);
		labelQuelle2.setText("Vorlage Ordnerstruktur");
		textOrtOrdner = new Text(composite, SWT.BORDER);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(textOrtOrdner);
		textOrtOrdner.setEnabled(false);

		final Button button12 = new Button(composite, SWT.NONE);
		button12.setText("...");
		button12.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseDown(final MouseEvent e) {
				final DirectoryDialog dlg = new DirectoryDialog(shell);

				dlg.setText("Ordner-Auswahl Dialog");

				dlg.setMessage("Ordner auswählen");

				final String dir = dlg.open();
				if (dir != null) {
					textOrtOrdner.setText(dir);
					final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
					final KvAoeNeuModell model = wizard.getModel();
					if (model.getConfig() != null) {
						model.getConfig().close();
						model.setConfig(null);
					}

				}

				// startJobLadeDatenModell();
			}
		});
		// button2 = new Button(composite, SWT.NONE);
		// button2.setText("...");

		createLine(composite, ncol);

		// // Choice of transport
		kvButton = new Button(composite, SWT.RADIO);
		kvButton.setText("Konfigurationsverantwortlicher");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol;
		kvButton.setLayoutData(gd);
		kvButton.setSelection(true);

		aoeButton = new Button(composite, SWT.RADIO);
		aoeButton.setText("Autarke Organisations Einheit");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol;
		aoeButton.setLayoutData(gd);

		// set the composite as the control for this page
		setControl(composite);
		addListeners();
	}

	private void addListeners() {
		// textKodierung.addListener(SWT.Modify, this);
		textName.addListener(SWT.Modify, this);
		textOrtOrdner.addListener(SWT.Modify, this);
		// textPid.addListener(SWT.Modify, this);
		// textOrtKonfig.addListener(SWT.Modify, this);
		// button.addListener(SWT.Selection, this);
		// textOrtOrdner.addListener(SWT.Modify, this);
		// button2.addListener(SWT.Selection, this);
	}

	/**
	 * @see Listener#handleEvent(Event)
	 */
	@Override
	public void handleEvent(final Event event) {
		// Initialize a variable with the no error status
		// Status status = new Status(IStatus.OK, "not_used", 0, "", null);
		// If the event is triggered by the destination or departure fields
		// set the corresponding status variable to the right value

		if (event.widget == textOrtOrdner) {
			final String sDir = textOrtOrdner.getText();
			source = new File(sDir);
			if (source.isDirectory()) {
				final MultiStatus status = new MultiStatus(KonfigurationsAssistent.PLUGIN_ID, IStatus.OK,
						"Überprüfen der Verwaltungsdatei", null);

				File config = new File(source.getAbsolutePath() + "/config");
				if (!config.exists()) {
					config = new File(source.getAbsolutePath() + "/konfiguration");
				}
				if (config.exists() && config.isDirectory()) {
					final File[] list = config.listFiles();

					boolean gefunden = false;

					if (list != null) {
						for (final File f : list) {
							if ("verwaltungsdaten.xml".equals(f.getName())) {

								ManagementFile mgmtFile = null;
								try {

									mgmtFile = new ManagementFile(f);
									mgmtFile.close();

									gefunden = true;
									status.merge(
											new Status(IStatus.OK, KonfigurationsAssistent.PLUGIN_ID, "XML-Verwaltungsdatei OK."));

								} catch (final Exception e) {

									status.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
											"Folgende Ausnahmen sind aufgetreten. ", e));

									MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(status);

									final String message = "Beim Überprüfen der Verwaltungsdatei sind Ausnahmen aufgetreten!";

									final MessageBox mb = new MessageBox(getShell(), SWT.OK | SWT.ICON_ERROR);
									mb.setText("Ungültige Verwaltungsdatei");
									mb.setMessage(message);
									mb.open();

									textOrtOrdner.setText("");
									canFlipToNextPage();
									return;
								}
							}
						}
					}
					if (!gefunden) {
						status.merge(new Status(IStatus.ERROR, KonfigurationsAssistent.PLUGIN_ID,
								"verwaltungsdaten.xml wurde nicht gefunden!"));

						MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(status);

						final String message = "Beim Überprüfen der Verwaltungsdatei sind Ausnahmen aufgetreten!";

						final MessageBox mb = new MessageBox(getShell(), SWT.OK | SWT.ICON_ERROR);
						mb.setText("Keine Verwaltungsdatei");
						mb.setMessage(message);
						mb.open();

						textOrtOrdner.setText("");
						canFlipToNextPage();
					}
				} else {
					final String message = "Ordnervorlage enthält keine entsprechenden Unterverzeichnisse !";

					final MessageBox mb = new MessageBox(getShell(), SWT.OK | SWT.ICON_ERROR);
					mb.setText("Ungültige Ordnervorlage");
					mb.setMessage(message);
					mb.open();

					textOrtOrdner.setText("");
				}
			}
		}

		// // Show the most serious error
		// applyToStatusLine(findMostSevere());
		getWizard().getContainer().updateButtons();
	}

	/*
	 * Returns the next page. Saves the values from this page in the model
	 * associated with the wizard. Initializes the widgets on the next page.
	 */

	@Override
	public IWizardPage getNextPage() {

		final String sDir = textOrtOrdner.getText();
		source = new File(sDir);
		final String s = source.getParent();
		File dest = new File(s);
		if (dest.isDirectory()) {
			final String[] list = dest.list();
			if (list != null) {
				for (final String ss : list) {
					if (ss.equals(textName.getText())) {
						final String message = "Die Pid für den KV/AOE: " + textName.getText()
								+ "wurde bereits vergeben";

						final MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(),
								SWT.OK | SWT.ICON_ERROR);
						mb.setText("Ungültige Pid");
						mb.setMessage(message);
						mb.open();

						return null;
					}

				}
			}
		}

		saveDataToModel();

		final String destNeu = dest.getAbsolutePath() + "/" + textName.getText();

		dest = new File(destNeu);

		final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
		final KvAoeNeuModell model = wizard.getModel();
		model.setZielVerzeichnis(dest.getAbsolutePath());
		model.setQuellVerzeichnis(source.getAbsolutePath());

		final File fsource = source;
		final File fdest = dest;

		final IRunnableWithProgress op2 = new IRunnableWithProgress() {

			@Override
			public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {

				try {
					int i = 0;

					final int cnt = KonfigAssDateiOperationen.copyDir(fsource, fdest, null, i, null, false);
					i = 0;
					KonfigAssDateiOperationen.copyDir(fsource, fdest, monitor, i, cnt, true);

				} catch (final IOException e) {
					KvAoeNeuStartSeite.LOGGER.error(e.getLocalizedMessage(), e);
				}
			}
		};
		try {
			final TkaWizardDialog container = (TkaWizardDialog) getContainer();
			container.run(true, true, op2);

		} catch (final InvocationTargetException e) {
			KvAoeNeuStartSeite.LOGGER.error(e.getLocalizedMessage(), e);
		} catch (final InterruptedException e) {
			KvAoeNeuStartSeite.LOGGER.error(e.getLocalizedMessage(), e);
		}

		final KvAoeNeuSeite1 page = ((KvAoeNeuAssistent) getWizard()).getSeite1();
		page.onEnterPage();
		return page;
	}

	/**
	 * @see IWizardPage#canFlipToNextPage()
	 */
	@Override
	public boolean canFlipToNextPage() {
		if (getErrorMessage() != null) {
			return false;
		}

		if (KvAoeNeuStartSeite.isTextNonEmpty(textName) && KvAoeNeuStartSeite.isTextNonEmpty(textOrtOrdner)) {
			return true;

		}
		return false;
	}

	/*
	 * Saves the uses choices from this page to the model. Called on exit of the
	 * page
	 */
	private void saveDataToModel() {
		// Gets the model
		final KvAoeNeuAssistent wizard = (KvAoeNeuAssistent) getWizard();
		final KvAoeNeuModell model = wizard.getModel();

		model.setUseKV(kvButton.getSelection());
		model.setUseAOE(aoeButton.getSelection());

		model.setPid(textName.getText());
		model.setZielVerzeichnis(textOrtOrdner.getText());

	}

	private static boolean isTextNonEmpty(final Text t) {
		if (t != null) {
			final String s = t.getText();
			if ((s != null) && (s.trim().length() > 0)) {
				return true;
			}
		}
		return false;
	}

	private void createLine(final Composite parent, final int ncol) {
		final Label line = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.BOLD);
		final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = ncol;
		line.setLayoutData(gridData);
	}

}
