/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.datenModellUpdateListener;

import java.util.Collections;
import java.util.EventObject;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Es wird ein Event (Ereignis) für die Benachrichtigung von TKA untereinander
 * modelliert
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, Sans (Dambach)
 *
 */
public class DatenModellUpdateEvent extends EventObject {
	private static final long serialVersionUID = 1365256169927819792L;

	/**
	 * Liste der geänderten Systemobjekte
	 */
	private final List<SystemObject> listeAktualisiert;

	/**
	 * Liste der gelöschten Systemobjekte
	 */
	private final List<SystemObject> listeGeloescht;

	/**
	 * Liste der neuen Systemobjekte
	 */
	private final List<SystemObject> listeNeu;

	/**
	 * Konstruktor
	 *
	 * @param quelle
	 *            Quelle des Events
	 * @param listeAktualisiert
	 *            Liste der aktualisierten Systemobjekte
	 * @param listeGeloescht
	 *            Liste der gelöschten Systemobjekte
	 * @param listeNeu
	 *            Liste der neuen Systemobjekte
	 */
	public DatenModellUpdateEvent(final Object quelle, final List<SystemObject> listeAktualisiert,
			final List<SystemObject> listeGeloescht, final List<SystemObject> listeNeu) {
		super(quelle);
		this.listeAktualisiert = listeAktualisiert;
		this.listeGeloescht = listeGeloescht;
		this.listeNeu = listeNeu;
	}

	/**
	 * Liefert die Liste der aktualisierten Systemobjekte zurück
	 *
	 * @return Liste der aktualisierten Systemobjekte
	 */
	public List<SystemObject> getListeSystemObjekte() {
		if (listeAktualisiert != null) {
			return Collections.unmodifiableList(listeAktualisiert);
		}
		return Collections.emptyList();
	}

	/**
	 * Liefert die Liste der geloeschten Systemobjekte zurück
	 *
	 * @return Liste der geloeschten Systemobjekte
	 */
	public List<SystemObject> getListeSystemObjekteGeloescht() {
		if (listeGeloescht != null) {
			return Collections.unmodifiableList(listeGeloescht);
		}
		return Collections.emptyList();
	}

	/**
	 * Liefert die Liste der neu erzeugten Systemobjekte zurück
	 *
	 * @return Liste der neu erzeugten Systemobjekte
	 */
	public List<SystemObject> getListeSystemObjekteNeu() {
		if (listeNeu != null) {
			return Collections.unmodifiableList(listeNeu);
		}
		return Collections.emptyList();
	}

}
