/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.log;

//~ NICHT JDK IMPORTE =========================================================

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.konfigass.KonfigurationsAssistent;

//~ KLASSEN ===================================================================

/**
 * Die Klasse definiert die Meldungen für das StandartView "ErrorLog".
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, Liliya Givorgizova (LG, inovat)
 *
 */
public final class Log {

	private Log() {
		// es gibt keine Instanzen der Klasse
	}

	// ~ METHODEN ==============================================================

	/**
	 * Ermittle den Text für den Status.
	 *
	 * @param status Status. Kann eine der Konstanten Log.OK, Log.INFO, Log.WARNUNG,
	 *               Log.FEHLER, Log.ABBRUCH sein.
	 *
	 * @return Text für den Status.
	 */
	public static String ermittleStatusText(final IStatus status) {
		if (IStatus.CANCEL == status.getSeverity()) {
			return "ABBRUCH";
		}
		if (IStatus.ERROR == status.getSeverity()) {
			return "FEHLER";
		}
		if (IStatus.INFO == status.getSeverity()) {
			return "INFO";
		}
		if (IStatus.OK == status.getSeverity()) {
			return "OK";
		}
		if (IStatus.WARNING == status.getSeverity()) {
			return "WARNUNG";
		}

		return "";
	}

	/**
	 * Erzeugt eine Meldung für "ErrorLog".
	 *
	 * @param typ         kann eine der Konstanten Log.OK, Log.INFO, Log.WARNUNG,
	 *                    Log.FEHLER_, Log.ABBRUCH sein.
	 * @param pluginID    ID von PlugIn, das die Meldung erzeugt hat. Wird oft über
	 *                    Activator.PLUGIN_ID erreichbar.
	 * @param meldungText Text der Meldung.
	 * @param exception   Exception, das beim Doppelklick auf der Meldung angezeigt
	 *                    wird.
	 *
	 * @return die erzeugte Meldung von Typ Status.
	 */
	public static IStatus erzeugeMeldung(final int typ, final String pluginID, final String meldungText,
			final Throwable exception) {
		return new Status(typ, pluginID, meldungText, exception);
	}

	/**
	 * Erzeugt eine MultiMeldung ohne Kinder für "ErrorLog". Die Kinder können mit
	 * der Methode add oder addAll hinzugefügt werden. Die MultiMeldung kriegt den
	 * schlimmsten Status seiner Kinder.
	 *
	 * @param pluginID    ID von PlugIn, das die Meldung erzeugt hat. Wird oft über
	 *                    Activator.PLUGIN_ID erreichbar.
	 * @param meldungText Text der Meldung.
	 * @param exception   Exception, das beim Doppelklick auf der Meldung angezeigt
	 *                    wird.
	 *
	 * @return die erzeugte MultiMeldung von Typ MultiStatus.
	 */
	public static MultiStatus erzeugeMultiMeldung(final String pluginID, final String meldungText,
			final Throwable exception) {
		return new MultiStatus(pluginID, 0, meldungText, exception);
	}

	/**
	 * Zeigt die Meldung im "ErrorLog".
	 *
	 * @param meldung die zu zeigende Meldung.
	 */
	public static void zeige(final IStatus meldung) {
		KonfigurationsAssistent.getDefault().getLog().log(meldung);
	}
}
