/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.log;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;

/**
 * Klasse beschreibt ein MeldungsElement, das im MeldungsBaum angezeigt wird.
 */
public class MeldungsElement {

	/** Kinder der Meldung. */
	private List<MeldungsElement> listeKinder = new ArrayList<>();

	public List<MeldungsElement> getListeKinder() {
		return listeKinder;
	}

	/** Meldung. */
	private IStatus meldung;

	protected IStatus getMeldung() {
		return meldung;
	}

	/** Zeit der letzten Meldungsänderung oder 0. */
	private LocalDateTime meldungsZeit = LocalDateTime.MIN;

	public LocalDateTime getMeldungsZeit() {
		return meldungsZeit;
	}

	/** Vorfahr der Meldung oder null. */
	private final MeldungsElement vorfahr;

	private String konfiguration;

	// ~ KONSTRUKTOREN (und vom Konstruktor verwendete Methoden) ==========

	protected String getKonfiguration() {
		return konfiguration;
	}

	/**
	 * Erzeugt eine Instanz der Klasse MeldungsElement.
	 *
	 * @param meldung Meldung.
	 */
	public MeldungsElement(final IStatus meldung, final LocalDateTime meldungsZeit, final String pfad) {
		this(meldung, null, meldungsZeit, pfad);

		// Falls Meldung eine MultiMeldung ist,
		// setze die Meldungszeit:
		if (meldung instanceof MultiMeldung) {
			final MultiMeldung mm = (MultiMeldung) meldung;

			this.meldungsZeit = mm.getMeldungsZeit();
			konfiguration = null;

			// Falls die MultiMeldung ein Kind mit dem gleichen Text hat,
			// wird die Meldung reduziert (nur das Kind angezeigt):
			if ((mm.getChildren().length == 1) && (mm.getMessage().equals(mm.getChildren()[0].getMessage()))) {
				final MeldungsElement me = new MeldungsElement(mm.getChildren()[0], null, null);
				this.meldung = me.meldung;
				listeKinder = me.listeKinder;
			}
		} else {
			this.meldungsZeit = meldungsZeit;
			konfiguration = pfad;
		}
	}

	/**
	 * Erzeugt eine Instanz der Klasse MeldungsElement.
	 *
	 * @param meldung Meldung.
	 * @param vorfahr Vorfahr der Meldung oder null.
	 */
	private MeldungsElement(final IStatus meldung, final MeldungsElement vorfahr, final LocalDateTime meldungsZeit,
			final String pfad) {
		this.meldung = meldung;
		this.vorfahr = vorfahr;

		if (meldung instanceof MultiStatus) {
			final MultiStatus ms = (MultiStatus) meldung;

			for (final IStatus mel : ms.getChildren()) {

				// Bearbeite die Kinder rekursiv:
				final MeldungsElement me = new MeldungsElement(mel, this, meldungsZeit, pfad);

				listeKinder.add(me);
			}
		}
	}

	public MeldungsElement getVorfahr() {
		return vorfahr;
	}
}