/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.log;

import java.time.LocalDateTime;

//~ NICHT JDK IMPORTE =========================================================

import org.eclipse.core.runtime.MultiStatus;

//~ KLASSEN ===================================================================

/**
 * Klasse erzeugt und verwaltet eine MultiMeldung.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, Liliya Givorgizova (LG, inovat)
 *
 */
public class MultiMeldung extends MultiStatus {

	/** Zeit der letzten Meldungsänderung. */
	private final LocalDateTime meldungsZeit;

	// ~ KONSTRUKTOREN (und vom Konstruktor verwendete Methoden) ==============

	/**
	 * Erzeugt eine Instanz der Klasse MultiMeldung.
	 *
	 * @param pluginID
	 *            ID vom PlugIn.
	 * @param text
	 *            Überschrift der MultiMeldung.
	 */
	private MultiMeldung(final String pluginID, final String text) {
		super(pluginID, 0, text, null);
		meldungsZeit = LocalDateTime.now();
	}

	/**
	 * Erzeugt eine Instanz der Klasse MultiMeldung. Für diese MultiMeldung wird ein
	 * Kind mit dem gleichen Text und eine Exception erzeugt.
	 *
	 * @param pluginID
	 *            ID vom PlugIn.
	 * @param typ
	 *            Typ vom Kind. Kann eine der Konstanten Log.OK, Log.INFO,
	 *            Log.WARNUNG, Log.FEHLER, Log.ABBRUCH sein.
	 * @param text
	 *            Überschrift der MultiMeldung und des Kindes.
	 * @param e
	 *            Exception.
	 */
	public MultiMeldung(final String pluginID, final int typ, final String text, final Exception e) {
		this(pluginID, text);
		this.add(Log.erzeugeMeldung(typ, this.getPlugin(), text, e));
	}


	// ~ GET METHODEN ==========================================================

	/**
	 * Getter für Eigenschaft 'meldungsZeit'.
	 *
	 * @return Wert der Eigenschaft 'meldungsZeit'.
	 *
	 * @see #meldungsZeit
	 */
	public LocalDateTime getMeldungsZeit() {
		return meldungsZeit;
	}

	// ~ METHODEN ==============================================================

	/**
	 * Zeige die MultiMeldung im ErrorLog. Falls die MultiMeldung nur ein Kind hat
	 * und die Überschriften gleich sind, wird nur dieses Kind angezeigt.
	 */
	public void zeige() {
		if ((this.getChildren().length == 1) && (this.getChildren()[0].getMessage().equals(this.getMessage()))) {
			Log.zeige(this.getChildren()[0]);
		} else {
			Log.zeige(this);
		}
	}
}

// ~Formatiert mit 'inovat Kodierkonvention' am 04.11.09
