/*
 * Rahmenwerk-Plug-in "Konfigurationsassistent"
 *
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.log;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;

//~ KLASSEN ===================================================================

/**
 * Klasse stellt das GUI 'Protokollmeldungen' dar.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, Liliya Givorgizova (LG, inovat)
 *
 */
public class ProtokollGUI extends Composite {

	/** Font der Meldungen. */
	private static final Font FONT_COURIER_9_NORMAL = new Font(Display.getCurrent(), "Courier", 9, SWT.NORMAL);

	// ~ FELDER ================================================================

	/** Schaltfläche 'Löschen'. */
	private Button btnLoeschen;

	/** Schaltfläche 'Speichern'. */
	private Button btnSpeichern;

	/** Verwaltungsinstanz. */
	private final ProtokollGUIVerwaltung guiVew;

	/** Baum mit den Meldungen. */
	private TreeViewer treeViewerMeldungen;

	/** Meldungstext. */
	private Text txtMeldung;

	// ~ KONSTRUKTOREN (und vom Konstruktor verwendete Methoden) ==============

	/**
	 * Erzeugt eine Instanz der Klasse ProtokollGUI.
	 *
	 * @param parent Parent-Composite.
	 */
	public ProtokollGUI(final Composite parent) {
		super(parent, SWT.NONE);
		guiVew = new ProtokollGUIVerwaltung(this);

		final GridData gdDompVerwaltungsBereich = new GridData(SWT.FILL, SWT.FILL, true, true);

		setLayoutData(gdDompVerwaltungsBereich);

		final GridLayout gridLayout = new GridLayout();

		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;
		setLayout(gridLayout);
		initGUI();
	}

	// ~ GET METHODEN ==========================================================

	/**
	 * Getter für Eigenschaft 'btnLoeschen'.
	 *
	 * @return Wert der Eigenschaft 'btnLoeschen'.
	 *
	 * @see #btnLoeschen
	 */
	public Button getBtnLoeschen() {
		return btnLoeschen;
	}

	/**
	 * Getter für Eigenschaft 'btnSpeichern'.
	 *
	 * @return Wert der Eigenschaft 'btnSpeichern'.
	 *
	 * @see #btnSpeichern
	 */
	public Button getBtnSpeichern() {
		return btnSpeichern;
	}

	/**
	 * Getter für Eigenschaft 'guiVew'.
	 *
	 * @return Wert der Eigenschaft 'guiVew'.
	 *
	 * @see #guiVew
	 */
	public ProtokollGUIVerwaltung getGuiVew() {
		return guiVew;
	}

	/**
	 * Getter für Eigenschaft 'treeViewerMeldungen'.
	 *
	 * @return Wert der Eigenschaft 'treeViewerMeldungen'.
	 *
	 * @see #treeViewerMeldungen
	 */
	public TreeViewer getTreeViewerMeldungen() {
		return treeViewerMeldungen;
	}

	/**
	 * Getter für Eigenschaft 'txtMeldung'.
	 *
	 * @return Wert der Eigenschaft 'txtMeldung'.
	 *
	 * @see #txtMeldung
	 */
	public Text getTxtMeldung() {
		return txtMeldung;
	}

	// ~ METHODEN ==============================================================

	/**
	 * Initialisiere das GUI.
	 */
	private void initGUI() {
		final Composite compAllgemein = new Composite(this, SWT.NONE);
		compAllgemein.addDisposeListener(event -> guiVew.dispose());

		compAllgemein.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		final GridLayout gridLayout1 = new GridLayout();

		gridLayout1.marginHeight = 0;
		gridLayout1.marginWidth = 0;
		compAllgemein.setLayout(gridLayout1);

		final SashForm sashComp = new SashForm(compAllgemein, SWT.BORDER | SWT.VERTICAL);

		sashComp.SASH_WIDTH = 1;
		sashComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		//
		final Composite compBaum = new Composite(sashComp, SWT.NONE);

		compBaum.setLayout(new GridLayout());
		treeViewerMeldungen = new TreeViewer(compBaum, SWT.FULL_SELECTION | SWT.BORDER);
		treeViewerMeldungen.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				txtMeldung.setText(getMeldungsText(event.getStructuredSelection().getFirstElement(), 0));
			}

			private String getMeldungsText(Object element, int level) {
				StringBuilder result = new StringBuilder();

				for( int idx = 1; idx < level; idx++) {
					result.append('\t');
				}
				
				if (element instanceof MeldungsElement) {
					result.append(getMeldungsText(((MeldungsElement) element).getMeldung(), level + 1));
				} else if (element instanceof MultiStatus) {
					result.append(((MultiStatus) element).getMessage());
					result.append('\n');
					for( IStatus status : ((MultiStatus) element).getChildren()) {
						result.append(getMeldungsText(status, level + 1));
						result.append('\n');
					}
				} else if (element instanceof IStatus) {
					result.append(((IStatus) element).getMessage());
				}
				return result.toString();
			}
		});
		treeViewerMeldungen.setContentProvider(new ProtokollGUIVerwaltung.LogTreeContentProvider());
		treeViewerMeldungen.setLabelProvider(new ProtokollGUIVerwaltung.LogTreeLabelProvider());

		final Tree tree = treeViewerMeldungen.getTree();

		tree.setLinesVisible(true);
		tree.setHeaderVisible(true);
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		final TreeColumn col1 = new TreeColumn(tree, SWT.LEFT);

		col1.setText("Meldung");
		col1.setWidth(500);

		final TreeColumn col2 = new TreeColumn(tree, SWT.LEFT);

		col2.setText("Plug-in");
		col2.setWidth(200);

		final TreeColumn col3 = new TreeColumn(tree, SWT.LEFT);

		col3.setText("Meldungszeit");
		col3.setWidth(120);

		final TreeColumn col4 = new TreeColumn(tree, SWT.LEFT);

		col4.setText("Konfiguration");
		col4.setWidth(120);

		final Composite compText = new Composite(sashComp, SWT.BORDER);

		compText.setLayout(new GridLayout());
		txtMeldung = new Text(compText, SWT.H_SCROLL | SWT.V_SCROLL | SWT.ICON_CANCEL | SWT.MULTI);
		txtMeldung.setFont(ProtokollGUI.FONT_COURIER_9_NORMAL);
		txtMeldung.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		sashComp.setWeights(new int[] { 1, 1 });

		final Composite compSchaltflaeche = new Composite(compAllgemein, SWT.NONE);

		compSchaltflaeche.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, true, false));

		final GridLayout gridLayout = new GridLayout();

		gridLayout.makeColumnsEqualWidth = true;
		gridLayout.numColumns = 2;
		compSchaltflaeche.setLayout(gridLayout);
		btnSpeichern = new Button(compSchaltflaeche, SWT.NONE);
		btnSpeichern.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				guiVew.btnSpeichernSelektiert();
			}
		});
		btnSpeichern.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		btnSpeichern.setText("Speichern");
		btnLoeschen = new Button(compSchaltflaeche, SWT.NONE);
		btnLoeschen.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		btnLoeschen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				guiVew.btnLoeschenSelektiert();
			}
		});
		btnLoeschen.setText("Löschen");
	}
}

// ~Formatiert mit 'inovat Kodierkonvention' am 01.03.10
